-- 202200647 DDL

-- UPDATE T_ACM_EVENT to add Note Type
ALTER TABLE T_ACM_EVENT
    ADD NOTE_TYPE_SID NUMBER;
    
ALTER TABLE T_ACM_EVENT
    ADD CONSTRAINT FK_ACMVNT_NT_TP_SD FOREIGN KEY (NOTE_TYPE_SID)
        REFERENCES T_ACM_NOTE_TYPE (SID);
        
-- Alter T_ACM_NOTE_TYPE to remove NOT NULL constraint on OBJ_TYPE column
ALTER TABLE T_ACM_NOTE_TYPE
    MODIFY OBJ_TYPE NULL;
    
-- Update to ACM_NOTE report to allow notes with no specific Object Type
create or replace PACKAGE BODY ACM_NOTE AS
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>9/11/2018</td><td>KDB</td><td>Copied from other baseline.</td></tr>
</table>
*/
    PL_ROWS_PER_PAGE NUMBER := 25;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>6/12/2018</td><td>KDB</td><td>Moved from ACM_PLUGIN to here.</td></tr>
</table>
*/
    PROCEDURE PRINT_REPORT(
        P_OBJ IN VARCHAR2,
        P_OBJS IN T_STR_ARRAY,
        P_ROW_START IN NUMBER := 1,
        P_ROW_END IN NUMBER := PL_ROWS_PER_PAGE,
        P_REGION IN APEX_PLUGIN.T_REGION,
        P_PLUGIN IN APEX_PLUGIN.T_PLUGIN,
        P_SORT_COLUMN IN VARCHAR2,
        P_SORT_DIRECTION IN VARCHAR2
    )IS
        L_REGION_REC ACM_NOTE.T_REGION_REC;
        L_RSLT_INDX NUMBER := 0;
        L_ROW_MAX NUMBER := PL_ROWS_PER_PAGE;
        L_MORE_ROWS BOOLEAN := FALSE;
        L_SQL_CURSOR INTEGER;
        L_EXEC_THROWAWAY INTEGER;
        L_OBJS DBMS_SQL.VARCHAR2_TABLE;
        L_REGION_SQL CLOB := 'SELECT'
        || ' ANT.DISPLAY NOTE_TYPE,'
        || ' TAO.TAGLINE NOTE_ON,'
        || ' TAO.ID FILE_ACT_ID,'
        || ' TAOT.DISPLAY OBJ_TYPE,'
        || ' AN.CREATED_BY CREATED_BY,'
        || ' DBMS_LOB.SUBSTR(AN.NOTE_TEXT,4000,1) NOTE_TEXT,'
        || ' AN.CREATED_ON CREATED_ON,'
        || ' AN.SID SID,'
        || ' TAO.SID OBJ'
        || ' FROM T_ACM_NOTE AN'
        || ' INNER JOIN T_ACM_NOTE_TYPE ANT ON AN.NOTE_TYPE = ANT.SID AND (:P_REGION_ATTRIBUTE_03 = ANT.USAGE OR :P_REGION_ATTRIBUTE_03 IS NULL)'
        || ' INNER JOIN T_ACM_OBJ TAO ON TAO.SID = AN.OBJ AND TAO.SID IN (SELECT COLUMN_VALUE FROM TABLE(:OBJ_ARRAY) )'
        || ' LEFT JOIN T_ACM_OBJ_TYPE TAOT ON TAOT.SID = TAO.OBJ_TYPE';
        L_REGION_CURSOR SYS_REFCURSOR;

        L_EDIT_AUTH_LIST T_STR_ARRAY := T_STR_ARRAY();
        PROCEDURE PRINT_COLUMN_HEADER (P_LABEL IN VARCHAR2, P_COLUMN_ID IN VARCHAR2)
        IS
        BEGIN
            IF P_COLUMN_ID = P_SORT_COLUMN AND P_SORT_DIRECTION != 'none' THEN
                IF P_SORT_DIRECTION = 'asc' THEN
                    ACM_REGION_UTIL.THC_SORTABLE_ASC(P_LABEL,P_COLUMN_ID,P_REGION.ID,APEX_PLUGIN.GET_AJAX_IDENTIFIER);
                ELSE
                    ACM_REGION_UTIL.THC_SORTABLE_DESC(P_LABEL,P_COLUMN_ID,P_REGION.ID,APEX_PLUGIN.GET_AJAX_IDENTIFIER);
                END IF;
            ELSE
                ACM_REGION_UTIL.THC_SORTABLE(P_LABEL,P_COLUMN_ID,P_REGION.ID,APEX_PLUGIN.GET_AJAX_IDENTIFIER);
            END IF;
        END;
    BEGIN
        ACM_REGION_UTIL.CHECK_ACTION_MANY_OBJS(L_EDIT_AUTH_LIST, 'EDIT_NOTE', P_OBJS);
        L_SQL_CURSOR := DBMS_SQL.OPEN_CURSOR();
        FOR I IN 1..P_OBJS.COUNT
        LOOP
            L_OBJS(I) := P_OBJS(I);
        END LOOP;
        L_REGION_SQL := L_REGION_SQL || ' ORDER BY ' || P_SORT_COLUMN || ' ' || P_SORT_DIRECTION || ' OFFSET ' || TO_CHAR(P_ROW_START  - 1) ||  ' ROWS FETCH NEXT ' || TO_CHAR(P_ROW_END - P_ROW_START + 2) || ' ROWS ONLY';
        DBMS_SQL.PARSE(C=>L_SQL_CURSOR,STATEMENT=>L_REGION_SQL,LANGUAGE_FLAG=>DBMS_SQL.NATIVE);
        DBMS_SQL.BIND_VARIABLE(L_SQL_CURSOR,':OBJ_ARRAY',P_OBJS);
        DBMS_SQL.BIND_VARIABLE(L_SQL_CURSOR,':P_REGION_ATTRIBUTE_03',P_REGION.ATTRIBUTE_03);
        L_EXEC_THROWAWAY := DBMS_SQL.EXECUTE(C=>L_SQL_CURSOR);
        L_REGION_CURSOR := DBMS_SQL.TO_REFCURSOR(cursor_number=>L_SQL_CURSOR);
        HTP.DIV(CATTRIBUTES => 'id="report_' || P_REGION.ID || '_catch"');
        LOOP
            FETCH L_REGION_CURSOR INTO L_REGION_REC;
            EXIT WHEN L_REGION_CURSOR%NOTFOUND;
            L_RSLT_INDX := L_RSLT_INDX + 1;
            IF L_RSLT_INDX = 1 THEN
                ACM_REGION_UTIL.START_TABLE(P_REGION.ID, 'Notes');
                IF P_OBJ MEMBER OF L_EDIT_AUTH_LIST THEN
                    ACM_REGION_UTIL.THC('Edit','SID');
                ELSE
                    ACM_REGION_UTIL.THC('View','SID');
                END IF;
                IF P_REGION.ATTRIBUTE_02 = 'Y' AND T_STR_ARRAY(P_OBJ) != P_OBJS THEN
                    PRINT_COLUMN_HEADER('File/Activity ID', 'FILE_ACT_ID');
                    PRINT_COLUMN_HEADER('Note On', 'NOTE_ON');
                    PRINT_COLUMN_HEADER('File/Activity Type', 'OBJ_TYPE');
                END IF;
                PRINT_COLUMN_HEADER('Note Type', 'NOTE_TYPE');
                PRINT_COLUMN_HEADER('Created By', 'CREATED_BY');
                /* Can't sort on clob */
                --print_column_header('Note Text', 'NOTE_TEXT', p_sort_column, p_sort_direction);
                htp.p('<th><span>Note Text</span></th>');
                PRINT_COLUMN_HEADER('Created On','CREATED_ON');
                HTP.TABLEROWCLOSE;
                HTP.P('</thead>');
                HTP.P('<tbody>');
            END IF;
            IF L_ROW_MAX >= L_RSLT_INDX THEN
                HTP.TABLEROWOPEN();
                ACM_REGION_UTIL.TD('<a href="' || PREPARE_URL('5011','5011','P5011_SID,P5011_USAGE',L_REGION_REC.SID || ',' || P_REGION.ATTRIBUTE_03,'apex.jQuery(''#' || P_REGION.STATIC_ID || ''')')|| '">' ||
                    CASE
                        WHEN L_REGION_REC.OBJ MEMBER OF L_EDIT_AUTH_LIST THEN ACM_REGION_UTIL.EDIT_IMAGE('Edit Note')
                        ELSE ACM_REGION_UTIL.VIEW_IMAGE('View Note')
                    END
                                        || '</a>','SID');
                IF P_REGION.ATTRIBUTE_02 = 'Y' AND T_STR_ARRAY(P_OBJ)!= P_OBJS THEN
                    ACM_REGION_UTIL.TD(L_REGION_REC.FILE_ACT_ID, 'FILE_ACT_ID');
                    ACM_REGION_UTIL.TD(ACM_UI.POPUP(L_REGION_REC.OBJ), 'NOTE_ON');
                    ACM_REGION_UTIL.TD(L_REGION_REC.OBJ_TYPE, 'OBJ_TYPE');
                END IF;
                ACM_REGION_UTIL.TD(L_REGION_REC.NOTE_TYPE, 'NOTE_TYPE');
                ACM_REGION_UTIL.TD(L_REGION_REC.CREATED_BY, 'CREATED_BY');
                ACM_REGION_UTIL.TD(L_REGION_REC.NOTE_TEXT, 'NOTE_TEXT');
                ACM_REGION_UTIL.TD(TO_CHAR(L_REGION_REC.CREATED_ON, FMT_DATE), 'CREATED_ON');
                HTP.TABLEROWCLOSE;
            ELSE
                L_MORE_ROWS := TRUE;
            END IF;
        END LOOP;
        CLOSE L_REGION_CURSOR;

        IF L_RSLT_INDX > 0 THEN
            HTP.P('</tbody>');
            HTP.TABLECLOSE;
            HTP.P('</div>');
            ACM_REGION_UTIL.PRINT_PAGINATION(P_REGION.ID,P_ROW_START,P_ROW_END,L_ROW_MAX,L_RSLT_INDX,L_MORE_ROWS,P_SORT_COLUMN,P_SORT_DIRECTION);
            HTP.P('</div>');
            HTP.P('</div>');
        ELSE
            ACM_REGION_UTIL.NDF('No notes added.');
        END IF;
        HTP.P('</div>');
    EXCEPTION
        WHEN OTHERS THEN
            ACM_LOG.ERROR(SQLERRM);
            RAISE;
    END;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>6/12/2018</td><td>KDB</td><td>Moved from ACM_PLUGIN to here.</td></tr>
</table>
*/
    FUNCTION I2MS_NOTES_REGION_RENDER (
        P_REGION IN APEX_PLUGIN.T_REGION,
        P_PLUGIN IN APEX_PLUGIN.T_PLUGIN,
        P_IS_PRINTER_FRIENDLY IN BOOLEAN
    )RETURN APEX_PLUGIN.T_REGION_RENDER_RESULT IS
        L_RTN APEX_PLUGIN.T_REGION_RENDER_RESULT;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_CREATE_AUTH VARCHAR2(4000);
  --P_REGION.ATTRIBUTE_01 PAGE ITEM OBJ, NECESSARY FOR GETTING VALUES FOR THE REPORT
  --P_REGION.ATTRIBUTE_02 SHOW ASSOCIATION FILTERS
  --P_REGION.ATTRIBUTE_03 USAGE
    BEGIN
        L_OBJ := APEX_UTIL.GET_SESSION_STATE(P_REGION.ATTRIBUTE_01);
        IF P_REGION.ATTRIBUTE_05 != 'ADD_NOTE' THEN
            L_CREATE_AUTH := ACM_BPE.CHECK_ACTION(P_REGION.ATTRIBUTE_05,L_OBJ);
        ELSE
            L_CREATE_AUTH := ACM_BPE.CHECK_ACTION('CREATE_NOTE',L_OBJ);
        END IF;
        IF L_CREATE_AUTH = 'Y' THEN
            ACM_REGION_UTIL.PRINT_BUTTON(PREPARE_URL('5011','5011','P5011_SID,P5011_OBJ',',' || L_OBJ || ',' || P_REGION.ATTRIBUTE_03,'apex.jQuery(''#' || P_REGION.STATIC_ID || ''')'),'B' || P_REGION.ID || '-add-note','add-note','Add Note');
        END IF;
        PRINT_REPORT(L_OBJ,T_STR_ARRAY(L_OBJ),1,PL_ROWS_PER_PAGE,P_REGION,P_PLUGIN, 'SID','desc');
        IF P_REGION.ATTRIBUTE_02 = 'Y' THEN
            ACM_REGION_UTIL.PRINT_FILTER_SET(P_REGION.ID,'Note Filter','This File''s Notes:Notes for Associated Activities:Notes for Associated Files:Inherited Notes','TF:AA:AF:IA','TF');
        END IF;
        ACM_REGION_UTIL.ADD_ONLOAD_JS(P_REGION,P_REGION.ATTRIBUTE_01);
        RETURN L_RTN;
    END I2MS_NOTES_REGION_RENDER;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>6/12/2018</td><td>KDB</td><td>Moved from ACM_PLUGIN to here.</td></tr>
</table>
*/
    FUNCTION I2MS_NOTES_REGION_AJAX (
        P_REGION IN APEX_PLUGIN.T_REGION,
        P_PLUGIN IN APEX_PLUGIN.T_PLUGIN
    )RETURN APEX_PLUGIN.T_REGION_AJAX_RESULT IS
        L_RTN APEX_PLUGIN.T_REGION_AJAX_RESULT;
        L_ROW_START NUMBER := 1;
        L_ROW_END NUMBER := 5;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_OBJS T_STR_ARRAY := T_STR_ARRAY();
    BEGIN
        L_OBJ := APEX_UTIL.GET_SESSION_STATE(P_REGION.ATTRIBUTE_01);
        IF APEX_APPLICATION.G_X03 = 'paginate' THEN
            L_ROW_START := APEX_APPLICATION.G_X04;
        ELSE
            L_ROW_START := 1;
        END IF;
        IF P_REGION.ATTRIBUTE_02 = 'Y' THEN
            L_OBJS := ACM_REGION_UTIL.GET_ASSOC_OBJ_LIST(APEX_APPLICATION.G_X01,L_OBJ);
        ELSE
            L_OBJS := T_STR_ARRAY(L_OBJ);
        END IF;
        L_ROW_END := L_ROW_START + PL_ROWS_PER_PAGE - 1;
        IF APEX_APPLICATION.G_X08 = 'none' THEN
            PRINT_REPORT(L_OBJ,L_OBJS,L_ROW_START,L_ROW_END,P_REGION,P_PLUGIN,'SID','desc');
        ELSE
            PRINT_REPORT(L_OBJ,L_OBJS,L_ROW_START,L_ROW_END,P_REGION,P_PLUGIN,APEX_APPLICATION.G_X07,APEX_APPLICATION.G_X08);
        END IF;
        RETURN L_RTN;
    END I2MS_NOTES_REGION_AJAX;
END ACM_NOTE;
/