-- 202200647 DML

-- Create LIFECYCLE Notes for the following Events:
-- CLOSED, CLOSED_SHORT, RECALLED, REJECTED, REJECTED_CLOSE_SHORT, SUBMITTED
INSERT ALL
    INTO T_ACM_NOTE_TYPE (ACTIVE, USAGE, CODE, DISPLAY) VALUES ('Y', 'LIFECYCLE', 'CLOSED', 'Closed Note')
    INTO T_ACM_NOTE_TYPE (ACTIVE, USAGE, CODE, DISPLAY) VALUES ('Y', 'LIFECYCLE', 'CLOSED_SHORT', 'Closed Short Note')
    INTO T_ACM_NOTE_TYPE (ACTIVE, USAGE, CODE, DISPLAY) VALUES ('Y', 'LIFECYCLE', 'RECALLED', 'Recalled Note')
    INTO T_ACM_NOTE_TYPE (ACTIVE, USAGE, CODE, DISPLAY) VALUES ('Y', 'LIFECYCLE', 'REJECTED', 'Rejected Note')
    INTO T_ACM_NOTE_TYPE (ACTIVE, USAGE, CODE, DISPLAY) VALUES ('Y', 'LIFECYCLE', 'REJECTED_CLOSE_SHORT', 'Rejected Close Short Note')
    INTO T_ACM_NOTE_TYPE (ACTIVE, USAGE, CODE, DISPLAY) VALUES ('Y', 'LIFECYCLE', 'SUBMITTED', 'Submitted Note')
SELECT * FROM DUAL;

-- Remove non-needed LIFECYCLE notes and note types
DELETE FROM T_ACM_NOTE
    WHERE NOTE_TYPE IN
    (
        SELECT SID FROM T_ACM_NOTE_TYPE
        WHERE USAGE = 'LIFECYCLE'
        AND OBJ_TYPE IS NOT NULL
    );
    
DELETE FROM T_ACM_NOTE_TYPE
    WHERE USAGE = 'LIFECYCLE'
    AND OBJ_TYPE IS NOT NULL;
    
-- Add Note Types to appropriate Events
BEGIN

    FOR X IN
    (
        select e.sid, e.code, e.display
        from t_acm_event e
        inner join t_acm_note_type nt
        on nt.code = e.code
        and nt.usage = 'LIFECYCLE'
    )
    LOOP
        UPDATE T_ACM_EVENT
            SET NOTE_TYPE_SID = (SELECT SID FROM T_ACM_NOTE_TYPE WHERE CODE = X.CODE AND USAGE = 'LIFECYCLE')
            WHERE SID = X.SID;
    END LOOP;

END;
/

-- Update Action Page for appropriate Rules (active File objects)
BEGIN

    FOR X IN
    (
        SELECT SID, CODE, DISPLAY, RULE_TYPE, ACTION_PAGE
        FROM T_ACM_RULE
        WHERE EVENT IN (SELECT SID FROM T_ACM_EVENT WHERE NOTE_TYPE_SID IS NOT NULL AND CODE IN ('RECALLED', 'REJECTED', 'CLOSED_SHORT'))
        AND ACTION_PAGE IS NOT NULL
        AND (CODE LIKE 'FILE%')
        AND OBJ_TYPE IN (SELECT SID FROM T_ACM_OBJ_TYPE WHERE ACTIVE = 'Y')
    )
    LOOP
        UPDATE T_ACM_RULE
            SET ACTION_PAGE = 25
            WHERE SID = X.SID;
    END LOOP;

END;
/