-- 202200875 DML
UPDATE T_ACM_OBJ_TYPE
    SET DISPLAY = 'Insider Threat Investigation',
        ACTIVE = 'Y'
    WHERE OBJ_TYPE_CODE = 'FILE.INV_1';
    
-- Object Metadata
DECLARE
    V_OBJ_TYPE      T_ACM_OBJ_TYPE.SID%TYPE;
    V_DATA_GROUP    T_ACM_OBJ_TYPE.DATA_GROUP_ID%TYPE;
BEGIN
    SELECT SID
    INTO V_OBJ_TYPE
    FROM T_ACM_OBJ_TYPE
    WHERE OBJ_TYPE_CODE = 'FILE.INV_1';
    
    SELECT DATA_GROUP_ID
    INTO V_DATA_GROUP
    FROM T_ACM_OBJ_TYPE
    WHERE OBJ_TYPE_CODE = 'FILE.INV_1';
    
    DELETE FROM T_ACM_STAKEHOLDER_ROLE
    WHERE OBJ_TYPE = V_OBJ_TYPE;
    
    FOR X IN
    (
        SELECT *
        FROM T_ACM_STAKEHOLDER_ROLE
        WHERE OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.PRE_ASSESS')
    )
    LOOP
        
        INSERT INTO T_ACM_STAKEHOLDER_ROLE
            (OBJ_TYPE, SH_OBJ_TYPE, CODE, DISPLAY, SEQ, MIN_NUM, MAX_NUM, DATA_GROUP_ID)
        VALUES
            (
                V_OBJ_TYPE,
                X.SH_OBJ_TYPE,
                X.CODE,
                X.DISPLAY,
                X.SEQ,
                X.MIN_NUM,
                X.MAX_NUM,
                V_DATA_GROUP
            );
        
    END LOOP;
    
    DELETE FROM T_PARTIC_ROLE_TYPE
    WHERE OBJ_TYPE = V_OBJ_TYPE;
    
    FOR Y IN
    (
        SELECT *
        FROM T_PARTIC_ROLE_TYPE
        WHERE OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.PRE_ASSESS')
    )
    LOOP
        
        INSERT INTO T_PARTIC_ROLE_TYPE
            (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE, ROLE_MAX, INDIV_ONLY, NONINDIV_ONLY, DATA_GROUP_ID)
        VALUES
            (
                V_OBJ_TYPE,
                Y.USAGE,
                Y.CODE,
                Y.DISPLAY,
                Y.SEQ,
                Y.ACTIVE,
                Y.ROLE_MAX,
                Y.INDIV_ONLY,
                Y.NONINDIV_ONLY,
                V_DATA_GROUP
            );
        
    END LOOP;
    
    DELETE FROM T_ACM_NOTE_TYPE
    WHERE OBJ_TYPE = V_OBJ_TYPE;
    
    FOR N IN
    (
        SELECT *
        FROM T_ACM_NOTE_TYPE
        WHERE OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.PRE_ASSESS')
    )
    LOOP
    
        INSERT INTO T_ACM_NOTE_TYPE
            (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, MIN_NUM, MAX_NUM, LOCK_MODE, ACTIVE, DATA_GROUP_ID)
        VALUES
            (
                V_OBJ_TYPE,
                N.USAGE,
                N.CODE,
                N.DISPLAY,
                N.SEQ,
                N.MIN_NUM,
                N.MAX_NUM,
                N.LOCK_MODE,
                N.ACTIVE,
                V_DATA_GROUP
            );
            
    END LOOP;
    
    DELETE FROM T_ACM_ATTACHMENT_TYPE
    WHERE OBJ_TYPE = V_OBJ_TYPE;
    
    FOR A IN
    (
        SELECT *
        FROM T_ACM_ATTACHMENT_TYPE
        WHERE OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.PRE_ASSESS')
    )
    LOOP
        
        INSERT INTO T_ACM_ATTACHMENT_TYPE
            (OBJ_TYPE, USAGE, CODE, DISPLAY, MIME_DISPOSITION, MIN_NUM, MAX_NUM, LOCK_MODE, SEQ, ACTIVE, DATA_GROUP_ID)
        VALUES
            (
                V_OBJ_TYPE,
                A.USAGE,
                A.CODE,
                A.DISPLAY,
                A.MIME_DISPOSITION,
                A.MIN_NUM,
                A.MAX_NUM,
                A.LOCK_MODE,
                A.SEQ,
                A.ACTIVE,
                V_DATA_GROUP
            );
        
    END LOOP;
    
    DELETE FROM T_ACM_ASSOC_TYPE
    WHERE FROM_TYPE = V_OBJ_TYPE
        OR TO_TYPE = V_OBJ_TYPE;
    
    FOR X IN
    (
        SELECT *
        FROM T_ACM_ASSOC_TYPE
        WHERE FROM_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.PRE_ASSESS')
        OR TO_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.PRE_ASSESS')
    )
    LOOP
        
        IF ACM_OBJ.GET_OBJTYPE_CODE(X.FROM_TYPE) = 'FILE.PRE_ASSESS' THEN
            
            INSERT INTO T_ACM_ASSOC_TYPE
                (FROM_TYPE, TO_TYPE, CODE, DISPLAY, DATA_GROUP_ID)
            VALUES
                (
                    V_OBJ_TYPE,
                    X.TO_TYPE,
                    acm_obj.get_objtype_code(V_OBJ_TYPE) || '_TO_' || acm_obj.get_objtype_code(X.TO_TYPE),
                    acm_obj.lookup_objtype_disp(V_OBJ_TYPE) || ' to ' || acm_obj.lookup_objtype_disp(X.TO_TYPE),
                    V_DATA_GROUP
                );
                
        ELSIF ACM_OBJ.GET_OBJTYPE_CODE(X.TO_TYPE) = 'FILE.PRE_ASSESS' THEN
            
            INSERT INTO T_ACM_ASSOC_TYPE
                (FROM_TYPE, TO_TYPE, CODE, DISPLAY, DATA_GROUP_ID)
            VALUES
                (
                    X.FROM_TYPE,
                    V_OBJ_TYPE,
                    acm_obj.get_objtype_code(X.FROM_TYPE) || '_TO_' || acm_obj.get_objtype_code(V_OBJ_TYPE),
                    acm_obj.lookup_objtype_disp(X.FROM_TYPE) || ' to ' || acm_obj.lookup_objtype_disp(V_OBJ_TYPE),
                    V_DATA_GROUP
                );
                
        END IF;
        
    END LOOP;

END;
/

set scan off
declare
  V_OBJ_TYPE_CODE                   VARCHAR2(300);
  V_PRIV_GROUP_SID                        T_ACM_PRIV_GROUP.SID%TYPE;
  V_PGROUP_PRIV_SID                       T_ACM_PGROUP_PRIV.SID%TYPE;
begin
  V_PRIV_GROUP_SID := ACM_MDE3.IMPORT_PRIV_GROUP( P_CODE => 'IRMO_LVL_1', P_DISPLAY => 'IRMO Level 1', P_COMMENTS => null, 
 P_DATA_GROUP => 1);
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.PLAN_STEP_ADD', P_PGROUP_CODE => 'IRMO_LVL_1');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.DOWNLOAD_ATTACH', P_PGROUP_CODE => 'IRMO_LVL_1');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.COMPLETE', P_PGROUP_CODE => 'IRMO_LVL_1');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.CREATE', P_PGROUP_CODE => 'IRMO_LVL_1');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.SAVE', P_PGROUP_CODE => 'IRMO_LVL_1');
exception
  when OTHERS then
    raise;
end;
/

set scan off
declare
  V_OBJ_TYPE_CODE                   VARCHAR2(300);
  V_PRIV_GROUP_SID                        T_ACM_PRIV_GROUP.SID%TYPE;
  V_PGROUP_PRIV_SID                       T_ACM_PGROUP_PRIV.SID%TYPE;
begin
  V_PRIV_GROUP_SID := ACM_MDE3.IMPORT_PRIV_GROUP( P_CODE => 'IRMO_LVL_2', P_DISPLAY => 'IRMO Level 2', P_COMMENTS => null, 
 P_DATA_GROUP => 1);
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.PLAN_STEP_DELETE', P_PGROUP_CODE => 'IRMO_LVL_2');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.CLOSE', P_PGROUP_CODE => 'IRMO_LVL_2');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.ADD_RESTRICTION', P_PGROUP_CODE => 'IRMO_LVL_2');
V_PGROUP_PRIV_SID := ACM_MDE3.ADD_PRIV_TO_PGROUP( P_PRIV_CODE => 'FILE.INV_1.EDIT_SUPPORTING_UNIT', P_PGROUP_CODE => 'IRMO_LVL_2');
exception
  when OTHERS then
    raise;
end;
/

-- T_F_IT_DISPOSITION_REF metadata
BEGIN 
    FOR X IN
        (
            SELECT *
            FROM T_F_DISPOSITION_REF
        )
    LOOP	
        INSERT INTO T_F_IT_DISPOSITION_REF 
            ( CODE, DISPLAY, SEQ )
        VALUES
            (
                X.CODE,
                X.DISPLAY,
                X.SEQ
            );
    END LOOP;
END;
/

-- New Disposition Conditions (Insider Threat)
set scan off
declare
  V_CONDITION_SID                        T_ACM_CONDITION.SID%TYPE;
begin
  V_CONDITION_SID := ACM_MDE3.Import_CONDITION(P_CODE => 'HAS_IT_MATTER', P_DISPLAY => 'File must have a Primary IT Matter', P_EXEC_FUNCTION => 'ACM_P_IT_MATTER.COND_FILE_IT_MATTER', P_DEV_COMMENT => null, P_ACTIVE => 'Y', P_ACCEPTED_PARAMS => null);
  V_CONDITION_SID := ACM_MDE3.Import_CONDITION(P_CODE => 'IT_HAS_DISP_RESOLVE', P_DISPLAY => 'IT File must have Resolution Response.', P_EXEC_FUNCTION => 'ACM_P_IT_DISPOSITION.COND_DISP_RESOLVE', P_DEV_COMMENT => null, P_ACTIVE => 'Y', P_ACCEPTED_PARAMS => null);
  V_CONDITION_SID := ACM_MDE3.Import_CONDITION(P_CODE => 'IT_HAS_DISPOSTION', P_DISPLAY => 'IT Disposition required', P_EXEC_FUNCTION => 'ACM_P_IT_DISPOSITION.COND_AT_LEAST_ONE_DISP', P_DEV_COMMENT => null, P_ACTIVE => 'Y', P_ACCEPTED_PARAMS => null);
  V_CONDITION_SID := ACM_MDE3.Import_CONDITION(P_CODE => 'IT_HAS_DISP_EXPLANATION', P_DISPLAY => 'Explanation of IT Disposition Required', P_EXEC_FUNCTION => 'ACM_P_IT_DISPOSITION.COND_DISP_EXPLANATION', P_DEV_COMMENT => null, P_ACTIVE => 'Y', P_ACCEPTED_PARAMS => null);
exception
  when OTHERS then
    raise;
end;
/

-- Update appropriate Rule Conditions
DECLARE
    V_COND T_ACM_CONDITION.SID%TYPE := NULL;
BEGIN

    FOR X IN
    (
        SELECT ARC.SID, ARC.ACM_RULE, ARC.CONDITION, C.EXEC_FUNCTION
        FROM T_ACM_RULE_CONDITION ARC
        INNER JOIN T_ACM_CONDITION C ON ARC.CONDITION = C.SID
        WHERE ARC.ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'))
        AND C.EXEC_FUNCTION LIKE 'ACM_P_DISPOSITION%'
    )
    LOOP
        
        SELECT SID
        INTO V_COND
        FROM T_ACM_CONDITION
        WHERE EXEC_FUNCTION = REPLACE(X.EXEC_FUNCTION, 'ACM_P_DISPOSITION', 'ACM_P_IT_DISPOSITION');
        
        UPDATE T_ACM_RULE_CONDITION
            SET CONDITION = V_COND
            WHERE SID = X.SID;
            
    END LOOP;

END;
/

DECLARE
    V_COND T_ACM_CONDITION.SID%TYPE := NULL;
BEGIN
    
    SELECT SID
    INTO V_COND
    FROM T_ACM_CONDITION
    WHERE CODE = 'HAS_IT_MATTER';
    
    FOR X IN
    (
        SELECT ARC.SID, ARC.ACM_RULE, ARC.CONDITION, C.CODE, C.EXEC_FUNCTION
        FROM T_ACM_RULE_CONDITION ARC
        INNER JOIN T_ACM_CONDITION C ON ARC.CONDITION = C.SID
        WHERE ARC.ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'))
        AND C.CODE = 'HAS_ALLEGATION'
    )
    LOOP
        
        UPDATE T_ACM_RULE_CONDITION
            SET CONDITION = V_COND
            WHERE SID = X.SID;
            
    END LOOP;

END;
/

-- IT Matter Reference Data (for INV_1 file)
INSERT ALL
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'MATTER_TYPE',
                'CYBER',
                'Cyber',
                1,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'MATTER_TYPE',
                'BEHAV',
                'Behaviorial',
                2,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'MATTER_TYPE',
                'FINANC',
                'Financial',
                3,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'MATTER_TYPE',
                'AIS',
                'AIS Incident',
                4,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'CYBER_SUBTYPE',
                'DOS',
                'Denial of Service',
                5,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'CYBER_SUBTYPE',
                'PHISH',
                'Phishing',
                6,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'BEHAV_SUBTYPE',
                'ERR_BEH',
                'Erratic Behavior',
                7,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'FINANC_SUBTYPE',
                'UFI',
                'Unusual Financial Influence',
                8,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'FINANC_SUBTYPE',
                'CREDIT',
                'Credit',
                9,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'AIS_SUBTYPE',
                'UNAUTH_DEV_IN_SCIF',
                'Unauthorized Device/Media in SCIF',
                10,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'AIS_SUBTYPE',
                'UNAUTH_DEV_CNCTD',
                'Unauthorized Device Connected to Network',
                11,
                'Y'
                )
    INTO T_F_INV_1_REF (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'),
                'AIS_SUBTYPE',
                'MISHANDLE_CLASS_INFO',
                'Mishandled Classified Information',
                12,
                'Y'
                )
SELECT * FROM DUAL;