-- 202200875 DML Fix
DELETE FROM T_ACM_RULE_CONDITION
WHERE CONDITION = (SELECT SID FROM T_ACM_CONDITION WHERE CODE = 'ASSOCIATED_TO_ICIR_FILE');

DELETE FROM T_ACM_RULE_CONDITION
WHERE CONDITION = (SELECT SID FROM T_ACM_CONDITION WHERE CODE = 'HAS_CATEGORY')
AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE CODE LIKE 'FILE.INV_1%');

DELETE FROM T_ACM_RULE_CONDITION
WHERE CONDITION = (SELECT SID FROM T_ACM_CONDITION WHERE CODE = 'HAS_PRIMARY_COUNTRY')
AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE CODE LIKE 'FILE.INV_1%');

DELETE FROM T_ACM_RULE_CONDITION
WHERE CONDITION = (SELECT SID FROM T_ACM_CONDITION WHERE CODE = 'SUBJECT_HAS_ETHNICITY')
AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE CODE LIKE 'FILE.INV_1%');

-- Remove Extraneous Attachment Types
DELETE FROM T_ACM_ATTACHMENT_TYPE
    WHERE CODE <> 'GEN_ATCH'
    AND OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1');
    
-- Participant Roles Update
UPDATE T_PARTIC_ROLE_TYPE
    SET CODE = 'OTHER',
    DISPLAY = 'Other'
    WHERE CODE = 'INCIDENTAL'
    AND OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1');
    
INSERT ALL
    INTO T_PARTIC_ROLE_TYPE (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'), 'PARTICIPANTS', 'COR', 'COR', 1, 'Y')
    INTO T_PARTIC_ROLE_TYPE (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'), 'PARTICIPANTS', 'SPRVSR', 'Supervisor', 1, 'Y')
    INTO T_PARTIC_ROLE_TYPE (OBJ_TYPE, USAGE, CODE, DISPLAY, SEQ, ACTIVE)
        VALUES ((SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'FILE.INV_1'), 'PARTICIPANTS', 'WITNESS', 'Witness', 1, 'Y')
SELECT * FROM DUAL;

-- Rules Update
DECLARE
    V_OT_SID    T_ACM_OBJ_TYPE.SID%TYPE;
BEGIN
    
    SELECT SID
    INTO V_OT_SID
    FROM T_ACM_OBJ_TYPE
    WHERE OBJ_TYPE_CODE = 'FILE.INV_1';
    
    UPDATE T_ACM_RULE
    SET DISPLAY = REPLACE(DISPLAY, 'HQ ', '')
    WHERE OBJ_TYPE = V_OT_SID;
    
    UPDATE T_ACM_RULE
    SET DISPLAY = REPLACE(DISPLAY, 'Regional ', '')
    WHERE OBJ_TYPE = V_OT_SID;
    
    UPDATE T_ACM_RULE
    SET DISPLAY = REPLACE(DISPLAY, 'Region ', '')
    WHERE OBJ_TYPE = V_OT_SID;
    
    -- NEW status
    UPDATE T_ACM_RULE_PROCESS
        SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'APPROVAL')
        WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_REGNL_APROVE')
        AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE OBJ_TYPE = V_OT_SID
                          AND STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'NEW'));
    
    -- AWAIT_HQ_APROVE status
    UPDATE T_ACM_RULE_PROCESS
        SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'NEW')
        WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_REGNL_APROVE')
        AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE OBJ_TYPE = V_OT_SID
                          AND STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_HQ_APROVE'));
                          
    UPDATE T_ACM_RULE
        SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'APPROVAL')
        WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_HQ_APROVE')
        AND OBJ_TYPE = V_OT_SID;
        
    -- OPEN status
    UPDATE T_ACM_RULE_PROCESS
        SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'COMPLETE')
        WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_REGION_COMP')
        AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE OBJ_TYPE = V_OT_SID
                          AND STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'OPEN'));
                          
    -- AWAIT_HQ_COMP status
    UPDATE T_ACM_RULE_PROCESS
        SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'OPEN')
        WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_REGION_COMP')
        AND ACM_RULE IN (SELECT SID FROM T_ACM_RULE WHERE OBJ_TYPE = V_OT_SID
                          AND STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_HQ_COMP'));
    
    UPDATE T_ACM_RULE
        SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'COMPLETE')
        WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_HQ_COMP')
        AND OBJ_TYPE = V_OT_SID;
        
    -- Update Status History Table    
    FOR OBJ IN
    (
        SELECT SID
        FROM T_ACM_OBJ
        WHERE OBJ_TYPE = V_OT_SID
    )
    LOOP
        
        FOR SH IN
        (
            SELECT OBJ, STATUS
            FROM T_ACM_STATUS_HISTORY
            WHERE OBJ = OBJ.SID
        )
        LOOP
            
            UPDATE T_ACM_STATUS_HISTORY
                SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'APPROVAL')
                WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_HQ_APROVE')
                AND OBJ = SH.OBJ;
            UPDATE T_ACM_STATUS_HISTORY
                SET STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'COMPLETE')
                WHERE STATUS = (SELECT SID FROM T_ACM_STATUS WHERE CODE = 'AWAIT_HQ_COMP')
                AND OBJ = SH.OBJ;
                
        END LOOP;
        
    END LOOP;
    
END;
/