
--Body

create or replace PACKAGE BODY       "ACM_BPE_COND" AS
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_STATUS_IS_NOT_CLOSED RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(SH.SID)
        INTO L_COUNT
        FROM T_ACM_STATUS_HISTORY SH
        INNER JOIN T_ACM_STATUS S ON SH.STATUS = S.SID 
        WHERE STARTED_ON <= SYSTIMESTAMP AND 
              COMPLETED_ON IS NULL AND
              SH.OBJ = ACM_BPE.GET_PARAM('OBJ') AND 
              S.CODE = 'CLOSED';
        IF L_COUNT = 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'This action cannot be completed on a closed object.';
        END IF;
        RETURN L_RTN;
    END COND_STATUS_IS_NOT_CLOSED;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUBJECT RETURN VARCHAR2 IS
        L_RTN       VARCHAR2(255) := 'Y';
        L_COUNT     NUMBER;
        L_OBJ       T_ACM_OBJ.SID%TYPE;
        L_ROLE      T_PARTIC_ROLE_TYPE.DISPLAY%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');

        SELECT
            DISPLAY
        INTO
            L_ROLE
        FROM
            T_PARTIC_ROLE_TYPE
        WHERE
            CODE = 'SUBJECT'
            AND ACTIVE = 'Y'
            AND OBJ_TYPE = (SELECT OBJ_TYPE FROM T_ACM_OBJ WHERE SID = L_OBJ);

        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE
            OPRT.CODE IN ('SUBJECT')
            AND OPRT.ACTIVE = 'Y'
            AND OPI.OBJ = L_OBJ
            AND ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Must have a participant with involvement role of "' || L_ROLE || '"';
        RETURN L_RTN;
    END COND_HAS_SUBJECT;
/** Summary
<table>
<tr><td>2/20/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_HAS_COMPONENT RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ_ROOT T_ACM_OBJ_TYPE.OBJ_ROOT%TYPE;
        L_OBJ_SPPRT_COUNT NUMBER := 0;
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT OT.OBJ_ROOT INTO L_OBJ_ROOT
          FROM T_ACM_OBJ_TYPE OT
          INNER JOIN T_ACM_OBJ O ON OT.SID = O.OBJ_TYPE
         WHERE O.SID = L_OBJ;
         /*
        SELECT COUNT(*) INTO L_OBJ_SPPRT_COUNT
          FROM T_ACM_OBJ_DHS_SUPPORT_TYPE
         WHERE OBJ = L_OBJ;
        IF L_OBJ_ROOT = 'ACT' AND L_OBJ_SPPRT_COUNT = 0 THEN
            RETURN 'Y';
        END IF;
        */
        SELECT COUNT(OPI.SID) 
          INTO L_COUNT
          FROM T_PARTIC_INVOLVEMENT OPI 
          INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
         WHERE OPRT.CODE IN ('TREASURY_BUREAU') AND 
               OPI.OBJ = L_OBJ;

         IF L_COUNT > 0 THEN
           RETURN L_RTN;
         ELSE
           L_RTN := 'Must have a related Treasury Bureau';
        END IF;
      RETURN L_RTN;
    END COND_HAS_COMPONENT;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUP_COMPONENT RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('DHS_AGENCY') AND 
              OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Supporting Component before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_SUP_COMPONENT;
/** Summary
<table>
<tr><td>4/21/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_OTHER_AGENCY RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.USAGE = 'OTHER_AGENCIES' AND 
              OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have at least one Other Agency';
        RETURN L_RTN;
    END COND_HAS_OTHER_AGENCY;    
/** Summary
<table>
<tr><td>2/20/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_HAS_BRIEFEE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('BRIEFED') AND 
              OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Must have a participant with involvement role of "Briefee"';
        RETURN L_RTN;
    END COND_HAS_BRIEFEE;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>12/29/2020</td><td>LLSB</td><td>Updated for Task 13282</td></tr>
</table>
*/
FUNCTION COND_HAS_TRAVELER RETURN VARCHAR2 IS
    L_RTN VARCHAR2(255) := 'Y';
	  L_COUNT NUMBER;
    L_OBJ T_ACM_OBJ.SID%TYPE;
    L_TRAVEL_TYPE T_F_TRAVEL.TYPE%TYPE;
  BEGIN
    L_OBJ := ACM_BPE.GET_PARAM('OBJ');
    SELECT COUNT(OPI.SID)
      INTO L_COUNT
      FROM T_PARTIC_INVOLVEMENT OPI
     INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
     WHERE OPRT.CODE IN ('TRAVELER') AND OPI.OBJ = L_OBJ;
    IF L_COUNT > 0 THEN
      FOR X IN (SELECT TYPE FROM T_F_TRAVEL WHERE OBJ = L_OBJ
      )LOOP
        L_TRAVEL_TYPE := X.TYPE;
      END LOOP;
	  IF L_COUNT > 1 AND L_TRAVEL_TYPE = 'TRAVEL_TYPE.PERSONAL' THEN
	    L_RTN := 'Must have only one Traveler for personal travel.';
      END IF;
    ELSE
      L_RTN := 'Must have at least one Traveler.';
    END IF;
	RETURN L_RTN;
  END COND_HAS_TRAVELER;

/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/    
    FUNCTION COND_HAS_COUNTRY RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TAOV.SID)
        INTO L_COUNT
        FROM T_ACM_OBJ_VENUE TAOV
        WHERE TAOV.OBJ = ACM_BPE.GET_PARAM('OBJ');
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'Object must have at least one Country';
        END IF;
        RETURN L_RTN;
    END COND_HAS_COUNTRY;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/    
    FUNCTION COND_HAS_ASSOCIATED_FILE RETURN VARCHAR2 IS
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER := 0;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = L_OBJ AND TO_OT.OBJ_ROOT = 'FILE') OR 
                  (TAA.TO_OBJ = L_OBJ AND FROM_OT.OBJ_ROOT = 'FILE')) AND ROWNUM = 1;
        IF L_COUNT = 0 THEN
            L_RTN := 'Object must be associated to a File.';
        END IF;
        RETURN L_RTN;
    END COND_HAS_ASSOCIATED_FILE;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/    
    FUNCTION COND_ASSOCIATED_TO_TRAVEL_FILE RETURN VARCHAR2 IS
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER := 0;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = L_OBJ AND TO_OT.OBJ_TYPE_CODE LIKE 'FILE.TRAVEL%') OR 
                  (TAA.TO_OBJ = L_OBJ AND FROM_OT.OBJ_TYPE_CODE LIKE 'FILE.TRAVEL%')) AND ROWNUM = 1;
        IF L_COUNT = 0 THEN
            L_RTN := 'Object must be associated to a Travel File.';
        END IF;
        RETURN L_RTN;
    END COND_ASSOCIATED_TO_TRAVEL_FILE;
/** Summary
<table>
<tr><td>06/30/2020</td><td>LLSB</td><td>Initial Creation for Task 12745</td></tr>
</table>
*/    
    FUNCTION COND_ASSOCIATED_TO_ICIR_FILE RETURN VARCHAR2 IS
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER := 0;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = L_OBJ AND TO_OT.OBJ_TYPE_CODE IN ('FILE.ICIR', 'FILE.ICIR_2', 'FILE.ICIR_3', 'FILE.ICIR_4',
            'FILE.ICIR_5', 'FILE.ICIR_6', 'FILE.ICIR_7', 'FILE.ICIR_8')) OR 
                  (TAA.TO_OBJ = L_OBJ AND FROM_OT.OBJ_TYPE_CODE IN ('FILE.ICIR', 'FILE.ICIR_2', 'FILE.ICIR_3', 'FILE.ICIR_4',
            'FILE.ICIR_5', 'FILE.ICIR_6', 'FILE.ICIR_7', 'FILE.ICIR_8', 'FILE.ICIR_9', 'FILE.ICIR_10'))) AND ROWNUM = 1;
        IF L_COUNT = 0 THEN
            L_RTN := 'Object must be associated to an ICIR File.';
        END IF;
        RETURN L_RTN;
    END COND_ASSOCIATED_TO_ICIR_FILE;
/** Summary
<table>
<tr><td>1/23/2019</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_IRR_VALUE RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TAOC.SID)
        INTO L_COUNT
        FROM T_ACM_OBJ_CI_METRICS TAOC
        WHERE TAOC.OBJ = ACM_BPE.GET_PARAM('OBJ') AND TAOC.IIR_COUNT IS NOT NULL;
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'IRR Count Metric must have a value';
        END IF;
        RETURN L_RTN;
    END COND_HAS_IRR_VALUE;
/** Summary
<table>
<tr><td>1/23/2019</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_IRR_WITH_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'IIR' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAOC.SID)
            INTO L_COUNT
            FROM T_F_ICIR TAOC
            WHERE TAOC.OBJ = ACM_BPE.GET_PARAM('OBJ') AND TAOC.IIR_NUMBER IS NOT NULL;
            IF L_COUNT > 0 THEN
                L_RTN := 'Y';
            ELSE
                L_RTN := 'Number of IIRs must have a value';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_HAS_IRR_WITH_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_FI_WITH_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_1') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_1')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a Full Investigation (FI) File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_HAS_FI_WITH_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_PI_WITH_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_3') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_3')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a Preliminary Investigation (PI) File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_HAS_PI_WITH_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_CI_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INVEST') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INVEST')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a CI Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_HAS_CI_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_CI_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.PRELIM') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.PRELIM')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a CI Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_HAS_CI_INQ_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CIS_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_2') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_2')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a CIS Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_CIS_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CIS_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_4') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_4')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a CIS Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_CIS_INQ_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/       
FUNCTION COND_TSA_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_6') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_6')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a TSA Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_TSA_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_TSA_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_5') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_5')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a TSA Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_TSA_INQ_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/       
FUNCTION COND_ICE_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_8') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_8')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to an ICE Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_ICE_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_ICE_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_7') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_7')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to an ICE Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_ICE_INQ_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/       
FUNCTION COND_FPS_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_9') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_9')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a FPS Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_FPS_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_FPS_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_10') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_10')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a FPS Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_FPS_INQ_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/       
FUNCTION COND_CISA_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_12') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_12')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a CISA Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_CISA_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CISA_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_11') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_11')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a CISA Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_CISA_INQ_W_HQ_FINAL_REC;
 /** Summary
<table>
<tr><td>4/13/2021</td><td>LAM</td><td>Created</td></tr>
</table>
*/       
FUNCTION COND_FEMA_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_13') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_13')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a FEMA Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_FEMA_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_FEMA_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_14') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_14')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a FEMA Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_FEMA_INQ_W_HQ_FINAL_REC;  
 /** Summary
<table>
<tr><td>4/13/2021</td><td>LAM</td><td>Created</td></tr>
</table>
*/       
FUNCTION COND_USSS_INV_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_INVEST' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_15') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_15')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a USSS Investigation File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_USSS_INV_W_HQ_FINAL_REC;
/** Summary
<table>
<tr><td>4/13/2021</td><td>LM</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_USSS_INQ_W_HQ_FINAL_REC RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'OPEN_PI' AND TFFRR.USAGE = 'RECOMMEND')
        AND ((TFFR.CODE = 'HQ' OR TFFR.CODE = 'FINAL') AND TFFR.USAGE = 'RECOMMEND_TYPE');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ASSOCIATION TAA 
            INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE 
            INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
            INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
            WHERE ((TAA.FROM_OBJ = ACM_BPE.GET_PARAM('OBJ') AND TO_OT.OBJ_TYPE_CODE = 'FILE.INV_16') OR 
                  (TAA.TO_OBJ = ACM_BPE.GET_PARAM('OBJ') AND FROM_OT.OBJ_TYPE_CODE = 'FILE.INV_16')) AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'Object must be associated to a USSS Inquiry File.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_USSS_INQ_W_HQ_FINAL_REC;    
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_RECEIPT_METHOD RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(sid)
        INTO L_COUNT
        FROM T_F_ICIR 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND RECEIPT_METHOD IS NOT NULL;
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'Method of Receipt must have a value';
        END IF;
        RETURN L_RTN;
    END COND_HAS_RECEIPT_METHOD;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_PARTICIPANT RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Participant before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_PARTICIPANT;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_NUMBER_BRIEFED RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPI.OBJ = L_OBJ AND
              OPI.NUM_BRIEFED IS NOT NULL AND
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a value for Number Briefed.';
        RETURN L_RTN;
    END COND_HAS_NUMBER_BRIEFED;
/** Summary
<table>
<tr><td>9/15/2020</td><td>JAF</td><td>Created.</td></tr>
<tr><td>12/9/2020</td><td>JAF</td><td>Updated to capture all FILE.ICIR(s)</td></tr>
</table>
*/
    FUNCTION NO_SUBJECT_BYPASS (P_OBJ IN VARCHAR2) RETURN VARCHAR2 AS
    L_RTN VARCHAR2(255) := 'N';
    BEGIN
         FOR A IN (SELECT 'X'
                            FROM T_ACM_OBJ O
                            JOIN T_ACM_OBJ_TYPE OT ON (OT.SID = O.OBJ_TYPE)
                            WHERE O.SID = P_OBJ
                            AND OT.OBJ_TYPE_CODE LIKE ('FILE.ICIR%'))
         LOOP
               RETURN 'Y';
         END LOOP;
    RETURN L_RTN;
    END;
/** Summary
<table>
<tr><td>1/29/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_ETHNICITY RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_ALL_PARTICS T_STR_ARRAY := T_STR_ARRAY();
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_CHARS_FOUND VARCHAR2(1) := 'N';
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_ALL_PARTICS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                L_RTN := 'Subject requires ethnicity.';
                RETURN L_RTN;
        END;
        FOR SUBJECT IN 1..L_ALL_PARTICS.COUNT
        LOOP
            L_CHARS_FOUND := 'N';
            FOR INDIV_CHAR IN ( SELECT
                                    C.SID,
                                    C.EFFECTIVE_DATE,
                                    C.ETHNICITY,
                                    O.TAGLINE
                                FROM
                                    T_PARTIC_INDIV_CHAR C
                                    INNER JOIN T_PARTIC_INDIV I ON I.SID = C.INDIV_PARTIC AND I.OBJ = L_ALL_PARTICS(SUBJECT) 
                                    INNER JOIN T_ACM_OBJ O ON O.SID = I.OBJ
                                WHERE
                                    EFFECTIVE_DATE = (SELECT MAX(EFFECTIVE_DATE) FROM T_PARTIC_INDIV_CHAR WHERE INDIV_PARTIC = I.SID)
                                ORDER BY
                                    C.EFFECTIVE_DATE DESC NULLS LAST,
                                    C.SID DESC
            ) LOOP
                L_CHARS_FOUND := 'Y';
                IF INDIV_CHAR.ETHNICITY IS NULL THEN
                    L_FAILED_SUBJECTS.EXTEND;
                    L_FAILED_SUBJECTS(L_FAILED_SUBJECTS.COUNT) := INDIV_CHAR.TAGLINE;
                END IF;
            END LOOP;
        END LOOP;
        IF L_ALL_PARTICS.COUNT > 0 AND L_CHARS_FOUND = 'N' THEN
            L_RTN := 'All Subjects require physical characteristics including Ethnicity.';
            RETURN L_RTN;
        END IF;
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Subject';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_FAILED_SUBJECTS(I) || ', ';
                ELSE
                    L_RTN := L_RTN || L_FAILED_SUBJECTS(I) || ' ';
                END IF;
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || ' ethnicity on most recent effective date physical characteristics.';
        END IF;
        RETURN L_RTN;
    END COND_SUBJECT_HAS_ETHNICITY;
/** Summary
<table>
<tr><td>1/29/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_NATIONALITY RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_ALL_PARTICS T_STR_ARRAY := T_STR_ARRAY();
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_CTRY_FOUND VARCHAR2(1) := 'N';       
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_ALL_PARTICS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                L_RTN := 'Subject requires nationality.';
                RETURN L_RTN;         
        END;
        FOR SUBJECT IN 1..L_ALL_PARTICS.COUNT
        LOOP
            L_CTRY_FOUND := 'N';
            FOR INDIV_C IN (
            SELECT C.SID, MAX(EFFECTIVE_DATE) OVER (PARTITION BY INDIV_PARTIC) EFFECTIVE_DATE, C.COUNTRY, O.TAGLINE
                FROM T_PARTIC_INDIV_CITIZENSHIP C
                INNER JOIN T_PARTIC_INDIV I ON I.SID = C.INDIV_PARTIC AND I.OBJ = L_ALL_PARTICS(SUBJECT) 
                INNER JOIN T_ACM_OBJ O ON O.SID = I.OBJ
                ORDER BY C.EFFECTIVE_DATE DESC NULLS LAST,
                         C.SID DESC)
            LOOP
                L_CTRY_FOUND := 'Y';
                IF INDIV_C.COUNTRY IS NULL THEN
                    L_FAILED_SUBJECTS.EXTEND;
                    L_FAILED_SUBJECTS(L_FAILED_SUBJECTS.COUNT) := INDIV_C.TAGLINE;
                END IF;
            END LOOP;
        END LOOP;
        IF L_ALL_PARTICS.COUNT > 0 AND L_CTRY_FOUND = 'N' THEN
            L_RTN := 'All Subjects require citizenship entry including Nationality.';
            RETURN L_RTN;
        END IF;       
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Subject';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_FAILED_SUBJECTS(I) || ', ';
                ELSE
                    L_RTN := L_RTN || L_FAILED_SUBJECTS(I) || ' ';
                END IF;
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || ' nationality on most recent effective date citizenship.';
        END IF;
        RETURN L_RTN;
    END COND_SUBJECT_HAS_NATIONALITY;
/** Summary
<table>
<tr><td>1/29/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_US_PERSON RETURN VARCHAR2 IS
        L_RTN VARCHAR2(1000) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_PARTIC_COUNT NUMBER := 0;
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_FAILED_SUBJECTS_NONINDIV T_STR_ARRAY := T_STR_ARRAY();
        L_SUBJECT_NAME VARCHAR2(255);
        L_SUBJECT_NON_INDIV_NAME VARCHAR2(255);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.PARTICIPANT) INTO L_PARTIC_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ;
        IF L_PARTIC_COUNT = 0 THEN
            IF NO_SUBJECT_BYPASS(L_OBJ) = 'N' THEN
              L_RTN := 'Subject requires US Person / US Non-Individual field as Yes or No.';
              RETURN L_RTN;
            ELSE
              RETURN L_RTN;
            END IF;            
        END IF;
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_FAILED_SUBJECTS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            INNER JOIN T_PARTIC_INDIV I ON I.OBJ = OPI.PARTICIPANT
            WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ AND US_PERSON IS NULL;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RETURN L_RTN;
        END;
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Subject';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                L_SUBJECT_NAME := ACM_OBJ.GET_TAGLINE(L_FAILED_SUBJECTS(I));
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_SUBJECT_NAME || ', ';
                ELSE
                    L_RTN := L_RTN || L_SUBJECT_NAME || ' ';
                END IF;
                L_SUBJECT_NAME := '';
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || 'US Person field as Yes or No. ';
        END IF;

            BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_FAILED_SUBJECTS_NONINDIV
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            INNER JOIN T_PARTIC_NONINDIV NI ON NI.OBJ = OPI.PARTICIPANT
            WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ AND US_NONINDIV IS NULL;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RETURN L_RTN;
        END;
        IF L_FAILED_SUBJECTS_NONINDIV.COUNT > 0 THEN
            IF L_RTN = 'Y' THEN
            L_RTN := ' ' || 'Subject';
            ELSE
            L_RTN := ' ' || L_RTN || 'Subject';
            END IF;
            IF L_FAILED_SUBJECTS_NONINDIV.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS_NONINDIV.COUNT
            LOOP
                L_SUBJECT_NON_INDIV_NAME := ACM_OBJ.GET_TAGLINE(L_FAILED_SUBJECTS_NONINDIV(I));
                IF I != L_FAILED_SUBJECTS_NONINDIV.COUNT THEN
                    L_RTN := L_RTN || L_SUBJECT_NON_INDIV_NAME || ', ';
                ELSE
                    L_RTN := L_RTN || L_SUBJECT_NON_INDIV_NAME || ' ';
                END IF;
                L_SUBJECT_NON_INDIV_NAME := '';
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS_NONINDIV.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || 'US Non-Individual field as Yes or No.';
        END IF;

        RETURN L_RTN;
    END COND_SUBJECT_US_PERSON ;
/** Summary
<table>
<tr><td>1/30/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_CLEARANCE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_PARTIC_COUNT NUMBER := 0;
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_SUBJECT_NAME VARCHAR2(255);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.PARTICIPANT) INTO L_PARTIC_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ;
        IF L_PARTIC_COUNT = 0 THEN
            IF NO_SUBJECT_BYPASS(L_OBJ) = 'N' THEN
              L_RTN :=  'Subject requires a value for clearance.';
              RETURN L_RTN;
            ELSE
              RETURN L_RTN;
            END IF;          
        END IF;
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_FAILED_SUBJECTS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            INNER JOIN T_PARTIC_INDIV I ON I.OBJ = OPI.PARTICIPANT
            WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ AND I.CLEARANCE IS NULL;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RETURN L_RTN;
        END;
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Subject';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                L_SUBJECT_NAME := ACM_OBJ.GET_TAGLINE(L_FAILED_SUBJECTS(I));
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_SUBJECT_NAME || ', ';
                ELSE
                    L_RTN := L_RTN || L_SUBJECT_NAME || ' ';
                END IF;
                L_SUBJECT_NAME := '';
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || 'clearance level.';
        END IF;
        RETURN L_RTN;
    END COND_SUBJECT_HAS_CLEARANCE ;
    /** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created.</td></tr>
<tr><td>7/08/2022</td><td>CPB</td><td>Changed function name and display for Treasury.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_TRS_EMP RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_PARTIC_COUNT NUMBER := 0;
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_SUBJECT_NAME VARCHAR2(255);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.PARTICIPANT) INTO L_PARTIC_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ;
        IF L_PARTIC_COUNT = 0 THEN
            IF NO_SUBJECT_BYPASS(L_OBJ) = 'N' THEN
              L_RTN :=  'Subject requires Treasury Employment.';
              RETURN L_RTN;
            ELSE
              RETURN L_RTN;
            END IF;         
        END IF;
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_FAILED_SUBJECTS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            INNER JOIN T_PARTIC_INDIV I ON I.OBJ = OPI.PARTICIPANT
            WHERE OPRT.CODE IN ('SUBJECT') AND OPI.OBJ = L_OBJ AND OPI.INDIV_DHS_COMP IS NULL;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RETURN L_RTN;
        END;
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Subject';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                L_SUBJECT_NAME := ACM_OBJ.GET_TAGLINE(L_FAILED_SUBJECTS(I));
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_SUBJECT_NAME || ', ';
                ELSE
                    L_RTN := L_RTN || L_SUBJECT_NAME || ' ';
                END IF;
                L_SUBJECT_NAME := '';
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || 'Treasury Employment.';
        END IF;
        RETURN L_RTN;
    END COND_SUBJECT_HAS_TRS_EMP ;
    /** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created.</td></tr>
<tr><td>7/08/2022</td><td>CPB</td><td>Changed function name and display for Treasury.</td></tr>
</table>
*/
    FUNCTION COND_TRAVELER_HAS_TRS_EMP RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_PARTIC_COUNT NUMBER := 0;
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_SUBJECT_NAME VARCHAR2(255);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.PARTICIPANT) INTO L_PARTIC_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('TRAVELER') AND OPI.OBJ = L_OBJ;
        IF L_PARTIC_COUNT = 0 THEN
            L_RTN := 'Traveler requires Treasury Employment.';
            RETURN L_RTN;
        END IF;
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_FAILED_SUBJECTS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            INNER JOIN T_PARTIC_INDIV I ON I.OBJ = OPI.PARTICIPANT
            WHERE OPRT.CODE IN ('TRAVELER') AND OPI.OBJ = L_OBJ AND OPI.INDIV_DHS_COMP IS NULL;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RETURN L_RTN;
        END;
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Traveler';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                L_SUBJECT_NAME := ACM_OBJ.GET_TAGLINE(L_FAILED_SUBJECTS(I));
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_SUBJECT_NAME || ', ';
                ELSE
                    L_RTN := L_RTN || L_SUBJECT_NAME || ' ';
                END IF;
                L_SUBJECT_NAME := '';
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || 'Treasury Employment.';
        END IF;
        RETURN L_RTN;
    END COND_TRAVELER_HAS_TRS_EMP ;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_TRAVELER_US_PERSON RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_PARTIC_COUNT NUMBER := 0;
        L_FAILED_SUBJECTS T_STR_ARRAY := T_STR_ARRAY();
        L_SUBJECT_NAME VARCHAR2(255);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.PARTICIPANT) INTO L_PARTIC_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('TRAVELER') AND OPI.OBJ = L_OBJ;
        IF L_PARTIC_COUNT = 0 THEN
            L_RTN := 'Traveler requires US Person field as Yes or No.';
            RETURN L_RTN;
        END IF;
        BEGIN
            SELECT OPI.PARTICIPANT BULK COLLECT INTO L_FAILED_SUBJECTS
            FROM T_PARTIC_INVOLVEMENT OPI 
            INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
            INNER JOIN T_PARTIC_INDIV I ON I.OBJ = OPI.PARTICIPANT
            WHERE OPRT.CODE IN ('TRAVELER') AND OPI.OBJ = L_OBJ AND US_PERSON IS NULL;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                RETURN L_RTN;
        END;
        IF L_FAILED_SUBJECTS.COUNT > 0 THEN
            L_RTN := 'Traveler';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || 's';
            END IF;
            L_RTN := L_RTN || ': ';
            FOR I IN 1..L_FAILED_SUBJECTS.COUNT
            LOOP
                L_SUBJECT_NAME := ACM_OBJ.GET_TAGLINE(L_FAILED_SUBJECTS(I));
                IF I != L_FAILED_SUBJECTS.COUNT THEN
                    L_RTN := L_RTN || L_SUBJECT_NAME || ', ';
                ELSE
                    L_RTN := L_RTN || L_SUBJECT_NAME || ' ';
                END IF;
                L_SUBJECT_NAME := '';
            END LOOP;
            L_RTN := L_RTN || 'require';
            IF L_FAILED_SUBJECTS.COUNT > 1 THEN
                L_RTN := L_RTN || ' ';
            ELSE
                L_RTN := L_RTN || 's ';
            END IF;
            L_RTN := L_RTN || 'US Person field as Yes or No.';
        END IF;
        RETURN L_RTN;
    END COND_TRAVELER_US_PERSON ;
/** Summary
<table>
<tr><td>3/26/2020</td><td>LLSB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_HAS_PRIMARY_COUNTRY RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_COUNTRY_COUNT NUMBER := 0;
        L_OBJ_ROOT T_ACM_OBJ_TYPE.OBJ_ROOT%TYPE;
        L_OBJ_TYPE_CODE T_ACM_OBJ_TYPE.OBJ_TYPE_CODE%TYPE;
        L_OBJ_SPPRT_COUNT NUMBER := 0;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT OT.OBJ_ROOT,OBJ_TYPE_CODE INTO L_OBJ_ROOT,L_OBJ_TYPE_CODE
          FROM T_ACM_OBJ_TYPE OT
          INNER JOIN T_ACM_OBJ O ON OT.SID = O.OBJ_TYPE
         WHERE O.SID = L_OBJ;

        SELECT COUNT(SID) INTO L_COUNTRY_COUNT
          FROM T_ACM_OBJ_VENUE
         WHERE OBJ = L_OBJ AND PRIMARY_CTRY = 'Y';
        IF L_COUNTRY_COUNT = 0 THEN
            L_RTN := 'Must have a Primary Target Country';
        END IF;
        RETURN L_RTN;
    END COND_HAS_PRIMARY_COUNTRY;
/** Summary
<table>
<tr><td>9/30/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUBJECT_VISITOR RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_VET_TYPE VARCHAR2(100);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT VETTING_TYPE INTO L_VET_TYPE FROM T_F_FOREIGN_VET WHERE OBJ = L_OBJ;
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('SUBJECT_VISITOR') AND
              OPI.OBJ = L_OBJ AND
              ROWNUM = 1;
        IF L_COUNT > 0 OR L_VET_TYPE = 'FILE.FOREIGN_VET.VETTING_TYPE.CONTACT' THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Subject Visitor before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_SUBJECT_VISITOR;
/** Summary
<table>
<tr><td>10/1/2020</td><td>LM</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_HAS_PRIMARY_LOCATION RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_LOCATION_COUNT NUMBER := 0;
        L_OBJ_ROOT T_ACM_OBJ_TYPE.OBJ_ROOT%TYPE;
        L_OBJ_TYPE_CODE T_ACM_OBJ_TYPE.OBJ_TYPE_CODE%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT OT.OBJ_ROOT,OBJ_TYPE_CODE INTO L_OBJ_ROOT,L_OBJ_TYPE_CODE
          FROM T_ACM_OBJ_TYPE OT
          INNER JOIN T_ACM_OBJ O ON OT.SID = O.OBJ_TYPE
         WHERE O.SID = L_OBJ;

        SELECT COUNT(SID) INTO L_LOCATION_COUNT
          FROM T_ACM_PLACE
         WHERE OBJ = L_OBJ;
        IF L_LOCATION_COUNT = 0 THEN
            L_RTN := 'There must be at least one location associated.';
        END IF;
        RETURN L_RTN;
    END COND_HAS_PRIMARY_LOCATION;
/** Summary
<table>
<tr><td>10/1/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_HAS_FOREIGN_CONTACT RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_VET_TYPE VARCHAR2(100);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT VETTING_TYPE INTO L_VET_TYPE FROM T_F_FOREIGN_VET WHERE OBJ = L_OBJ;
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('FOREIGN_CONTACT') AND
              OPI.OBJ = L_OBJ AND
              ROWNUM = 1;
        IF L_COUNT > 0 OR L_VET_TYPE = 'FILE.FOREIGN_VET.VETTING_TYPE.VISIT' THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Foreign Contact before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_FOREIGN_CONTACT;
/** Summary
<table>
<tr><td>10/1/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_FOR_CONTACT_US_PERS RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_VET_TYPE VARCHAR2(100);
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT VETTING_TYPE INTO L_VET_TYPE FROM T_F_FOREIGN_VET WHERE OBJ = L_OBJ;
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        INNER JOIN T_PARTIC_INDIV TPI ON OPI.PARTICIPANT = TPI.OBJ
        INNER JOIN T_PARTIC_INDIV_CITIZENSHIP TPIC ON TPI.SID = TPIC.INDIV_PARTIC
        WHERE OPRT.CODE IN ('FOREIGN_CONTACT') AND
              OPI.OBJ = L_OBJ AND
              TPI.US_PERSON IS NOT NULL AND
              TPIC.COUNTRY IS NOT NULL AND
              ROWNUM = 1;
        IF L_COUNT > 0 OR L_VET_TYPE = 'FILE.FOREIGN_VET.VETTING_TYPE.VISIT' THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Foreign Contact needs US Person data and Citizenship before commiting this action.';
        RETURN L_RTN;
    END COND_FOR_CONTACT_US_PERS;
/** Summary
<table>
<tr><td>12/28/2020</td><td>LLSB</td><td>Initial creation for Task 13282</td></tr>
</table>
*/
  FUNCTION COND_HAS_PRE_BRIEF RETURN VARCHAR2 IS
    L_RTN VARCHAR2(255) := 'Y';
    L_OBJ T_ACM_OBJ.SID%TYPE;
    L_COUNT NUMBER := 0;
  BEGIN
    L_OBJ := ACM_BPE.GET_PARAM('OBJ');

    FOR X IN (SELECT TAA.FROM_OBJ AS ASSOC_OBJ FROM T_ACM_ASSOCIATION TAA 
      INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
      INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE
     WHERE TAA.TO_OBJ = L_OBJ AND FROM_OT.OBJ_TYPE_CODE = 'ACT.FOR_BRIEFING'
     UNION
     SELECT TAA.TO_OBJ AS ASSOC_OBJ FROM T_ACM_ASSOCIATION TAA 
      INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
      INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
     WHERE TAA.FROM_OBJ = L_OBJ AND TO_OT.OBJ_TYPE_CODE = 'ACT.FOR_BRIEFING'
    )LOOP
      FOR Y IN (SELECT SID FROM T_A_FOR_BRIEFING WHERE OBJ = X.ASSOC_OBJ AND BRIEFING_TYPE = 'BRIEFING_TYPE.PB'
      )LOOP
        L_COUNT := L_COUNT + 1;
      END LOOP;
    END LOOP;
    IF L_COUNT = 0 THEN
      L_RTN := 'Must have at least one Pre-briefing Activity.';
    END IF;

    RETURN L_RTN;
  END COND_HAS_PRE_BRIEF;

/** Summary
<table>
<tr><td>12/29/2020</td><td>LLSB</td><td>Initial creation for Task 13282</td></tr>
</table>
*/
  FUNCTION COND_HAS_DEBRIEF RETURN VARCHAR2 IS
    L_RTN VARCHAR2(255) := 'Y';
    L_OBJ T_ACM_OBJ.SID%TYPE;
    L_COUNT NUMBER := 0;
  BEGIN
    L_OBJ := ACM_BPE.GET_PARAM('OBJ');

    FOR X IN (SELECT TAA.FROM_OBJ AS ASSOC_OBJ FROM T_ACM_ASSOCIATION TAA 
      INNER JOIN T_ACM_OBJ FROM_OBJ ON TAA.FROM_OBJ = FROM_OBJ.SID
      INNER JOIN T_ACM_OBJ_TYPE FROM_OT ON FROM_OT.SID = FROM_OBJ.OBJ_TYPE
     WHERE TAA.TO_OBJ = L_OBJ AND FROM_OT.OBJ_TYPE_CODE = 'ACT.FOR_BRIEFING'
     UNION
     SELECT TAA.TO_OBJ AS ASSOC_OBJ FROM T_ACM_ASSOCIATION TAA 
      INNER JOIN T_ACM_OBJ TO_OBJ ON TAA.TO_OBJ = TO_OBJ.SID
      INNER JOIN T_ACM_OBJ_TYPE TO_OT ON TO_OT.SID = TO_OBJ.OBJ_TYPE
     WHERE TAA.FROM_OBJ = L_OBJ AND TO_OT.OBJ_TYPE_CODE = 'ACT.FOR_BRIEFING'
    )LOOP
      FOR Y IN (SELECT SID FROM T_A_FOR_BRIEFING WHERE OBJ = X.ASSOC_OBJ AND BRIEFING_TYPE = 'BRIEFING_TYPE.DB'
      )LOOP
        L_COUNT := L_COUNT + 1;
      END LOOP;
    END LOOP;
    IF L_COUNT = 0 THEN
      L_RTN := 'Must have at least one Debrief Activity.';
    END IF;

    RETURN L_RTN;
  END COND_HAS_DEBRIEF;

/** Summary
<table>
<tr><td>12/29/2020</td><td>LLSB</td><td>Initial creation for Task 13282</td></tr>
</table>
*/
  FUNCTION COND_HAS_POST_TRAVEL_REC RETURN VARCHAR2 IS
    L_RTN VARCHAR2(255) := 'Y';
    L_OBJ T_ACM_OBJ.SID%TYPE;
  BEGIN
    L_OBJ := ACM_BPE.GET_PARAM('OBJ');

    FOR X IN (SELECT SID, POST_TRAVEL_REC FROM T_F_TRAVEL WHERE OBJ = L_OBJ
    )LOOP
      IF X.POST_TRAVEL_REC IS NULL THEN
        L_RTN := 'Must have a Post Travel Recommendation.';
      END IF;
    END LOOP;

    RETURN L_RTN;
  END COND_HAS_POST_TRAVEL_REC;

/** Summary
<table>
<tr><td>1/8/2021</td><td>CPB</td><td>Initial creation for Task 202013364</td></tr>
</table>
*/
    FUNCTION COND_PARTICS_HAS_INVOLV_CLEAR RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_SID_COUNT NUMBER;
        L_CLEARANCE_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID), COUNT(CLEARANCE)
        INTO L_SID_COUNT, L_CLEARANCE_COUNT
        FROM T_PARTIC_INVOLVEMENT
        WHERE OBJ = L_OBJ AND GET_OBJTYPE_CODE(ACM_OBJ.GET_OBJTYPE(PARTICIPANT)) LIKE 'PARTIC.INDIV';
        IF L_CLEARANCE_COUNT = L_SID_COUNT THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'All individual participants must have a clearance at involvement before committing this action.';
        RETURN L_RTN;
    END COND_PARTICS_HAS_INVOLV_CLEAR;
/** Summary
<table>
<tr><td>2/15/2021</td><td>LM</td><td>Task 202100100</td></tr>
</table>
*/
    FUNCTION COND_HAS_ATTENDEE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_COUNT2 NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('BRIEFED') AND 
              OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Must have at least one Attendee';
        RETURN L_RTN;
    END COND_HAS_ATTENDEE;
/** Summary
<table>
<tr><td>2/15/2021</td><td>LM</td><td>Task 202100100</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUBJ_DISCUSSED RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('SUBJECT_DISCUSSED') AND 
              OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a participant with involvement role of "Subject Discussed" before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_SUBJ_DISCUSSED;
/** Summary
<table>
<tr><td>2/15/2021</td><td>LM</td><td>Task 202100100</td></tr>
</table>
*/
    FUNCTION COND_HAS_FOREIGNER RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('FOREIGNER') AND 
              OPI.OBJ = L_OBJ AND 
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Must have a participant with involvement role of "Foreigner"';
        RETURN L_RTN;
    END COND_HAS_FOREIGNER;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
    FUNCTION COND_HAS_FOREIGN_ACQUIRER RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('ACQUIRER') AND
              OPI.OBJ = L_OBJ AND
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Foreign Acquirer Participant before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_FOREIGN_ACQUIRER;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
    FUNCTION COND_HAS_TARGET RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('TARGET') AND
              OPI.OBJ = L_OBJ AND
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Target Participant before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_TARGET;
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_PURCHASE_DT RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND PURCHASE_DATE IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Purchase Date must have a value.';
        RETURN L_RTN;
    END COND_HAS_PURCHASE_DT; 
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_LVL_CONFIDENCE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND LEVEL_OF_CONFIDENCE IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Level of Confidence must have a value.';
        RETURN L_RTN;
    END COND_HAS_LVL_CONFIDENCE;
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_LVL_LIKELIHOOD RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND LEVEL_OF_LIKELIHOOD IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Level of Likelihood must have a value.';
        RETURN L_RTN;
    END COND_HAS_LVL_LIKELIHOOD;     
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_TARGET_NARRATIVE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND TARGET_NARRATIVE IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Target Narrative must have a value.';
        RETURN L_RTN;
    END COND_HAS_TARGET_NARRATIVE;
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_ACQUIRER_NARRATIVE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND ACQUIRER_NARRATIVE IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Acquirer Narrative must have a value.';
        RETURN L_RTN;
    END COND_HAS_ACQUIRER_NARRATIVE;

/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_TRANSACTION_VALUE RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND TRANSACTION_VALUE IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Transaction Amount must have value.';
        RETURN L_RTN;
    END COND_HAS_TRANSACTION_VALUE;
/** Summary
<table>
<tr><td>08/09/2021</td><td>LM</td><td>Supply Chain</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUPPLIER RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE OPRT.CODE IN ('SUPPLIER') AND
              OPI.OBJ = L_OBJ AND
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Object must have a Supplier before commiting this action.';
        RETURN L_RTN;
    END COND_HAS_SUPPLIER;
/** Summary
<table>
<tr><td>08/09/2021</td><td>LM</td><td>Supply Chain</td></tr>
</table>
*/
    FUNCTION COND_SUPPLIER_HAS_LEGAL_NAME RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        INNER JOIN T_PARTIC_NAME TPN ON TPN.OBJ = OPI.PARTICIPANT
        WHERE OPRT.CODE IN ('SUPPLIER') AND
              OPI.OBJ = L_OBJ AND
              TPN.NAME_TYPE = 'PARTIC.NONINDIV.NAME_TYPE.LEGAL' AND
              ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Supplier must have legal name.';
        RETURN L_RTN;
    END COND_SUPPLIER_HAS_LEGAL_NAME;
/** Summary
<table>
<tr><td>8/09/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
    FUNCTION COND_HAS_INDUSTRY_TARGET RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
        INTO L_COUNT
        FROM T_F_CFIUS TFC 
        WHERE OBJ = ACM_BPE.GET_PARAM('OBJ') AND INDUSTRY_TGT IS NOT NULL;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Industry Target must have value.';
        RETURN L_RTN;
    END COND_HAS_INDUSTRY_TARGET;    



/** Summary
<table>
<tr><td>9/17/2021</td><td>SK</td><td>Task 202100879</td></tr>
</table>
*/
    FUNCTION COND_HAS_CI_INDICATOR_ID RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
          INTO L_COUNT
          FROM T_F_TRAVEL 
         WHERE OBJ = L_OBJ AND CI_INDICATOR IS NOT NULL;
        IF L_COUNT = 0 THEN
            L_RTN := 'CI indicator identified must have value.';
        END IF;
        RETURN L_RTN;
    END COND_HAS_CI_INDICATOR_ID;
/** Summary
<table>
<tr><td>9/17/2021</td><td>SK</td><td>Task 202100879</td></tr>
</table>
*/
    FUNCTION COND_HAS_CI_INDICATOR_ICIR RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        SELECT COUNT(SID) 
          INTO L_COUNT
          FROM T_F_TRAVEL
         WHERE OBJ = L_OBJ AND CI_INDICATOR = 'Y';
        IF L_COUNT > 0 THEN
            IF COND_ASSOCIATED_TO_ICIR_FILE <> 'Y' THEN
                L_RTN := 'Travel File must be associated to an ICIR when CI Indicator identified.';
            END IF;
        END IF;
        RETURN L_RTN;
    END COND_HAS_CI_INDICATOR_ICIR; 
/** Summary
<table>
<tr><td>10/04/2021</td><td>LLSB</td><td>Task 202100967</td></tr>
</table>
*/
    FUNCTION COND_HAS_ANP_PRODUCT RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_LEAD T_ACM_STAKEHOLDER.STAKEHOLDER%TYPE := NULL;
        L_REQUEST T_ACM_STAKEHOLDER.STAKEHOLDER%TYPE := NULL;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        -- Products
        SELECT COUNT(SID) 
          INTO L_COUNT
          FROM T_ACM_ATTACHMENT
         WHERE OBJ = L_OBJ AND ATCH_TYPE IN (SELECT SID FROM T_ACM_ATTACHMENT_TYPE WHERE CODE = 'PRODUCT');
        IF L_COUNT = 0 THEN
            SELECT COUNT(SID) 
              INTO L_COUNT
              FROM T_ACM_NOTE
             WHERE OBJ = L_OBJ AND NOTE_TYPE IN (SELECT SID FROM T_ACM_NOTE_TYPE WHERE CODE = 'PRODUCT');
            IF L_COUNT = 0 THEN
                L_RTN := 'Must have a Product Note or an Analysis Product Attachment. ';
            END IF;
        END IF;
        RETURN L_RTN;
    END COND_HAS_ANP_PRODUCT;

/** Summary
<table>
<tr><td>10/20/2021</td><td>LLSB</td><td>Task 202100967</td></tr>
</table>
*/
    FUNCTION COND_HAS_ANP_LEAD RETURN VARCHAR2 IS
        L_RTN VARCHAR2(255) := 'Y';
        L_COUNT NUMBER;
        L_OBJ T_ACM_OBJ.SID%TYPE;
        L_LEAD T_ACM_STAKEHOLDER.STAKEHOLDER%TYPE := NULL;
        L_REQUEST T_ACM_STAKEHOLDER.STAKEHOLDER%TYPE := NULL;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');
        -- Stakeholders
        SELECT COUNT (*)
          INTO L_COUNT
          FROM T_ACM_STAKEHOLDER TAS
          INNER JOIN T_ACM_STAKEHOLDER_ROLE TASR ON TAS.SH_ROLE = TASR.SID
         WHERE TAS.OBJ = L_OBJ AND TASR.CODE = 'REQUESTER';
        IF L_COUNT > 0 THEN
            SELECT STAKEHOLDER
              INTO L_REQUEST
              FROM T_ACM_STAKEHOLDER TAS
              INNER JOIN T_ACM_STAKEHOLDER_ROLE TASR ON TAS.SH_ROLE = TASR.SID
             WHERE TAS.OBJ = L_OBJ AND TASR.CODE = 'REQUESTER' AND TAS.END_DATE IS NULL;
            SELECT STAKEHOLDER
              INTO L_LEAD
              FROM T_ACM_STAKEHOLDER TAS
              INNER JOIN T_ACM_STAKEHOLDER_ROLE TASR ON TAS.SH_ROLE = TASR.SID
             WHERE TAS.OBJ = L_OBJ AND TASR.CODE = 'LEAD' AND TAS.END_DATE IS NULL;
            IF L_LEAD = L_REQUEST THEN
                L_RTN := 'Lead and Requesting Lead should not match.';
            END IF;
        END IF;
        RETURN L_RTN;
    END COND_HAS_ANP_LEAD;

/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_ICIR_CIRA_REC_HAS_ATTCH RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT COUNT(TFF.SID)
        INTO L_COUNT_2
        FROM T_F_FINDING TFF
        INNER JOIN T_F_FINDING_REF TFFRR ON TFFRR.REF_KEY = TFF.RECOMMENDATION
        INNER JOIN T_F_FINDING_REF TFFR ON TFFR.REF_KEY = TFF.TYPE
        WHERE TFF.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        AND (TFFRR.CODE = 'CIRA' AND TFFRR.USAGE = 'RECOMMEND');
        IF L_COUNT_2 > 0 THEN
            SELECT COUNT(TAA.SID) INTO L_COUNT FROM T_ACM_ATTACHMENT TAA 
            INNER JOIN T_ACM_OBJ O ON TAA.OBJ = O.SID
			INNER JOIN T_ACM_ATTACHMENT_TYPE AAT ON TAA.ATCH_TYPE = AAT.SID
            WHERE (TAA.OBJ = ACM_BPE.GET_PARAM('OBJ') AND AAT.CODE = 'CIRA') AND ROWNUM = 1;
            IF L_COUNT = 0 THEN
                L_RTN := 'ICIR with CIRA Recommendation must have an attached CIRA.';
            ELSE
                L_RTN := 'Y';
            END IF;
        ELSE
            L_RTN := 'Y';       
        END IF;
        RETURN L_RTN;
    END COND_ICIR_CIRA_REC_HAS_ATTCH;

/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CIRA_HAS_SUM_NARR RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT
            COUNT(N.SID)
        INTO
            L_COUNT
        FROM
            T_F_RISK_ASSESS_NARRATIVES N
            INNER JOIN T_F_RISK_ASSESS_REF R ON R.SID = N.NARRATIVE_TYPE
        WHERE
            N.OBJ = ACM_BPE.GET_PARAM('OBJ') 
            AND R.USAGE = 'RA_FILE_SUM_TYPE' AND R.CODE = 'PRED';
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'Must have a Summary Narrative';
        END IF;
        RETURN L_RTN;
    END COND_CIRA_HAS_SUM_NARR;

	/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_TIRA_HAS_SUM_NARR RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200);
    BEGIN
        SELECT
            COUNT(N.SID)
        INTO
            L_COUNT
        FROM
            T_F_IT_RA_NARRATIVES N
            INNER JOIN T_F_IT_RA_REF R ON R.SID = N.NARRATIVE_TYPE
        WHERE
            N.OBJ = ACM_BPE.GET_PARAM('OBJ') 
            AND R.USAGE = 'RA_FILE_SUM_TYPE' AND R.CODE = 'PRED';
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'Must have a Summary Narrative';
        END IF;
        RETURN L_RTN;
    END COND_TIRA_HAS_SUM_NARR;

/** Summary
<table>
<tr><td>4/21/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_SUBJ_HAS_ID RETURN VARCHAR2 IS
        L_COUNT_1 NUMBER := 0;
        L_COUNT_2 NUMBER := 0;
        L_RTN VARCHAR2(200) := 'Y';
    BEGIN
        FOR X IN (  SELECT
                        PARTICIPANT
                    FROM
                        T_PARTIC_INVOLVEMENT I
                        INNER JOIN T_PARTIC_ROLE_TYPE R ON R.SID = I.INVOLVEMENT_ROLE AND R.CODE = 'SUBJECT'
                    WHERE
                        I.OBJ = ACM_BPE.GET_PARAM('OBJ') 
        ) LOOP
            L_COUNT_1 := 0;
            L_COUNT_2 := 0;

            SELECT
                COUNT(1)
            INTO
                L_COUNT_1
            FROM
                T_PARTIC_NUMBER
            WHERE
                OBJ = X.PARTICIPANT;

            SELECT
                COUNT(1)
            INTO
                L_COUNT_2
            FROM
                T_ACM_NOTE N
                INNER JOIN T_ACM_NOTE_TYPE T ON T.SID = N.NOTE_TYPE
            WHERE
                N.OBJ = X.PARTICIPANT
                AND T.CODE = 'REASON'
                AND T.OBJ_TYPE = (SELECT SID FROM T_ACM_OBJ_TYPE WHERE OBJ_TYPE_CODE = 'PARTIC.INDIV');

            IF L_COUNT_1 = 0 AND L_COUNT_2 = 0 THEN
                L_RTN := 'All subjects require at least one Identifying Number or a "Reason ID Number Missing" note';
                RETURN L_RTN;
            END IF;
        END LOOP;
        RETURN L_RTN;
    END COND_SUBJ_HAS_ID;
/** Summary
<table>
<tr><td>4/21/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_ROIFP_ATTACH RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200) := 'Y';
    BEGIN
        SELECT
            COUNT(1)
        INTO
            L_COUNT
        FROM
            T_ACM_ATTACHMENT A
            INNER JOIN T_ACM_ATTACHMENT_TYPE T ON T.SID = A.ATCH_TYPE
        WHERE
            A.OBJ = ACM_BPE.GET_PARAM('OBJ')
            AND T.CODE = 'ROIFP';
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'Must have ROI/CSR Final Published report attached';
        END IF;
        RETURN L_RTN;
    END COND_HAS_ROIFP_ATTACH;
/** Summary
<table>
<tr><td>4/28/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_LEAD_NOTE RETURN VARCHAR2 IS
        L_COUNT NUMBER := 0;
        L_RTN VARCHAR2(200) := 'Y';
    BEGIN
        SELECT
            COUNT(1)
        INTO
            L_COUNT
        FROM
            T_ACM_NOTE N
            INNER JOIN T_ACM_NOTE_TYPE T ON T.SID = N.NOTE_TYPE
        WHERE
            N.OBJ = ACM_BPE.GET_PARAM('OBJ')
            AND T.CODE = 'LEAD';
        IF L_COUNT > 0 THEN
            L_RTN := 'Y';
        ELSE
            L_RTN := 'Must have a Lead Note';
        END IF;
        RETURN L_RTN;
    END COND_HAS_LEAD_NOTE;
/** Summary
<table>
<tr><td>5/2/2022</td><td>CPB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_HAS_CYBER_TGT RETURN VARCHAR2 IS
        L_RTN       VARCHAR2(255) := 'Y';
        L_COUNT     NUMBER;
        L_OBJ       T_ACM_OBJ.SID%TYPE;
        L_ROLE      T_PARTIC_ROLE_TYPE.DISPLAY%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');

        SELECT
            DISPLAY
        INTO
            L_ROLE
        FROM
            T_PARTIC_ROLE_TYPE
        WHERE
            CODE = 'CYBER_TARGET'
            AND ACTIVE = 'Y'
            AND OBJ_TYPE = (SELECT OBJ_TYPE FROM T_ACM_OBJ WHERE SID = L_OBJ);

        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE
            OPRT.CODE IN ('CYBER_TARGET')
            AND OPRT.ACTIVE = 'Y'
            AND OPI.OBJ = L_OBJ
            AND ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Must have a participant with involvement role of "' || L_ROLE || '"';
        RETURN L_RTN;
    END COND_HAS_CYBER_TGT;
/** Summary
<table>
<tr><td>5/2/2022</td><td>CPB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_HAS_REQ_OFFICE RETURN VARCHAR2 IS
        L_RTN       VARCHAR2(255) := 'Y';
        L_COUNT     NUMBER;
        L_OBJ       T_ACM_OBJ.SID%TYPE;
        L_ROLE      T_PARTIC_ROLE_TYPE.DISPLAY%TYPE;
    BEGIN
        L_OBJ := ACM_BPE.GET_PARAM('OBJ');

        SELECT
            DISPLAY
        INTO
            L_ROLE
        FROM
            T_PARTIC_ROLE_TYPE
        WHERE
            CODE = 'REQ_OFFICE'
            AND ACTIVE = 'Y'
            AND OBJ_TYPE = (SELECT OBJ_TYPE FROM T_ACM_OBJ WHERE SID = L_OBJ);

        SELECT COUNT(OPI.SID) 
        INTO L_COUNT
        FROM T_PARTIC_INVOLVEMENT OPI 
        INNER JOIN T_PARTIC_ROLE_TYPE OPRT ON OPI.INVOLVEMENT_ROLE = OPRT.SID
        WHERE
            OPRT.CODE IN ('REQ_OFFICE')
            AND OPRT.ACTIVE = 'Y'
            AND OPI.OBJ = L_OBJ
            AND ROWNUM = 1;
        IF L_COUNT > 0 THEN
            RETURN L_RTN;
        END IF;
        L_RTN := 'Must have a participant with involvement role of "' || L_ROLE || '"';
        RETURN L_RTN;
    END COND_HAS_REQ_OFFICE;
END ACM_BPE_COND;
/



--Package HEADER




create or replace PACKAGE       "ACM_BPE_COND" as 
/** Generic BPE conditions that apply to multiple object root types.
*<table>
*<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
<tr><td>1/21/2019</td><td>KDB</td><td>COND_NOT_PENDING_RESPONSE deprecated as of switch to stasuses, so removed.</td></tr>
*</table>
*/
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_STATUS_IS_NOT_CLOSED RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUBJECT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_COMPONENT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUP_COMPONENT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>4/21/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_OTHER_AGENCY RETURN VARCHAR2;    
/** Summary
<table>
<tr><td>2/20/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_HAS_BRIEFEE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_HAS_TRAVELER RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/20/2020</td><td>DL</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/

    FUNCTION COND_HAS_COUNTRY RETURN VARCHAR2;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_HAS_ASSOCIATED_FILE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
</table>
*/
    FUNCTION COND_ASSOCIATED_TO_TRAVEL_FILE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>06/30/2020</td><td>LLSB</td><td>Initial Creation for Task 12745</td></tr>
</table>
*/    
    FUNCTION COND_ASSOCIATED_TO_ICIR_FILE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/23/2019</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_IRR_VALUE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_IRR_WITH_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_FI_WITH_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_PI_WITH_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_CI_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_CI_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CIS_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CIS_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_TSA_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_TSA_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_ICE_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_ICE_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_FPS_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_FPS_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CISA_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CISA_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_FEMA_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_FEMA_INQ_W_HQ_FINAL_REC RETURN VARCHAR2; 
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_USSS_INV_W_HQ_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_USSS_INQ_W_HQ_FINAL_REC RETURN VARCHAR2;     
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_RECEIPT_METHOD RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_PARTICIPANT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/21/2019</td><td>KDB</td><td>Merged ACM_ACTIVITY.COND_ACTIVITY_HAS_SUBJECT and ACM_FILE.COND_FILE_HAS_SUBJECT to here.</td></tr>
</table>
*/
    FUNCTION COND_HAS_NUMBER_BRIEFED RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/29/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_ETHNICITY RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/29/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_NATIONALITY RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/29/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_US_PERSON RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/30/2019</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_CLEARANCE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created.</td></tr>
<tr><td>7/08/2022</td><td>CPB</td><td>Changed function name and display for Treasury.</td></tr>
</table>
*/
    FUNCTION COND_SUBJECT_HAS_TRS_EMP RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>DL</td><td>Created.</td></tr>
<tr><td>7/08/2022</td><td>CPB</td><td>Changed function name and display for Treasury.</td></tr>
</table>
*/
    FUNCTION COND_TRAVELER_HAS_TRS_EMP RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/24/2020</td><td>KDB</td><td>Created.</td></tr>
</table>
*/
    FUNCTION COND_TRAVELER_US_PERSON RETURN VARCHAR2;
/** Summary
<table>
<tr><td>3/26/2020</td><td>LLSB</td><td>Created.</td></tr>
</table>
*/    
    FUNCTION COND_HAS_PRIMARY_COUNTRY RETURN VARCHAR2;
/** Summary
<table>
<tr><td>9/30/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_HAS_SUBJECT_VISITOR RETURN VARCHAR2; 
/** Summary
<table>
<tr><td>10/1/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_HAS_PRIMARY_LOCATION RETURN VARCHAR2;
/** Summary
<table>
<tr><td>10/1/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_HAS_FOREIGN_CONTACT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>10/1/2020</td><td>LM</td><td>Foreign Vetting File</td></tr>
</table>
*/
    FUNCTION COND_FOR_CONTACT_US_PERS RETURN VARCHAR2;
/** Summary
<table>
<tr><td>12/28/2020</td><td>LLSB</td><td>Created.</td></tr>
</table>
*/
  FUNCTION COND_HAS_PRE_BRIEF RETURN VARCHAR2;

/** Summary
<table>
<tr><td>12/29/2020</td><td>LLSB</td><td>Created.</td></tr>
</table>
*/
  FUNCTION COND_HAS_DEBRIEF RETURN VARCHAR2;

/** Summary
<table>
<tr><td>12/29/2020</td><td>LLSB</td><td>Created.</td></tr>
</table>
*/
  FUNCTION COND_HAS_POST_TRAVEL_REC RETURN VARCHAR2;

/** Summary
<table>
<tr><td>1/8/2021</td><td>CPB</td><td>Initial creation for Task 202013364</td></tr>
</table>
*/
  FUNCTION COND_PARTICS_HAS_INVOLV_CLEAR RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/15/2021</td><td>LM</td><td>Task 202100100</td></tr>
</table>
*/
FUNCTION COND_HAS_ATTENDEE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/15/2021</td><td>LM</td><td>Task 202100100</td></tr>
</table>
*/
FUNCTION COND_HAS_SUBJ_DISCUSSED RETURN VARCHAR2;
/** Summary
<table>
<tr><td>2/15/2021</td><td>LM</td><td>Task 202100100</td></tr>
</table>
*/
FUNCTION COND_HAS_FOREIGNER RETURN VARCHAR2;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
FUNCTION COND_HAS_FOREIGN_ACQUIRER RETURN VARCHAR2;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
FUNCTION COND_HAS_TARGET RETURN VARCHAR2;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
FUNCTION COND_HAS_PURCHASE_DT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
FUNCTION COND_HAS_LVL_CONFIDENCE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>06/30/2021</td><td>LM</td><td>CFIUS</td></tr>
</table>
*/
FUNCTION COND_HAS_LVL_LIKELIHOOD RETURN VARCHAR2;
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
FUNCTION COND_HAS_TARGET_NARRATIVE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
FUNCTION COND_HAS_ACQUIRER_NARRATIVE RETURN VARCHAR2; 
/** Summary
<table>
<tr><td>8/02/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
FUNCTION COND_HAS_TRANSACTION_VALUE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>8/09/2021</td><td>LM</td><td>Supply chain file</td></tr>
</table>
*/
FUNCTION COND_HAS_SUPPLIER RETURN VARCHAR2;
/** Summary
<table>
<tr><td>08/09/2021</td><td>LM</td><td>Supply Chain</td></tr>
</table>
*/
FUNCTION COND_SUPPLIER_HAS_LEGAL_NAME RETURN VARCHAR2;
/** Summary
<table>
<tr><td>8/09/2021</td><td>LM</td><td>Task 202100738</td></tr>
</table>
*/
FUNCTION COND_HAS_INDUSTRY_TARGET RETURN VARCHAR2;
/** Summary
<table>
<tr><td>9/17/2021</td><td></td><td>Task 202100879</td></tr>
</table>
*/
FUNCTION COND_HAS_CI_INDICATOR_ID RETURN VARCHAR2;
/** Summary
<table>
<tr><td>9/17/2021</td><td></td><td>Task 202100879</td></tr>
</table>
*/
FUNCTION COND_HAS_CI_INDICATOR_ICIR RETURN VARCHAR2;
/** Summary
<table>
<tr><td>10/04/2021</td><td>LLSB</td><td>Task 202100967</td></tr>
</table>
*/
FUNCTION COND_HAS_ANP_PRODUCT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>10/20/2021</td><td>LLSB</td><td>Task 202100967</td></tr>
</table>
*/
    FUNCTION COND_HAS_ANP_LEAD RETURN VARCHAR2;

/** Summary
<table>
<tr><td>2/24/2020</td><td>JAF</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_ICIR_CIRA_REC_HAS_ATTCH RETURN VARCHAR2;

/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_CIRA_HAS_SUM_NARR RETURN VARCHAR2;

/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_TIRA_HAS_SUM_NARR RETURN VARCHAR2;

/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_SUBJ_HAS_ID RETURN VARCHAR2;

/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_ROIFP_ATTACH RETURN VARCHAR2;
/** Summary
<table>
<tr><td>4/20/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/    
    FUNCTION COND_HAS_LEAD_NOTE RETURN VARCHAR2;
/** Summary
<table>
<tr><td>5/2/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_CYBER_TGT RETURN VARCHAR2;
/** Summary
<table>
<tr><td>5/2/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_REQ_OFFICE RETURN VARCHAR2;    
end acm_bpe_cond;
/