-- 202200962 DDL Fix
create or replace PACKAGE BODY     DOCX_REPORT AS
/** Contains functions and procedure related to all DOCX Reports.
*<table>
*<tr><td>6/4/2018</td><td>KDB</td><td>Added Comments to satisfy documentation generation.</td></tr>
*</table>
*/

/**
<table>
<tr><td>07/13/2021</td><td>CPB</td><td>Created</td></tr>
</table
*/
    PROCEDURE GET_SPEC (
        P_OBJECT        IN T_ACM_OBJ.SID%TYPE,
        P_REPORT_CODE   IN T_DOCX_REPORT_TYPE.CODE%TYPE,
        P_SPEC_ROW      IN OUT NOCOPY T_DOCX_REPORT_SPEC%ROWTYPE
    ) IS
    BEGIN
        SELECT
            S.*
        INTO P_SPEC_ROW
        FROM
            T_DOCX_REPORT_SPEC S
            INNER JOIN T_DOCX_REPORT_TYPE R ON R.SID = S.REPORT_TYPE AND USES_SPEC = 'Y'
        WHERE
            OBJ = P_OBJECT
            AND R.CODE LIKE '%' || P_REPORT_CODE;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
            ACM_LOG.INFO('Generating report without spec for P_OBJECT: ' || P_OBJECT);
    END;

/**
<table>
<tr><td>07/14/2021</td><td>CB</td><td>Created</td></tr>
</table>
*/
    FUNCTION GET_LEAD (
        P_OBJECT        IN VARCHAR2
    ) RETURN VARCHAR2 IS
    BEGIN
        FOR X IN (  SELECT
                        SH_DISPLAY
                    FROM
                        V_CURR_OBJ_LEAD_PERS
                    WHERE
                        OBJ = P_OBJECT
                        AND ROWNUM = 1
        )LOOP
            RETURN X.SH_DISPLAY;
        END LOOP;
        RETURN 'No Current Lead on Record';
    END GET_LEAD;
/**
<table>
<tr><td>03/16/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION GET_OWNING_UNIT (
        P_OBJECT        IN VARCHAR2
    ) RETURN VARCHAR2 IS
    BEGIN
        FOR X IN (  SELECT
                        SH_DISPLAY
                    FROM
                        V_CURR_OBJ_OWNING_UNIT
                    WHERE
                        OBJ = P_OBJECT
                        AND ROWNUM = 1
        )LOOP
            RETURN X.SH_DISPLAY;
        END LOOP;
        RETURN 'No Current Owning Unit on Record';
    END GET_OWNING_UNIT;
/**
<table>
<tr><td>03/28/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION GET_REPORT_PERSONNEL_NAME (
        P_OBJECT        IN VARCHAR2,
        P_SPEC_SID      IN T_DOCX_REPORT_SPEC.SID%TYPE
    ) RETURN VARCHAR2 IS
    BEGIN
        IF P_SPEC_SID IS NOT NULL THEN
            FOR X IN (
                SELECT
                    CLASSIFIED_BY
                FROM
                    T_DOCX_REPORT_SPEC
                WHERE
                    SID = P_SPEC_SID
            ) LOOP
                RETURN X.CLASSIFIED_BY;
            END LOOP;
        ELSE
            FOR X IN (
                SELECT
                    LAST_NAME || ', ' || FIRST_NAME AS PERSONNEL_NAME
                FROM
                    T_ACM_PERSONNEL
                WHERE
                    OBJ = ACM_CONTEXT.PERSONNEL_OBJ
            ) LOOP
                RETURN X.PERSONNEL_NAME;
            END LOOP;
        END IF;

        RETURN 'No Personnel Name on Record';
    END GET_REPORT_PERSONNEL_NAME;
/**
<table>
<tr><td>03/29/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    FUNCTION REPLACE_DOC_XML (
        P_TABLE_CAPTION     IN VARCHAR2,
        P_DATA_TO_INSERT    IN CLOB,
        P_FILE              IN BLOB,
        P_DELETE_TBL_HEADER IN VARCHAR2 DEFAULT 'Y'
    ) RETURN BLOB IS
        l_file_list         zip_util_pkg.t_file_list;
        l_blob              BLOB;
        l_clob              CLOB;
        l_returnvalue       BLOB;
        l_xml               xmltype;
        l_data_to_insert    xmltype;
        l_xquery            varchar2(4000);
        l_only_last_row     varchar2(20) := '';
    BEGIN
        IF P_DELETE_TBL_HEADER = 'N' THEN 
            l_only_last_row := '[last()]';
        END IF;

        l_xquery := 'declare namespace w = "http://schemas.openxmlformats.org/wordprocessingml/2006/main";
            copy $i := $document_xml modify
            (delete node $i//w:tbl[fn:count(./w:tblPr/w:tblCaption[@w:val="' || P_TABLE_CAPTION || '"]) > 0]/w:tr' || l_only_last_row ||',
            insert nodes $p_new_xml/dummy/w:tr as last into $i//w:tbl[fn:count(./w:tblPr/w:tblCaption[@w:val="' || P_TABLE_CAPTION || '"]) > 0])
            return $i';

        l_file_list := zip_util_pkg.get_file_list (P_FILE);

        FOR i IN 1 .. l_file_list.COUNT
        LOOP
            -- have to unzip all as header/footer could have been replaced elsewhere.
            -- We should strongly considering clobs/blobs/maybe even xmltype for each file we're modifying, keeping them in memory
            -- instead of unzip/zipping constantly
            -- There's no dedicated unzip/zip hardware in cpus it could potentially saturate at scale as we've seen with some customers
            l_blob := zip_util_pkg.get_file (P_FILE, l_file_list (i));

            IF (l_file_list (i) = 'word/document.xml') THEN
                l_clob := sql_util_pkg.blob_to_clob (l_blob);
                l_xml := xmltype(l_clob);
                l_data_to_insert := xmltype(p_data_to_insert);
                select xmlquery(l_xquery passing l_xml as "document_xml",l_data_to_insert as "p_new_xml" returning content).getClobVal()
                into l_clob
                from dual;
                l_blob := sql_util_pkg.clob_to_blob (l_clob);
            END IF;
            zip_util_pkg.add_file (l_returnvalue, l_file_list (i), l_blob);
        END LOOP;

        zip_util_pkg.finish_zip (l_returnvalue);

        RETURN l_returnvalue;
    END REPLACE_DOC_XML;
/**
<table>
<tr><td>03/29/2022</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    PROCEDURE GET_DOCX_BODY_TABLE_ROW_TEMPLATE (
        P_TABLE_CAPTION IN VARCHAR2,
        P_FILE_BLOB IN BLOB,
        P_CLOB IN OUT NOCOPY CLOB
    ) IS
        L_XQUERY VARCHAR2(4000);
    BEGIN
        L_XQUERY := 'declare namespace w = "http://schemas.openxmlformats.org/wordprocessingml/2006/main";
            for $i in //w:tbl
            where $i/w:tblPr/w:tblCaption/@w:val="' || P_TABLE_CAPTION || '"
            return $i//w:tr[fn:last()]';
        select xmlquery(L_XQUERY passing a.doc_xml returning content).getclobval()
        into P_CLOB
        from (
            select xmltype(sql_util_pkg.blob_to_clob(zip_util_pkg.get_file(P_FILE_BLOB, 'word/document.xml'))) doc_xml
            from DUAL) a;

    END GET_DOCX_BODY_TABLE_ROW_TEMPLATE;
/**
<table>
<tr><td>06/09/2021</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
    PROCEDURE START_REPORT (
        P_OBJECT            IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT       IN OUT NOCOPY BLOB,
        P_TEMPLATE_NAME     IN T_DOCX_REPORT_TEMPLATE.TEMPLATE_NAME%TYPE
    ) IS
    BEGIN
        ACM_LOG.INFO(MSG => 'Starting Report for object: ' || P_OBJECT);
        DOCX_TEMPLATE.GET_LATEST(P_TEMPLATE_NAME, P_DOCX_REPORT);
        DELETE FROM T_DOCX_REPORT_TEMPORARY
        WHERE
            APEX_SESSION = NVL(APEX_UTIL.GET_SESSION_STATE('SESSION'), 'XXX');
    END START_REPORT;
/**
<table>
<tr><td>06/09/2021</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
    FUNCTION FINISH_REPORT (
        P_FILE_NAME         IN VARCHAR2,
        P_DOCX_REPORT       IN BLOB
    ) RETURN NUMBER IS
        L_TEMP_SESSION      NUMBER := SEQ_SID.NEXTVAL;
        L_DOCX_TEMP_SID     NUMBER := SEQ_SID.NEXTVAL;
    BEGIN
        ACM_LOG.INFO('Finished report: ' || P_FILE_NAME);
        INSERT INTO T_DOCX_REPORT_TEMPORARY (
            SID,
            APEX_SESSION,
            FILENAME,
            REPORT
        ) VALUES (
            L_DOCX_TEMP_SID,
            NVL(APEX_UTIL.GET_SESSION_STATE('SESSION'), L_TEMP_SESSION),
            P_FILE_NAME,
            P_DOCX_REPORT
        );
        RETURN L_DOCX_TEMP_SID;
    END FINISH_REPORT;
/** Summary
<table>
<tr><td>04/26/2021</td><td>KDB</td><td>Created</td></tr>
</table>
*/
    PROCEDURE APPEND_CLASSIFICATION_MARKING (
        P_OBJECT            IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT       IN OUT NOCOPY BLOB
    ) IS
    BEGIN
        DOCX_TEMPLATE.REPLACE_TAG(P_DOCX_REPORT, 'CLASSIFICATION', NVL(TRIM(ACM_CLASSIFICATION.GET_CLASSIFICATION(P_OBJECT)),'CLASSIFICATION NOT SPECIFIED'));
    END APPEND_CLASSIFICATION_MARKING;
/** Summary
*<table>
*<tr><td>06/30/2021</td><td>CPB</td><td>Created</td></tr>
*</table>
*/
    PROCEDURE REPLACE_CLASSIFICATION_BLOCK (
        P_DOCX_REPORT       IN OUT NOCOPY BLOB,
        P_SPEC_SID          IN T_DOCX_REPORT_SPEC.SID%TYPE
    ) IS
        L_DECLASSIFY_DATE   VARCHAR2(200) := '50X1-HUM';
        L_DERIVED_FROM      VARCHAR2(200) := 'I&A-002.1 Dated 20150201';
    BEGIN
        IF P_SPEC_SID IS NOT NULL THEN
            FOR X IN (
                SELECT
                    CLASSIFIED_BY,
                    DERIVED_FROM,
                    DECLASSIFY_ON
                FROM
                    T_DOCX_REPORT_SPEC
                WHERE
                    SID = P_SPEC_SID
            ) LOOP
                DOCX_TEMPLATE.REPLACE_TAG(P_DOCX_REPORT, 'CLASSIFICATION_BLOCK',
                    'Classified By: ' || DOCX_TEMPLATE.SANITIZE_TEXT(UPPER(X.CLASSIFIED_BY)) ||
                    '<w:br/>Derived From: ' || DOCX_TEMPLATE.SANITIZE_TEXT(UPPER(X.DERIVED_FROM)) ||
                    '<w:br/>Declassify On: ' || DOCX_TEMPLATE.SANITIZE_TEXT(TO_CHAR(X.DECLASSIFY_ON, 'YYYYMMDD')), 'N');
            END LOOP;
        ELSE
            FOR X IN (
                SELECT
                    PERSONNEL_NUM
                FROM
                    T_ACM_PERSONNEL
                WHERE
                    OBJ = ACM_CONTEXT.PERSONNEL_OBJ
            ) LOOP
                DOCX_TEMPLATE.REPLACE_TAG(P_DOCX_REPORT, 'CLASSIFICATION_BLOCK',
                    'Classified By: ' || DOCX_TEMPLATE.SANITIZE_TEXT(X.PERSONNEL_NUM) ||
                    '<w:br/>Derived From: ' || DOCX_TEMPLATE.SANITIZE_TEXT(L_DERIVED_FROM) ||
                    '<w:br/>Declassify On: ' || DOCX_TEMPLATE.SANITIZE_TEXT(L_DECLASSIFY_DATE), 'N');
            END LOOP;
        END IF;
    END REPLACE_CLASSIFICATION_BLOCK;
/**
<table>
<tr><td>06/09/2021</td><td>LLSB</td><td>Created</td></tr>
<tr><td>03/30/2022</td><td>CPB</td><td>Updated to utilize xml tables as templates</td></tr>
</table>
*/
    PROCEDURE APPEND_PARTICIPANTS_DEFAULT (
        P_OBJECT                    IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT               IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION             IN VARCHAR2,
        P_CODES_INCLUDE             IN T_STR_ARRAY := T_STR_ARRAY(),
        P_CODES_EXCLUDE             IN T_STR_ARRAY := T_STR_ARRAY(),
        P_NO_RESULT_MSG             IN VARCHAR2 := ''
    ) IS
        L_PARTIC_ENTRY              CLOB;
        L_PARTIC_LIST               CLOB;
        L_INDIV_TEMPLATE            CLOB;
        L_NON_INDIV_TEMPLATE        CLOB;
        L_CODES_INCLUDE_CNT         PLS_INTEGER := 0;
        L_CODES_EXCLUDE_CNT         PLS_INTEGER := 0;
        L_PARTIC_OBJ_TYPE           T_ACM_OBJ_TYPE.OBJ_TYPE_CODE%TYPE;
        L_PARTIC_NAME               VARCHAR2(200) := NULL;
        L_PARTIC_ROLE               T_PARTIC_ROLE_TYPE.DISPLAY%TYPE := NULL;
        L_CLEARANCE                 T_PARTIC_INDIV_REF.DISPLAY%TYPE := NULL;
        L_RESIDENCE                 VARCHAR2(200) := NULL;
        L_ADDRESS                   VARCHAR2(200) := NULL;
        L_STATE                     T_ACM_REF_CTRY_DIVISION.DIV_CODE%TYPE := NULL;
        L_BIRTH_PLACE               VARCHAR2(200) := NULL;
        L_PHONE                     T_PARTIC_CONTACT.VALUE%TYPE := NULL;
        L_EMAIL                     T_PARTIC_CONTACT.VALUE%TYPE := NULL;
        L_SSN                       T_PARTIC_NUMBER.VALUE%TYPE := NULL;
        L_EMPLOY                    VARCHAR2(500) := NULL;
        L_EMPLOY_TYPE               VARCHAR2(4000) := NULL;
        L_PARTIC_CIT                VARCHAR2(2000) := NULL;
        L_COUNTRY                   VARCHAR2(2000) := NULL;
        L_PARTIC_CNT                PLS_INTEGER := 0;
    BEGIN
        -- Count any partic types that should be included or excluded
        L_CODES_INCLUDE_CNT := P_CODES_INCLUDE.COUNT;
        L_CODES_EXCLUDE_CNT := P_CODES_EXCLUDE.COUNT;

        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'INDIV_PARTICIPANT_BLOCK', P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_INDIV_TEMPLATE);
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'NON_INDIV_PARTICIPANT_BLOCK', P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_NON_INDIV_TEMPLATE);

        -- Open list CLOB for all the entryies
        DBMS_LOB.CREATETEMPORARY(L_PARTIC_LIST, FALSE);
        DBMS_LOB.OPEN(L_PARTIC_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_PARTIC_LIST, '<dummy>');


        -- Queries all participants
        FOR X IN (  SELECT
                        TPI.PARTICIPANT,
                        TPRT.DISPLAY ROLE_DISPLAY,
                        TPI.CLEARANCE,
                        TPI.INDIV_DHS_COMP
                    FROM
                        T_PARTIC_INVOLVEMENT TPI
                        INNER JOIN T_PARTIC_ROLE_TYPE TPRT ON TPI.INVOLVEMENT_ROLE = TPRT.SID
                    WHERE
                        TPI.OBJ = P_OBJECT AND ( L_CODES_INCLUDE_CNT = 0 OR ( L_CODES_INCLUDE_CNT > 0 AND TPRT.CODE MEMBER OF P_CODES_INCLUDE ) )
                        AND ( L_CODES_EXCLUDE_CNT = 0 OR ( L_CODES_EXCLUDE_CNT > 0 AND TPRT.CODE NOT MEMBER OF P_CODES_EXCLUDE ) )
                    ORDER BY
                        INVOLVEMENT_ROLE ASC
        )LOOP
            -- Iterate participant count
            L_PARTIC_CNT := L_PARTIC_CNT + 1;

            -- Involvement Role
            L_PARTIC_ROLE := X.ROLE_DISPLAY;

            -- Partic Type
            L_PARTIC_OBJ_TYPE := ACM_OBJ.GET_OBJTYPE_CODE(ACM_OBJ.GET_OBJTYPE(X.PARTICIPANT));

            -- Name
            SELECT TAGLINE INTO L_PARTIC_NAME FROM T_ACM_OBJ WHERE SID = X.PARTICIPANT;

            -- For individual participants
            IF L_PARTIC_OBJ_TYPE = 'PARTIC.INDIV' THEN

                -- Sets up clob for indiv partic
                DBMS_LOB.CREATETEMPORARY (L_PARTIC_ENTRY, FALSE);
                DBMS_LOB.OPEN (L_PARTIC_ENTRY, DBMS_LOB.LOB_READWRITE);
                DBMS_LOB.COPY (L_PARTIC_ENTRY, L_INDIV_TEMPLATE, DBMS_LOB.GETLENGTH (L_INDIV_TEMPLATE));

                -- POB
                FOR Y IN (SELECT CITY, STATE, COUNTRY FROM T_ACM_PLACE WHERE OBJ = X.PARTICIPANT AND PLACE_TYPE = 'PARTIC.INDIV.PLACE_TYPE.BIRTH'
                )LOOP
                    IF Y.STATE IS NOT NULL THEN
                        BEGIN
                            SELECT DIV_CODE INTO L_STATE FROM T_ACM_REF_CTRY_DIVISION WHERE REF_KEY = Y.STATE;
                        EXCEPTION
                            WHEN NO_DATA_FOUND
                            THEN
                                ACM_LOG.ERROR('Birth STATE No Data Found from T_NIBRS_STATE: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                            WHEN TOO_MANY_ROWS
                            THEN
                                ACM_LOG.ERROR('Birth STATE Too Many Rows from T_NIBRS_STATE: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                        END;
                    END IF;
                    IF Y.CITY IS NOT NULL THEN
                        L_BIRTH_PLACE := Y.CITY || ', ';
                    END IF;
                    IF L_STATE IS NOT NULL THEN
                        L_BIRTH_PLACE := L_BIRTH_PLACE || L_STATE || ' ';
                    END IF;
                    IF Y.COUNTRY IS NOT NULL THEN
                        L_BIRTH_PLACE := L_BIRTH_PLACE || Y.COUNTRY;
                    END IF;
                    L_STATE := NULL;
                END LOOP;

                -- Phone number
                FOR Y IN (SELECT VALUE FROM T_PARTIC_CONTACT WHERE OBJ = X.PARTICIPANT AND CONTACT_TYPE IN ('PARTIC.INDIV.CONTACT_TYPE.OFFP', 'PARTIC.INDIV.CONTACT_TYPE.OFFA', 'PARTIC.INDIV.CONTACT_TYPE.MOBP', 'PARTIC.INDIV.CONTACT_TYPE.MOBA', 'PARTIC.INDIV.CONTACT_TYPE.HOMEP', 'PARTIC.INDIV.CONTACT_TYPE.HOMEA') AND ROWNUM = 1
                  ORDER BY CONTACT_TYPE DESC
                )LOOP
                    L_PHONE := Y.VALUE;
                END LOOP;

                -- Email
                FOR Y IN (SELECT VALUE FROM T_PARTIC_CONTACT WHERE OBJ = X.PARTICIPANT AND CONTACT_TYPE IN ('PARTIC.INDIV.CONTACT_TYPE.EMLA', 'PARTIC.INDIV.CONTACT_TYPE.EMLP', 'PARTIC.INDIV.CONTACT_TYPE.EMLPERS', 'PARTIC.INDIV.CONTACT_TYPE.EMLW') AND ROWNUM = 1
                  ORDER BY DECODE(CONTACT_TYPE, 'PARTIC.INDIV.CONTACT_TYPE.EMLP', 1, 'PARTIC.INDIV.CONTACT_TYPE.EMLW', 2, 'PARTIC.INDIV.CONTACT_TYPE.EMLA', 3, 'PARTIC.INDIV.CONTACT_TYPE.EMLPERS', 4)
                )LOOP
                    L_EMAIL := Y.VALUE;
                END LOOP;

                -- Rank, DOB, POB
                FOR Y IN (SELECT SERVICE_RANK, DOB FROM T_PARTIC_INDIV WHERE OBJ = X.PARTICIPANT
                )LOOP
                    DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'RANK_RATE', NVL(Y.SERVICE_RANK, 'No Rank on Record'));
                    DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'DPOB', (NVL(TO_CHAR(Y.DOB, FMT_DATE), 'No DOB on Record,') || ' ' || NVL(L_BIRTH_PLACE, 'No POB on Record')));
                END LOOP;

                -- Clearance at involvement
                FOR Y IN (SELECT DISPLAY FROM T_PARTIC_INDIV_REF WHERE REF_KEY = X.CLEARANCE
                    )LOOP
                        L_CLEARANCE := Y.DISPLAY;
                END LOOP;

                -- Citizenship
                FOR Y IN (  SELECT
                                TPIC.COUNTRY
                            FROM
                                T_PARTIC_INDIV_CITIZENSHIP TPIC
                                INNER JOIN T_PARTIC_INDIV TPI ON TPIC.INDIV_PARTIC = TPI.SID
                            WHERE
                                TPI.OBJ = X.PARTICIPANT
                            ORDER BY
                                TPIC.LAST_MODIFIED_ON DESC
                )LOOP
                  BEGIN
                    SELECT DISPLAY INTO L_COUNTRY FROM T_ACM_REF_CTRY WHERE ISO_3166_1_ALPHA_2 = Y.COUNTRY;
                  EXCEPTION
                    WHEN NO_DATA_FOUND
                    THEN
                      ACM_LOG.ERROR('COUNTRY No Data Found from T_ACM_REF_CTRY: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                    WHEN TOO_MANY_ROWS
                    THEN
                      ACM_LOG.ERROR('COUNTRY Too Many Rows from T_ACM_REF_CTRY: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                  END;
                  IF L_PARTIC_CIT IS NOT NULL THEN
                    L_PARTIC_CIT := L_PARTIC_CIT || ', ';
                  END IF;
                  L_PARTIC_CIT := L_PARTIC_CIT || L_COUNTRY;
                END LOOP;

                -- Sex
                FOR Y IN (  SELECT
                                TPIC.SEX
                            FROM
                                T_PARTIC_INDIV_CHAR TPIC
                                INNER JOIN T_PARTIC_INDIV TPI ON TPIC.INDIV_PARTIC = TPI.SID
                            WHERE
                                TPI.OBJ = X.PARTICIPANT
                            ORDER BY
                                TPIC.LAST_MODIFIED_ON DESC
                )LOOP
                  IF Y.SEX IS NOT NULL THEN
                    CASE Y.SEX
                      WHEN 'PARTIC.INDIV.SEX.FEMALE' THEN DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_SEX', 'Female');
                      WHEN 'PARTIC.INDIV.SEX.MALE' THEN DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_SEX', 'Male');
                      WHEN 'PARTIC.INDIV.SEX.UNKNOWN' THEN DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_SEX', 'Unknown');
                    END CASE;
                  END IF;
                END LOOP;

                -- SSN
                FOR Y IN (SELECT VALUE FROM T_PARTIC_NUMBER WHERE OBJ = X.PARTICIPANT AND NUMBER_TYPE = 'PARTIC.INDIV.NUMBER_TYPE.SSN' AND ROWNUM = 1 ORDER BY LAST_MODIFIED_ON DESC
                )LOOP
                    L_SSN := Y.VALUE;
                END LOOP;

                -- Employment
                FOR Y IN (  SELECT
                                TPN.NONINDIV_NAME,
                                TPIR.DISPLAY
                            FROM
                                T_PARTIC_INDIV_DHS_COMP TPIDC
                                INNER JOIN T_PARTIC_NAME TPN ON TPIDC.DHS_COMPONENT = TPN.OBJ AND TPN.IS_CURRENT = 'Y'
                                LEFT OUTER JOIN T_PARTIC_INDIV_REF TPIR ON TPIR.REF_KEY = TPIDC.EMPLOYMENT_TYPE
                            WHERE
                                TPIDC.SID = X.INDIV_DHS_COMP
                )LOOP
                    L_EMPLOY := Y.NONINDIV_NAME;
                    L_EMPLOY_TYPE := Y.DISPLAY;
                END LOOP;

                -- Residence
                FOR Y IN (SELECT ADDRESS_1, ADDRESS_2, CITY, STATE, POSTAL_CODE, COUNTRY FROM T_ACM_PLACE WHERE OBJ = X.PARTICIPANT
                 AND PLACE_TYPE = 'PARTIC.INDIV.PLACE_TYPE.RES' AND ROWNUM = 1 ORDER BY LAST_MODIFIED_ON DESC
                )LOOP
                    IF Y.ADDRESS_2 IS NOT NULL THEN
                        L_RESIDENCE := Y.ADDRESS_1 || ', ' || Y.ADDRESS_2;
                    ELSE
                        L_RESIDENCE := Y.ADDRESS_1;
                    END IF;
                    L_RESIDENCE := L_RESIDENCE || ', ' || Y.CITY;
                    IF Y.STATE IS NOT NULL THEN
                        BEGIN
                            SELECT DIV_CODE INTO L_STATE FROM T_ACM_REF_CTRY_DIVISION WHERE REF_KEY = Y.STATE;
                        EXCEPTION
                            WHEN NO_DATA_FOUND
                            THEN
                                ACM_LOG.ERROR('Residence STATE No Data Found from T_NIBRS_STATE: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                            WHEN TOO_MANY_ROWS
                            THEN
                                ACM_LOG.ERROR('Residence STATE Too Many Rows from T_NIBRS_STATE: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                        END;
                    END IF;
                    L_RESIDENCE := L_RESIDENCE || ', ' || L_STATE;
                    L_RESIDENCE := L_RESIDENCE || '  ' || Y.POSTAL_CODE;
                    L_RESIDENCE := L_RESIDENCE || '  ' || Y.COUNTRY;
                END LOOP;

                -- Replace the tokens
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_ROLE', L_PARTIC_ROLE);
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'NAME', L_PARTIC_NAME);
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PHONE', NVL(L_PHONE,'No Phone on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'EMAIL', NVL(L_EMAIL,'No Email on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'SSN', NVL(L_SSN, 'No SSN on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'EMPLOY', NVL(L_EMPLOY, 'No Employment on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'EMPLOY_TYPE', NVL(L_EMPLOY_TYPE, 'No Employment Type on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'RESIDENCE', NVL(L_RESIDENCE, 'No Residence on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'CLEARANCE', NVL(L_CLEARANCE, 'No Clearance on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_CITIZENSHIP', NVL(L_PARTIC_CIT, 'No Citizenship on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_SEX', 'Unknown');

                -- Resets values for the next loop iteration
                L_PARTIC_NAME := NULL;
                L_PARTIC_ROLE := NULL;
                L_CLEARANCE := NULL;
                L_RESIDENCE := NULL;
                L_STATE := NULL;
                L_BIRTH_PLACE := NULL;
                L_PHONE := NULL;
                L_EMAIL := NULL;
                L_SSN := NULL;
                L_EMPLOY := NULL;
                L_EMPLOY_TYPE := NULL;
                L_PARTIC_CIT := NULL;
                L_COUNTRY := NULL;

            -- For non-individual participants
            ELSE
                -- Sets up clob for non-indiv partic
                DBMS_LOB.CREATETEMPORARY (L_PARTIC_ENTRY, FALSE);
                DBMS_LOB.OPEN (L_PARTIC_ENTRY, DBMS_LOB.LOB_READWRITE);
                DBMS_LOB.COPY (L_PARTIC_ENTRY, L_NON_INDIV_TEMPLATE, DBMS_LOB.GETLENGTH (L_NON_INDIV_TEMPLATE));

                -- Get the Type of the Non-Individual Participant and other information
                FOR Y IN (  SELECT
                                ( PNT.DISPLAY || ' / ' || PNS.DISPLAY ) NONINDIV_TYPE_DISPLAY,
                                TPC.VALUE AS WEBSITE,
                                TPN.VALUE AS DUNS
                            FROM
                                T_PARTIC_NONINDIV PN
                                LEFT JOIN T_PARTIC_NONINDIV_TYPE PNT ON PNT.SID = PN.NONINDIV_TYPE
                                LEFT JOIN T_PARTIC_NONINDIV_SUBTYPE PNS ON PNS.SID = PN.NONINDIV_SUBTYPE
                                LEFT JOIN T_PARTIC_CONTACT TPC ON TPC.OBJ = PN.OBJ AND TPC.CONTACT_TYPE = 'PARTIC.NONINDIV.CONTACT_TYPE.WEBP'
                                LEFT JOIN T_PARTIC_NUMBER TPN ON TPN.OBJ = PN.OBJ AND TPN.NUMBER_TYPE = 'PARTIC.NONINDIV.NUMBER_TYPE.DUNS'
                            WHERE
                                PN.OBJ = X.PARTICIPANT
                ) LOOP
                    DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'NON_INDIV_TYPE', NVL(Y.NONINDIV_TYPE_DISPLAY,'No Type on Record'));
                    DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'DUNS_NUMBER', NVL(Y.DUNS,'No Duns Number on Record'));
                    DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'WEBSITE', NVL(Y.WEBSITE,'No Website on Record'));
                END LOOP;

                -- Permanent location
                FOR Y IN (  SELECT
                                ADDRESS_1,
                                ADDRESS_2,
                                CITY,
                                STATE,
                                POSTAL_CODE,
                                COUNTRY
                            FROM
                                T_ACM_PLACE
                            WHERE
                                OBJ = X.PARTICIPANT
                                AND PLACE_TYPE = 'PARTIC.NONINDIV.PLACE_TYPE.PERM'
                                AND ROWNUM = 1 ORDER BY LAST_MODIFIED_ON DESC
                ) LOOP

                    IF Y.ADDRESS_1 IS NOT NULL AND Y.ADDRESS_2 IS NOT NULL THEN
                        L_ADDRESS := Y.ADDRESS_1 || ', ' || Y.ADDRESS_2;
                    ELSE
                        L_ADDRESS := Y.ADDRESS_1 || Y.ADDRESS_2;
                    END IF;

                    IF L_ADDRESS IS NOT NULL AND Y.CITY IS NOT NULL THEN
                        L_ADDRESS := L_ADDRESS|| ', ' || Y.CITY;
                    ELSE
                        L_ADDRESS := L_ADDRESS || Y.CITY;
                    END IF;

                    IF Y.STATE IS NOT NULL THEN
                        BEGIN
                            SELECT DIV_CODE INTO L_STATE FROM T_ACM_REF_CTRY_DIVISION WHERE REF_KEY = Y.STATE;
                        EXCEPTION
                            WHEN NO_DATA_FOUND
                            THEN
                                ACM_LOG.ERROR('Address STATE No Data Found from T_NIBRS_STATE: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                            WHEN TOO_MANY_ROWS
                            THEN
                                ACM_LOG.ERROR('Address STATE Too Many Rows from T_NIBRS_STATE: SQLERRM', 'DOCX_REPORT.APPEND_PARTICIPANTS_DEFAULT');
                        END;
                    END IF;

                    IF L_ADDRESS IS NOT NULL AND (Y.STATE || Y.POSTAL_CODE) IS NOT NULL THEN
                        L_ADDRESS := L_ADDRESS|| ', ' || Y.STATE || Y.POSTAL_CODE;
                    ELSE
                        L_ADDRESS := L_ADDRESS || Y.STATE || Y.POSTAL_CODE;
                    END IF;

                    L_ADDRESS := L_ADDRESS || Y.COUNTRY;

                END LOOP;

                -- Replace the tokens
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'PARTIC_ROLE', L_PARTIC_ROLE);
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'NAME', L_PARTIC_NAME);
                DOCX_TEMPLATE.REPLACE_TAG (L_PARTIC_ENTRY, 'ADDRESS', NVL(L_ADDRESS,'No Permanent Address on Record'));

                -- Resets values for the next loop iteration
                L_PARTIC_NAME := NULL;
                L_PARTIC_ROLE := NULL;
                L_ADDRESS := NULL;
            END IF;

            -- Clean up the working copy
            DBMS_LOB.APPEND (L_PARTIC_LIST, L_PARTIC_ENTRY);
            DBMS_LOB.CLOSE (L_PARTIC_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_PARTIC_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_PARTIC_CNT = 0 THEN
            DBMS_LOB.APPEND(L_PARTIC_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_PARTIC_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_PARTIC_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_PARTIC_LIST);
        DBMS_LOB.FREETEMPORARY (L_PARTIC_LIST);

    END APPEND_PARTICIPANTS_DEFAULT;
/**
<table>
<tr><td>08/12/2021</td><td>LM</td><td>Created</td></tr>
<tr><td>04/05/2022</td><td>CPB</td><td>Updated</td></tr>
</table>
*/
    PROCEDURE APPEND_PRODUCTS (
        P_OBJECT                IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_NO_RESULT_MSG         IN VARCHAR2 := ''
    ) IS
        L_ITEM_TEMPLATE         CLOB;
        L_ITEM_ENTRY            CLOB;
        L_ITEM_LIST             CLOB;
        L_PRODUCT_TYPE          T_F_SPLYCHN_REF.DESCRIPTION%TYPE := NULL;
        L_PRODUCT_NAME          T_F_SPLYCHN_PRODUCT.PRODUCT_NAME%TYPE := NULL;
        L_SERIAL_NUM            T_F_SPLYCHN_PRODUCT.SERIAL_NUM%TYPE := NULL;
        L_PRODUCT_COMMENTS      T_F_SPLYCHN_PRODUCT.COMMENTS%TYPE := NULL;
        L_PRODUCT_CNT           PLS_INTEGER := 0;    
    BEGIN
        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_ITEM_TEMPLATE);

        -- Setup CLOB for list
        DBMS_LOB.CREATETEMPORARY (L_ITEM_LIST, FALSE);
        DBMS_LOB.OPEN (L_ITEM_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ITEM_LIST, '<dummy>');

        -- Query and loop through products
        FOR X IN (  SELECT
                        TFSP.OBJ,
                        TFSP.PRODUCT_NAME,
                        TFSR.DESCRIPTION PRODUCT_TYPE,
                        TFSP.SERIAL_NUM,
                        TFSP.COMMENTS
                    FROM
                        T_F_SPLYCHN_PRODUCT TFSP 
                        LEFT JOIN T_F_SPLYCHN_REF TFSR ON TFSR.SID = TFSP.PRODUCT_TYPE
                    WHERE
                        TFSP.OBJ = P_OBJECT
        ) LOOP
            -- Iterate counter
            L_PRODUCT_CNT := L_PRODUCT_CNT + 1;

            -- Setup clob for entry
            DBMS_LOB.CREATETEMPORARY (L_ITEM_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ITEM_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ITEM_ENTRY, L_ITEM_TEMPLATE, DBMS_LOB.GETLENGTH (L_ITEM_TEMPLATE));

            -- Product Type
            L_PRODUCT_TYPE := X.PRODUCT_TYPE;
            -- Product Name
            L_PRODUCT_NAME := X.PRODUCT_NAME;
            -- Serial num
            L_SERIAL_NUM := X.SERIAL_NUM;
            -- Comments
            L_PRODUCT_COMMENTS := X.COMMENTS;

            -- Replace the tokens
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PRODUCT_TYPE', L_PRODUCT_TYPE);
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PRODUCT_NAME', L_PRODUCT_NAME);
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'SERIAL_NUM', NVL(L_SERIAL_NUM,'No Serial Number on Record'));
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PRODUCT_COMMENTS', NVL(L_PRODUCT_COMMENTS,'No Comments on Record'));

            -- Resets values for the next loop iteration
            L_PRODUCT_TYPE := NULL;
            L_PRODUCT_NAME := NULL;
            L_SERIAL_NUM := NULL;
            L_PRODUCT_COMMENTS := NULL;

            -- Clean up the working copy
            DBMS_LOB.APPEND (L_ITEM_LIST, L_ITEM_ENTRY);
            DBMS_LOB.CLOSE (L_ITEM_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ITEM_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_PRODUCT_CNT = 0 THEN
            DBMS_LOB.APPEND(L_ITEM_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ITEM_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_ITEM_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ITEM_LIST);
        DBMS_LOB.FREETEMPORARY (L_ITEM_LIST);

    END APPEND_PRODUCTS;  
/**
<table>
<tr><td>08/12/2021</td><td>LM</td><td>Created</td></tr>
<tr><td>04/05/2022</td><td>CPB</td><td>Updated</td></tr>
</table>
*/
    PROCEDURE APPEND_COMPANY_BG_DETAILS (
        P_OBJECT                IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_NO_RESULT_MSG         IN VARCHAR2 := ''
    ) IS
        L_ITEM_TEMPLATE         CLOB;
        L_ITEM_ENTRY            CLOB;
        L_ITEM_LIST             CLOB;
        L_PRODUCT_TYPE          T_F_SPLYCHN_REF.DESCRIPTION%TYPE := NULL;
        L_NARRATIVE_TYPE        T_F_SPLYCHN_REF.DESCRIPTION%TYPE := NULL;
        L_NARRATIVE             T_F_SUPPLY_CHAIN_NARRATIVES.NARRATIVE%TYPE := NULL;
        L_DETAIL_CNT            PLS_INTEGER := 0;
    BEGIN    
        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_ITEM_TEMPLATE);

        -- Setup CLOB for list
        DBMS_LOB.CREATETEMPORARY (L_ITEM_LIST, FALSE);
        DBMS_LOB.OPEN (L_ITEM_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ITEM_LIST, '<dummy>');

        -- Query and loop through company details
        FOR X IN (  SELECT
                        TFSN.OBJ,
                        TFSN.NARRATIVE,
                        TFSR.DESCRIPTION NARRATIVE_TYPE,
                        TFSR.SEQ SEQUENCE
                    FROM
                        T_F_SUPPLY_CHAIN_NARRATIVES TFSN
                        LEFT OUTER JOIN T_F_SPLYCHN_REF TFSR ON TFSR.SID = TFSN.NARRATIVE_TYPE
                    WHERE
                        TFSN.OBJ = P_OBJECT
                        ORDER BY SEQUENCE ASC
        )LOOP
            -- Iterate counter
            L_DETAIL_CNT := L_DETAIL_CNT + 1;

            -- Set up clob for entry
            DBMS_LOB.CREATETEMPORARY (L_ITEM_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ITEM_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ITEM_ENTRY, L_ITEM_TEMPLATE, DBMS_LOB.GETLENGTH (L_ITEM_TEMPLATE));

            -- Narrative Type
            L_NARRATIVE_TYPE := X.NARRATIVE_TYPE;

            -- Narrative
            L_NARRATIVE := X.NARRATIVE;

            -- Replace the tokens
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'NARRATIVE_TYPE', L_NARRATIVE_TYPE);
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'NARRATIVE', L_NARRATIVE);

            -- Resets values for the next loop iteration
            L_NARRATIVE := NULL;
            L_NARRATIVE_TYPE := NULL;

            -- Clean up the working copy
            DBMS_LOB.APPEND (L_ITEM_LIST, L_ITEM_ENTRY);
            DBMS_LOB.CLOSE (L_ITEM_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ITEM_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_DETAIL_CNT = 0 THEN
            DBMS_LOB.APPEND(L_ITEM_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ITEM_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_ITEM_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ITEM_LIST);
        DBMS_LOB.FREETEMPORARY (L_ITEM_LIST);
    END APPEND_COMPANY_BG_DETAILS;    
/** Summary
<table>
<tr><td>07/12/2021</td><td>CPB</td><td>Created</td></tr>
</table>
*/
    PROCEDURE APPEND_LOCATION (
        P_OBJECT            IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT       IN OUT NOCOPY BLOB,
        P_TOKEN_NAME        IN VARCHAR2
    ) IS
        L_DATA_VALUE        VARCHAR2(200);
        L_STATE             VARCHAR2(200);
        L_COUNTRY           VARCHAR2(200);
    BEGIN
        FOR X IN (  SELECT
                        ADDRESS_1,
                        ADDRESS_2,
                        CITY,
                        STATE,
                        POSTAL_CODE,
                        COUNTRY
                    FROM
                        T_ACM_PLACE
                    WHERE
                        OBJ = P_OBJECT
        ) LOOP
            IF X.STATE IS NOT NULL THEN
                BEGIN
                    SELECT DIVISION_DISPLAY INTO L_STATE FROM T_ACM_REF_CTRY_DIVISION WHERE REF_KEY = X.STATE;
                EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                        ACM_LOG.ERROR('STATE No Data Found from T_ACM_REF_CTRY_DIVISION: SQLERRM', 'DOCX_REPORT.APPEND_LOCATION');
                    WHEN TOO_MANY_ROWS THEN
                        ACM_LOG.ERROR('STATE Too Many Rows from T_ACM_REF_CTRY_DIVISION: SQLERRM', 'DOCX_REPORT.APPEND_LOCATION');
                END;
            END IF;
            IF X.COUNTRY IS NOT NULL THEN
                BEGIN
                    SELECT DISPLAY INTO L_COUNTRY FROM T_ACM_REF_CTRY WHERE ISO_3166_1_ALPHA_2 = X.COUNTRY;
                EXCEPTION
                    WHEN NO_DATA_FOUND THEN
                        ACM_LOG.ERROR('COUNTRY No Data Found from T_ACM_REF_CTRY: SQLERRM', 'DOCX_REPORT.APPEND_LOCATION');
                    WHEN TOO_MANY_ROWS THEN
                        ACM_LOG.ERROR('COUNTRY Too Many Rows from T_ACM_REF_CTRY: SQLERRM', 'DOCX_REPORT.APPEND_LOCATION');
                END;
            END IF;
            L_DATA_VALUE := X.ADDRESS_1 || ', ';
            IF X.ADDRESS_2 IS NOT NULL THEN
                L_DATA_VALUE := L_DATA_VALUE || X.ADDRESS_2 || ', ';
            END IF;
            IF X.CITY IS NOT NULL THEN
                L_DATA_VALUE := L_DATA_VALUE || X.CITY || ', ';
            END IF;
            IF L_STATE IS NOT NULL THEN
                L_DATA_VALUE := L_DATA_VALUE || L_STATE || '  ';
            END IF;
            IF X.POSTAL_CODE IS NOT NULL THEN
                L_DATA_VALUE := L_DATA_VALUE || X.POSTAL_CODE || '  ';
            END IF;
            IF L_COUNTRY IS NOT NULL THEN
                L_DATA_VALUE := L_DATA_VALUE || L_COUNTRY;
            END IF;
            DOCX_TEMPLATE.REPLACE_TAG (P_DOCX_REPORT, P_TOKEN_NAME, L_DATA_VALUE, 'N');
        END LOOP;
        DOCX_TEMPLATE.REPLACE_TAG (P_DOCX_REPORT, P_TOKEN_NAME, 'No location on record', 'N');
    END APPEND_LOCATION;
/** Summary
*<table>
*<tr><td>02/13/2020</td><td>LLSB</td><td>Initial addition to the Baseline</td></tr>
*<tr><td>03/30/2022</td><td>CPB</td><td>Updated to utilize xml tables as templates</td></tr>
*</table>
*/
    PROCEDURE APPEND_ATTACHMENTS_DEFAULT (
        P_OBJECT                    IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT               IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION             IN VARCHAR2,
        P_CODES_INCLUDE             IN T_STR_ARRAY := T_STR_ARRAY(),
        P_CODES_EXCLUDE             IN T_STR_ARRAY := T_STR_ARRAY(),
        P_SPEC_SID                  IN T_DOCX_REPORT_SPEC.SID%TYPE := NULL,
        P_NO_RESULT_MSG             IN VARCHAR2 := ''
    ) IS
        L_ATTACH_ENTRY_TEMPLATE     CLOB;
        L_ATTACH_ENTRY              CLOB;
        L_ATTACH_LIST               CLOB;
        L_CODES_INCLUDE_CNT         PLS_INTEGER := 0;
        L_CODES_EXCLUDE_CNT         PLS_INTEGER := 0;
        L_ATTACH_CNT                PLS_INTEGER := 0;
        L_SPEC_PICK_ATTACH          T_DOCX_REPORT_TYPE.PICK_ATTACHMENTS%TYPE := 'N';
    BEGIN
        -- Count to see if any attachment type codes are to be included or excluded
        L_CODES_INCLUDE_CNT := P_CODES_INCLUDE.COUNT;
        L_CODES_EXCLUDE_CNT := P_CODES_EXCLUDE.COUNT;

        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_ATTACH_ENTRY_TEMPLATE);

        -- Setup CLOB for attachment list
        DBMS_LOB.CREATETEMPORARY (L_ATTACH_LIST, FALSE);
        DBMS_LOB.OPEN (L_ATTACH_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ATTACH_LIST, '<dummy>');


        -- Determine if notes can be picked based on the report spec
        IF P_SPEC_SID IS NOT NULL THEN
            SELECT PICK_ATTACHMENTS
            INTO L_SPEC_PICK_ATTACH
            FROM T_DOCX_REPORT_TYPE
            WHERE SID = (SELECT REPORT_TYPE FROM T_DOCX_REPORT_SPEC WHERE SID = P_SPEC_SID);
        END IF;

        -- Query for attachments
        FOR X IN (  SELECT
                        A.FILENAME,
                        A.CLASSIFICATION,
                        A.ATCH_DESCRIPTION
                    FROM
                        T_ACM_ATTACHMENT A
                        INNER JOIN T_ACM_ATTACHMENT_TYPE AT ON A.ATCH_TYPE = AT.SID
                        LEFT JOIN T_DOCX_REPORT_SPEC_ATTACH SA ON A.SID = SA.ATTACHMENT
                    WHERE
                        ( ( L_SPEC_PICK_ATTACH = 'Y' AND SA.SPEC = P_SPEC_SID ) OR ( L_SPEC_PICK_ATTACH = 'N' AND A.OBJ = P_OBJECT ) )
                        AND ( L_CODES_INCLUDE_CNT = 0 OR ( L_CODES_INCLUDE_CNT > 0 AND AT.CODE MEMBER OF P_CODES_INCLUDE ) )
                        AND ( L_CODES_EXCLUDE_CNT = 0 OR ( L_CODES_EXCLUDE_CNT > 0 AND AT.CODE NOT MEMBER OF P_CODES_EXCLUDE ) )
                    ORDER BY
                        A.CREATED_ON ASC
        )LOOP
            -- Iterate attachment count
            L_ATTACH_CNT := L_ATTACH_CNT + 1;

            -- Setup CLOB for attachment entry
            DBMS_LOB.CREATETEMPORARY (L_ATTACH_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ATTACH_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ATTACH_ENTRY, L_ATTACH_ENTRY_TEMPLATE, DBMS_LOB.GETLENGTH (L_ATTACH_ENTRY_TEMPLATE));

            -- Replace tags in attachment entry CLOB
            IF X.FILENAME IS NOT NULL THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ATTACH_ENTRY, 'FILENAME', (L_ATTACH_CNT || '. ' || X.CLASSIFICATION ||
                    ' ' || X.FILENAME));
            ELSE
                DOCX_TEMPLATE.REPLACE_TAG (L_ATTACH_ENTRY, 'FILENAME', (L_ATTACH_CNT || '. ' || X.CLASSIFICATION ||
                    ' ' || X.ATCH_DESCRIPTION));
            END IF;

            -- Append attachment entry CLOB to the attachment list CLOB, then clean up attachment entry CLOB
            DBMS_LOB.APPEND (L_ATTACH_LIST, L_ATTACH_ENTRY);
            DBMS_LOB.CLOSE (L_ATTACH_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ATTACH_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_ATTACH_CNT = 0 THEN
            DBMS_LOB.APPEND(L_ATTACH_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ATTACH_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_ATTACH_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ATTACH_LIST);
        DBMS_LOB.FREETEMPORARY (L_ATTACH_LIST);

    END APPEND_ATTACHMENTS_DEFAULT;
/** Summary
*<table>
*<tr><td>02/13/2020</td><td>LLSB</td><td>Initial addition to the Baseline</td></tr>
*<tr><td>07/25/2020</td><td>LLSB</td><td>Task 12103 Updates</td></tr>
*<tr><td>03/30/2022</td><td>CPB</td><td>Updated to utilize xml tables as templates</td></tr>
*</table>
*/
    PROCEDURE APPEND_NOTES_DEFAULT (
        P_OBJECT                IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_CODES_INCLUDE         IN T_STR_ARRAY := T_STR_ARRAY(),
        P_CODES_EXCLUDE         IN T_STR_ARRAY := T_STR_ARRAY(),
        P_SPEC_SID              IN T_DOCX_REPORT_SPEC.SID%TYPE := NULL,
        P_NO_RESULT_MSG         IN VARCHAR2 := ''
    ) IS
        L_NOTE_ENTRY_TEMPLATE   CLOB;
        L_NOTE_ENTRY            CLOB;
        L_NOTE_LIST             CLOB;
        L_CODES_INCLUDE_CNT     PLS_INTEGER := 0;
        L_CODES_EXCLUDE_CNT     PLS_INTEGER := 0;
        L_NOTE_TYPE             T_ACM_NOTE_TYPE.DISPLAY%TYPE;
        L_NOTE_CNT              PLS_INTEGER := 0;
        L_SPEC_PICK_NOTES       T_DOCX_REPORT_TYPE.PICK_NOTES%TYPE := 'N';
    BEGIN
        -- Count to see if any note type codes are to be included or excluded
        L_CODES_INCLUDE_CNT := P_CODES_INCLUDE.COUNT;
        L_CODES_EXCLUDE_CNT := P_CODES_EXCLUDE.COUNT;

        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_NOTE_ENTRY_TEMPLATE);

        -- Setup CLOB for attachment list
        DBMS_LOB.CREATETEMPORARY (L_NOTE_LIST, FALSE);
        DBMS_LOB.OPEN (L_NOTE_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_NOTE_LIST, '<dummy>');

        -- Determine if notes can be picked based on the report spec
        IF P_SPEC_SID IS NOT NULL THEN
            SELECT PICK_NOTES
            INTO L_SPEC_PICK_NOTES
            FROM T_DOCX_REPORT_TYPE
            WHERE SID = (SELECT REPORT_TYPE FROM T_DOCX_REPORT_SPEC WHERE SID = P_SPEC_SID);
        END IF;

        -- Query for notes
        FOR X IN (  SELECT
                        NT.CODE,
                        NT.DISPLAY,
                        N.NOTE_TYPE,
                        N.CREATED_ON,
                        N.CREATED_BY,
                        N.NOTE_TEXT
                    FROM
                        T_ACM_NOTE N
                        INNER JOIN T_ACM_NOTE_TYPE NT ON N.NOTE_TYPE = NT.SID
                        LEFT JOIN T_DOCX_REPORT_SPEC_NOTE SN ON N.SID = SN.NOTE
                    WHERE
                        ( ( L_SPEC_PICK_NOTES = 'Y' AND SN.SPEC = P_SPEC_SID ) OR ( L_SPEC_PICK_NOTES = 'N' AND N.OBJ = P_OBJECT ) )
                        AND ( L_CODES_INCLUDE_CNT = 0 OR ( L_CODES_INCLUDE_CNT > 0 AND NT.CODE MEMBER OF P_CODES_INCLUDE ) )
                        AND ( L_CODES_EXCLUDE_CNT = 0 OR ( L_CODES_EXCLUDE_CNT > 0 AND NT.CODE NOT MEMBER OF P_CODES_EXCLUDE ) )
                    ORDER BY
                        N.CREATED_ON ASC
        )LOOP
            -- Iterate note count
            L_NOTE_CNT := L_NOTE_CNT + 1;
            -- Error handling for missing note type display
            BEGIN
                SELECT DISPLAY INTO L_NOTE_TYPE FROM T_ACM_NOTE_TYPE WHERE SID = X.NOTE_TYPE;
                    EXCEPTION
                        WHEN NO_DATA_FOUND THEN
                          ACM_LOG.ERROR('NOTE_TYPE No Data Found for DISPLAY from T_ACM_NOTE_TYPE: SQLERRM', 'DOCX_REPORT.APPEND_NOTES_DEFAULT');
                          L_NOTE_TYPE := 'Work Note';
                        WHEN TOO_MANY_ROWS THEN
                          ACM_LOG.ERROR('NOTE_TYPE Too Many Rows for DISPLAY from T_ACM_NOTE_TYPE: SQLERRM', 'DOCX_REPORT.APPEND_NOTES_DEFAULT');
                          L_NOTE_TYPE := 'Work Note';
            END;
            -- Setup CLOB for note entry
            DBMS_LOB.CREATETEMPORARY (L_NOTE_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_NOTE_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_NOTE_ENTRY, L_NOTE_ENTRY_TEMPLATE, DBMS_LOB.GETLENGTH (L_NOTE_ENTRY_TEMPLATE));

            -- Replace tags in note entry CLOB
            DOCX_TEMPLATE.REPLACE_TAG (L_NOTE_ENTRY, 'NOTE_TYPE', L_NOTE_CNT || '. ' || L_NOTE_TYPE);
            DOCX_TEMPLATE.REPLACE_TAG (L_NOTE_ENTRY, 'NOTE_DATE', TO_CHAR(X.CREATED_ON, FMT_DATE));
            DOCX_TEMPLATE.REPLACE_TAG (L_NOTE_ENTRY, 'NOTE_AUTHOR', X.CREATED_BY);
            DOCX_TEMPLATE.REPLACE_TAG (L_NOTE_ENTRY, 'NOTE_TEXT', X.NOTE_TEXT);

            -- Append note entry CLOB to the note list CLOB, then clean up note entry CLOB
            DBMS_LOB.APPEND (L_NOTE_LIST, L_NOTE_ENTRY);
            DBMS_LOB.CLOSE (L_NOTE_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_NOTE_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_NOTE_CNT = 0 THEN
            DBMS_LOB.APPEND(L_NOTE_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_NOTE_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_NOTE_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_NOTE_LIST);
        DBMS_LOB.FREETEMPORARY (L_NOTE_LIST);

    END APPEND_NOTES_DEFAULT;
/** Summary
*<table>
*<tr><td>10/22/2019</td><td>LLSB</td><td>Initial addition to the baseline</td></tr>
*<tr><td>07/25/2020</td><td>LLSB</td><td>Task 12103 Updates</td></tr>
*</table>
*/  
    PROCEDURE APPEND_ACTIVITIES_DEFAULT(
        P_OBJECT                IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_CODES_INCLUDE         IN T_STR_ARRAY := T_STR_ARRAY(),
        P_CODES_EXCLUDE         IN T_STR_ARRAY := T_STR_ARRAY(),
        P_SPEC_SID              IN NUMBER,
        P_NO_RESULT_MSG         IN VARCHAR2 := ''
    ) IS  
        L_ATTACH                VARCHAR2(4000) := NULL;
        L_ACT_ENTRY             CLOB;
        L_ACT_ENTRY_TEMPLATE    CLOB;
        L_ACT_LIST              CLOB;
        L_ACT_TYPE              T_ACM_OBJ_TYPE.OBJ_TYPE_CODE%TYPE := NULL;
        L_OWNING_UNIT           V_CURR_OBJ_OWNING_UNIT.SH_OBJ%TYPE;
        L_CODES_INCLUDE_CNT     PLS_INTEGER := 0;
        L_CODES_EXCLUDE_CNT     PLS_INTEGER := 0;
        L_ACT_CNT               PLS_INTEGER := 0;
        L_ATTACH_CNT            PLS_INTEGER := 0;
        L_SPEC_PICK_ACT         T_DOCX_REPORT_TYPE.PICK_ACTIVITIES%TYPE := 'N';
    BEGIN
        -- Count to see if any note type codes are to be included or excluded
        L_CODES_INCLUDE_CNT := P_CODES_INCLUDE.COUNT;
        L_CODES_EXCLUDE_CNT := P_CODES_EXCLUDE.COUNT;

        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_ACT_ENTRY_TEMPLATE);

        -- Setup CLOB for attachment list
        DBMS_LOB.CREATETEMPORARY (L_ACT_LIST, FALSE);
        DBMS_LOB.OPEN (L_ACT_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ACT_LIST, '<dummy>');


        -- Determine if notes can be picked based on the report spec
        IF P_SPEC_SID IS NOT NULL THEN
            SELECT PICK_ACTIVITIES
            INTO L_SPEC_PICK_ACT
            FROM T_DOCX_REPORT_TYPE
            WHERE SID = (SELECT REPORT_TYPE FROM T_DOCX_REPORT_SPEC WHERE SID = P_SPEC_SID);
        END IF;

        -- Query for activities
        FOR X IN (  SELECT
                        TAO.SID P_ACT_OBJ,
                        TAOT.DISPLAY,
                        VADN.ACTIVITY_DATE
                    FROM
                        T_ACM_OBJ TAO
                        INNER JOIN T_ACM_OBJ_TYPE TAOT ON TAO.OBJ_TYPE = TAOT.SID AND TAOT.OBJ_ROOT = 'ACT'
                        LEFT JOIN V_ACTIVITY_DATE_NARRATIVE VADN ON TAO.SID = VADN.ACTIVITY_OBJ
                        LEFT JOIN T_DOCX_REPORT_SPEC_ACT TDRSA ON TAO.SID = TDRSA.ACTIVITY
                    WHERE
                        TAO.SID IN (    SELECT
                                            TAA.FROM_OBJ ACT_OBJ
                                        FROM
                                            T_ACM_ASSOCIATION TAA
                                        WHERE
                                            TAA.TO_OBJ = P_OBJECT
                                    UNION ALL
                                        SELECT
                                            TAA.TO_OBJ ACT_OBJ
                                        FROM
                                            T_ACM_ASSOCIATION TAA
                                        WHERE
                                            TAA.FROM_OBJ = P_OBJECT)
                        AND ( L_SPEC_PICK_ACT = 'N' OR ( L_SPEC_PICK_ACT = 'Y' AND TDRSA.SPEC = P_SPEC_SID ) )
                        AND ( L_CODES_INCLUDE_CNT = 0 OR ( L_CODES_INCLUDE_CNT > 0 AND TAOT.OBJ_TYPE_CODE MEMBER OF P_CODES_INCLUDE ) )
                        AND ( L_CODES_EXCLUDE_CNT = 0 OR ( L_CODES_EXCLUDE_CNT > 0 AND TAOT.OBJ_TYPE_CODE NOT MEMBER OF P_CODES_EXCLUDE ) )
                    ORDER BY
                        TAOT.DISPLAY,VADN.ACTIVITY_DATE
        )LOOP
            -- Iterate activity count
            L_ACT_CNT := L_ACT_CNT + 1;

            -- Setup CLOB for activity entry
            DBMS_LOB.CREATETEMPORARY (L_ACT_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ACT_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ACT_ENTRY, L_ACT_ENTRY_TEMPLATE, DBMS_LOB.GETLENGTH (L_ACT_ENTRY_TEMPLATE));

            -- Check if the activity type has changed
            IF L_ACT_TYPE <> X.DISPLAY OR L_ACT_TYPE IS NULL THEN
                -- Replace activity type token
                DOCX_TEMPLATE.REPLACE_TAG(L_ACT_ENTRY, 'ACTIVITY_TYPE', CHR(13) || UPPER(X.DISPLAY) ||':');
                -- Set the current activity type
                L_ACT_TYPE := X.DISPLAY;
            ELSE
                -- Replace activity type token
                DOCX_TEMPLATE.REPLACE_TAG(L_ACT_ENTRY, 'ACTIVITY_TYPE', '');
            END IF;

            -- Activity entry details
            FOR Y IN (  SELECT
                            ACTIVITY_OBJ,
                            ACTIVITY_DATE,
                            ACTIVITY_ID,
                            TITLE,
                            NARRATIVE,
                            ACTIVITY_METHOD,
                            ACTIVITY_AGENT
                        FROM
                            V_ACTIVITY_DATE_NARRATIVE
                        WHERE
                            ACTIVITY_OBJ = X.P_ACT_OBJ
            )LOOP
                -- Replace tags in activity entry CLOB
                DOCX_TEMPLATE.REPLACE_TAG (L_ACT_ENTRY, 'ACTIVITY_TITLE', NVL((Y.TITLE || ' (' || Y.ACTIVITY_ID || ')'), 'No Title on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_ACT_ENTRY, 'ACTIVITY_DATE', NVL(TO_CHAR(Y.ACTIVITY_DATE, FMT_DATE), 'No Date on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_ACT_ENTRY, 'ACT_NARRATIVE', NVL(Y.NARRATIVE, 'No Narrative on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_ACT_ENTRY, 'ACTIVITY_METHOD', NVL(Y.ACTIVITY_METHOD,'No Method on Record'));
                DOCX_TEMPLATE.REPLACE_TAG (L_ACT_ENTRY, 'ACTIVITY_AGENT', NVL(Y.ACTIVITY_AGENT,'No Lead on Record'));

            END LOOP;

            -- Activity entry attachments
            FOR Y IN (  SELECT
                            A.FILENAME,
                            A.CLASSIFICATION,
                            A.ATCH_DESCRIPTION
                        FROM
                            T_ACM_ATTACHMENT A
                            LEFT JOIN T_DOCX_REPORT_SPEC_ATTACH ASPEC ON A.SID = ASPEC.ATTACHMENT
                        WHERE
                            A.OBJ = X.P_ACT_OBJ
                            AND (P_SPEC_SID IS NULL OR (P_SPEC_SID IS NOT NULL
                                                        AND (ASPEC.ATCH_OBJ = X.P_ACT_OBJ
                                                             AND ASPEC.SPEC = P_SPEC_SID)
                                                        )
                                )
            )LOOP
                -- Iterate attachment counter
                L_ATTACH_CNT := L_ATTACH_CNT + 1;

                IF L_ATTACH_CNT > 1 THEN
                    IF Y.FILENAME IS NOT NULL THEN
                        L_ATTACH :=  L_ATTACH || CHR(13) || '     ' || L_ATTACH_CNT || '. ' || Y.CLASSIFICATION || ' ' || Y.FILENAME;
                    ELSE
                        L_ATTACH :=  L_ATTACH || CHR(13) || '     ' || L_ATTACH_CNT || '. ' || Y.CLASSIFICATION || ' ' || Y.ATCH_DESCRIPTION;
                    END IF;
                ELSE
                    IF Y.FILENAME IS NOT NULL THEN
                        L_ATTACH :=  L_ATTACH || L_ATTACH_CNT || '. ' || Y.CLASSIFICATION || ' ' || Y.FILENAME;
                    ELSE
                        L_ATTACH :=  L_ATTACH || L_ATTACH_CNT || '. ' || Y.CLASSIFICATION || ' ' || Y.ATCH_DESCRIPTION;
                    END IF;
                END IF;
            END LOOP;

            DOCX_TEMPLATE.REPLACE_TAG (L_ACT_ENTRY, 'ACT_ATTACHMENT', NVL(L_ATTACH, 'No Attachments on Record'));

            -- Reset values
            L_ATTACH := NULL;
            L_ATTACH_CNT := 0;

            -- Appends activity entry to activity list
            DBMS_LOB.APPEND (L_ACT_LIST, L_ACT_ENTRY);
            DBMS_LOB.CLOSE (L_ACT_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ACT_ENTRY);

        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_ACT_CNT = 0 THEN
            DBMS_LOB.APPEND(L_ACT_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ACT_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_ACT_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ACT_LIST);
        DBMS_LOB.FREETEMPORARY (L_ACT_LIST);

    END APPEND_ACTIVITIES_DEFAULT;   
/** Summary
<table>
<tr><td>04/23/2021</td><td>KDB</td><td>Created, moved all the report generation of associated files to here. Called with 2 params, the file sid, and the docx blob.</td></tr>
</table>
*/
    PROCEDURE APPEND_ASSOCIATED_FILES_DEFAULT (
        P_OBJECT                IN T_ACM_OBJ.SID%TYPE,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_NO_RESULT_MSG         IN VARCHAR2
    ) IS
        L_FILE_ENTRY_TEMPLATE   CLOB;
        L_FILE_ENTRY            CLOB;
        L_FILE_LIST             CLOB;
        L_FILE_TYPE_DESC        VARCHAR2(2000);
        L_FILE_TITLE            VARCHAR2(2000);
        L_FILE_SIDS             T_STR_ARRAY := ACM_FILE.GET_ASSOC_FILES(P_OBJECT);
        L_FILE_CNT              PLS_INTEGER := 0;
    BEGIN
        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_FILE_ENTRY_TEMPLATE);

        -- Setup CLOB for attachment list
        DBMS_LOB.CREATETEMPORARY (L_FILE_LIST, FALSE);
        DBMS_LOB.OPEN (L_FILE_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_FILE_LIST, '<dummy>');


        FOR X IN 1..L_FILE_SIDS.COUNT LOOP
            -- Iterate the file count
            L_FILE_CNT := L_FILE_CNT + 1;

            ACM_LOG.INFO('Number of loops in APPEND_ASSOCIATED_FILES: ' || L_FILE_CNT);

            -- Template
            DBMS_LOB.CREATETEMPORARY(L_FILE_ENTRY, FALSE);
            DBMS_LOB.OPEN(L_FILE_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY(L_FILE_ENTRY, L_FILE_ENTRY_TEMPLATE, DBMS_LOB.GETLENGTH(L_FILE_ENTRY_TEMPLATE));

            -- Get File Type Details
            L_FILE_TYPE_DESC := ACM_OBJ.LOOKUP_OBJTYPE_DISP(ACM_OBJ.GET_OBJTYPE(L_FILE_SIDS(X)));
            L_FILE_TITLE := ACM_OBJ.GET_TAGLINE(L_FILE_SIDS(X));

            -- Replace Tokens
            DOCX_TEMPLATE.REPLACE_TAG(L_FILE_ENTRY, 'NAME', L_FILE_TYPE_DESC || ' - ' || ACM_OBJ.GET_ID(L_FILE_SIDS(X)));
            DOCX_TEMPLATE.REPLACE_TAG(L_FILE_ENTRY, 'TITLE', L_FILE_TITLE);

            -- Append to the List
            DBMS_LOB.APPEND(L_FILE_LIST, L_FILE_ENTRY);
            DBMS_LOB.CLOSE(L_FILE_ENTRY);
            DBMS_LOB.FREETEMPORARY(L_FILE_ENTRY);

        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_FILE_CNT = 0 THEN
            DBMS_LOB.APPEND(L_FILE_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_FILE_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_FILE_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_FILE_LIST);
        DBMS_LOB.FREETEMPORARY (L_FILE_LIST);

    END APPEND_ASSOCIATED_FILES_DEFAULT;
/** Summary
*<table>
*<tr><td>02/27/2019</td><td>LM</td><td>Added Comments to satisfy documentation generation.</td></tr>
*<tr><td>07/25/2020</td><td>LLSB</td><td>Task 12103 Updates</td></tr>
*</table>
*/
    FUNCTION DOWNLOAD_SPEC_ATTACHMENTS(
        P_OBJECT        IN VARCHAR2
    ) RETURN NUMBER IS
        L_ZIP           BLOB;
        L_COUNT         NUMBER := 0;
        L_SPEC_ROW      T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Get spec if it exists
        GET_SPEC(P_OBJECT => P_OBJECT, P_REPORT_CODE => 'DOWNLOAD_ATTACH', P_SPEC_ROW => L_SPEC_ROW);

        -- Query and add spec attachments
        SYS.DBMS_LOB.CREATETEMPORARY(LOB_LOC => L_ZIP, CACHE => TRUE, DUR => DBMS_LOB.SESSION);
        FOR ATTACHMENT IN ( SELECT
                                A.FILENAME AS FILENAME,
                                A.ATCH_CONTENT AS CONTENT
                            FROM
                                T_ACM_ATTACHMENT A
                            WHERE
                                A.SID IN (  SELECT
                                                ATTACHMENT
                                            FROM
                                                T_DOCX_REPORT_SPEC_ATTACH
                                            WHERE
                                                SPEC = L_SPEC_ROW.SID
                                          )
                                AND A.STORAGE_LOC_TYPE = 'DB'
        ) LOOP
        APEX_ZIP.ADD_FILE(L_ZIP, ATTACHMENT.FILENAME, ATTACHMENT.CONTENT);
        L_COUNT := 1;
        END LOOP;
        APEX_ZIP.FINISH(L_ZIP);

        -- Insert the new blob report into the table to download
        IF L_COUNT = 1 THEN
            RETURN FINISH_REPORT( P_FILE_NAME => TO_CHAR(SYSDATE, 'yyyymmdd') || '-attachments.zip', P_DOCX_REPORT => L_ZIP );
        END IF;
    END DOWNLOAD_SPEC_ATTACHMENTS;
/** Summary
*<table>
*<tr><td>07/29/2020</td><td>LLSB</td><td>Task 12103 Updates</td></tr>
*</table>
*/
    FUNCTION DOWNLOAD_ALL_ATTACHMENTS(
        P_OBJECT        IN VARCHAR2
    ) RETURN NUMBER IS
        L_SID           VARCHAR2(200);
        L_ZIP           BLOB;
        L_COUNT         NUMBER := 0;
    BEGIN
        -- Query attachments to object and add to BLOB
        SYS.DBMS_LOB.CREATETEMPORARY(LOB_LOC => L_ZIP, CACHE => TRUE, DUR => DBMS_LOB.SESSION);
        FOR ATTACHMENT IN ( SELECT
                                A.FILENAME AS FILENAME,
                                A.ATCH_CONTENT AS CONTENT
                            FROM
                                T_ACM_ATTACHMENT A
                            WHERE
                                A.OBJ = P_OBJECT
                                AND A.STORAGE_LOC_TYPE = 'DB'
        ) LOOP
            APEX_ZIP.ADD_FILE(L_ZIP, ATTACHMENT.FILENAME, ATTACHMENT.CONTENT);
            L_COUNT := 1;
        END LOOP;

        -- Query attachments from associated activities and add to BLOB
        FOR ATTACHMENT IN ( SELECT
                                A.FILENAME AS FILENAME,
                                A.ATCH_CONTENT AS CONTENT
                            FROM
                                T_ACM_ATTACHMENT A
                            WHERE
                                A.OBJ IN (  SELECT
                                                TAA.FROM_OBJ ACT_OBJ
                                            FROM
                                                T_ACM_ASSOCIATION TAA
                                                INNER JOIN T_ACM_OBJ O ON O.SID = TAA.FROM_OBJ 
                                                INNER JOIN T_ACM_OBJ_TYPE OT ON OT.SID = O.OBJ_TYPE AND OT.OBJ_TYPE = 'ACT'
                                            WHERE
                                                TAA.TO_OBJ = P_OBJECT
                                            UNION
                                            SELECT
                                                TAA.TO_OBJ ACT_OBJ
                                            FROM
                                                T_ACM_ASSOCIATION TAA
                                                INNER JOIN T_ACM_OBJ O ON O.SID = TAA.TO_OBJ 
                                                INNER JOIN T_ACM_OBJ_TYPE OT ON OT.SID = O.OBJ_TYPE AND OT.OBJ_TYPE = 'ACT'
                                            WHERE
                                                TAA.FROM_OBJ = P_OBJECT
                                        )
                                AND A.STORAGE_LOC_TYPE = 'DB'
        ) LOOP
            APEX_ZIP.ADD_FILE(L_ZIP, ATTACHMENT.FILENAME, ATTACHMENT.CONTENT);
            L_COUNT := 1;
        END LOOP;
        APEX_ZIP.FINISH(L_ZIP);

        -- Insert the new blob report into the table to download
        IF L_COUNT = 1 THEN
            RETURN FINISH_REPORT( P_FILE_NAME => TO_CHAR(SYSDATE, 'yyyymmdd') || '-attachments.zip', P_DOCX_REPORT => L_ZIP );
        END IF;
    END DOWNLOAD_ALL_ATTACHMENTS;
/** Summary
*<table> 
*<tr><td>07/29/2020</td><td>LLSB</td><td>Task 12103 Updates</td></tr>
*</table>
*/ 
    PROCEDURE APPEND_ALLEGATIONS_DEFAULT(
        P_OBJECT                IN VARCHAR2,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_NO_RESULT_MSG         IN VARCHAR2 := ''
    ) IS
        L_ALLEG_ENTRY_TEMPLATE  CLOB;
        L_ALLEG_ENTRY           CLOB;
        L_ALLEG_LIST            CLOB;
        L_ALLEG_CNT             PLS_INTEGER := 0;
    BEGIN
        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_ALLEG_ENTRY_TEMPLATE);

        -- Setup CLOB for attachment list
        DBMS_LOB.CREATETEMPORARY (L_ALLEG_LIST, FALSE);
        DBMS_LOB.OPEN (L_ALLEG_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ALLEG_LIST, '<dummy>');

        -- Query Allegations
        FOR X IN (  SELECT
                        AT.DISPLAY,
                        AT.CODE,
                        AO.PRIORITY,
                        AO.INCIDENT_CITY,
                        AO.COUNTRY,
                        AO.INCIDENT_DATE
                    FROM
                        T_F_ALLEG_OFFENSE AO
                        LEFT JOIN T_F_ALLEG_TYPE AT ON AO.ALLEGATION = AT.REF_KEY
                    WHERE
                        OBJ = P_OBJECT
                    ORDER BY
                        PRIORITY ASC
        )LOOP
            -- Iterate allegation count
            L_ALLEG_CNT := L_ALLEG_CNT + 1;

            -- Setup allegation entry CLOB
            DBMS_LOB.CREATETEMPORARY (L_ALLEG_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ALLEG_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ALLEG_ENTRY, L_ALLEG_ENTRY_TEMPLATE, DBMS_LOB.GETLENGTH (L_ALLEG_ENTRY_TEMPLATE));

            -- Replace tags in allegation entry CLOB
            DOCX_TEMPLATE.REPLACE_TAG (L_ALLEG_ENTRY, 'ALLEGATION', NVL(X.DISPLAY,'No CI Matter on Record') ||
                ' (' || NVL(X.CODE,'Unknown') || ')' );
            DOCX_TEMPLATE.REPLACE_TAG (L_ALLEG_ENTRY, 'ALLEGATION_DATE', NVL(TO_CHAR(X.INCIDENT_DATE, FMT_DATE),
                'No Date on Record'));
            DOCX_TEMPLATE.REPLACE_TAG (L_ALLEG_ENTRY, 'ALLE_LOCATION', NVL(X.INCIDENT_CITY, 'No City on Record') || ',' ||
                NVL(X.COUNTRY, 'No Country on Record'));
            IF X.PRIORITY = 'ALLEGATION_PRIORITY.PRIMARY' THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ALLEG_ENTRY, 'ALLEGATION_PRIORITY', 'PRIMARY CI MATTER');
            ELSE
                DOCX_TEMPLATE.REPLACE_TAG (L_ALLEG_ENTRY, 'ALLEGATION_PRIORITY', 'SECONDARY CI MATTER');
            END IF;

            -- Append allegation entry to allegation list CLOB
            DBMS_LOB.APPEND (L_ALLEG_LIST, L_ALLEG_ENTRY);
            DBMS_LOB.CLOSE (L_ALLEG_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ALLEG_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_ALLEG_CNT = 0 THEN
            DBMS_LOB.APPEND(L_ALLEG_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ALLEG_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_ALLEG_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ALLEG_LIST);
        DBMS_LOB.FREETEMPORARY (L_ALLEG_LIST);

    END APPEND_ALLEGATIONS_DEFAULT;

/** Summary
*<table>
*<tr><td>03/16/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION ACTIVITY_SUMMARY_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_DATA_VALUE        VARCHAR2(200) := NULL;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
        L_ACTIVITY_NBR      V_ACTIVITY_DATE_NARRATIVE.ACTIVITY_ID%TYPE;
    BEGIN
        -- Start report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'ACT_SUMMARY_RPT');

        --Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'ACT_SUM_RPT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Get data values
        FOR X IN (  SELECT
                        ACTIVITY_OBJ,
                        ACTIVITY_DATE,
                        ACTIVITY_ID,
                        TITLE,
                        NARRATIVE,
                        ACTIVITY_AGENT
                    FROM
                        V_ACTIVITY_DATE_NARRATIVE 
                    WHERE
                        ACTIVITY_OBJ = P_OBJECT)
        LOOP
            -- Tag replacements
            DOCX_TEMPLATE.REPLACE_TAG(L_REPORT, 'SYS_DATE', TO_CHAR (SYSDATE, FMT_DATE));
            DOCX_TEMPLATE.REPLACE_TAG(L_REPORT, 'TITLE', X.TITLE);
            DOCX_TEMPLATE.REPLACE_TAG(L_REPORT, 'NARRATIVE', X.NARRATIVE);
            DOCX_TEMPLATE.REPLACE_TAG(L_REPORT, 'RPT_DATE', TO_CHAR(X.ACTIVITY_DATE, FMT_DATE));
            DOCX_TEMPLATE.REPLACE_TAG(L_REPORT, 'ACTIVITY_NO', X.ACTIVITY_ID);
            DOCX_TEMPLATE.REPLACE_TAG(L_REPORT, 'LEADAGENT_NAME', X.ACTIVITY_AGENT);
            L_ACTIVITY_NBR := X.ACTIVITY_ID;
        END LOOP;

        -- Replace location tag
        IF ACM_OBJ.GET_OBJTYPE_CODE(ACM_OBJ.GET_OBJTYPE(P_OBJECT)) IN ( 'ACT.BRIEFING.ENTERPRISE',
        'ACT.FOR_CONTACT','ACT.FOR_BRIEFING','ACT.INTERVIEW','ACT.SEARCH.PERSON','ACT.SEARCH.PLACE',
        'ACT.SEARCH.PROPERTY') THEN 
            APPEND_LOCATION (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TOKEN_NAME => 'PLACE_OF_ACTIVITY');
        ELSE
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'PLACE_OF_ACTIVITY', GET_OWNING_UNIT(P_OBJECT), 'N');
        END IF;

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Activity_Summary_Report_' || L_ACTIVITY_NBR || '.docx' , P_DOCX_REPORT => L_REPORT );

    END ACTIVITY_SUMMARY_REPORT;

/** Summary
*<table>
*<tr><td>03/24/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION ANP_FILE_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'COMMON_FILE');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'ANP_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_TYPE', 'GENERAL FILE');
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        NARRATIVE
                    FROM
                        T_F_ANP 
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'NARRATIVE', NVL(X.NARRATIVE, ''));
        END LOOP;

        -- Make Participant List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PARTIC_LIST',
            P_NO_RESULT_MSG => '(No Participants)');

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Associated Activities)' );

        -- Make Associated Files List
        APPEND_ASSOCIATED_FILES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'FILES_LIST',
            P_NO_RESULT_MSG => '(No Associated Files)' );

        -- Make Notes List
        APPEND_NOTES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'NOTES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Notes)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'General_File_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END ANP_FILE_REPORT;

/** Summary
*<table>
*<tr><td>03/24/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION CASE_STATUS_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        --Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'CASE_STATUS');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'CASE_STAT_RPT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initial tag replacement
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));


        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION
                    FROM
                        T_F_INQUIRY
                    WHERE
                        OBJ = P_OBJECT
                    UNION ALL
                    SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION
                    FROM
                        T_F_INV_ZERO
                    WHERE
                        OBJ = P_OBJECT
                    UNION ALL
                    SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION
                    FROM
                        T_F_INVESTIGATION
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            -- Replace tags
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUMMARY', NVL(X.SUMMARY_INVESTIGATION, '(No Summary)'));
        END LOOP;

        -- Make Subject List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'SUBJECT_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUBJECT'), P_NO_RESULT_MSG => '(No Subjects)' );

        -- Make Allegations List
        APPEND_ALLEGATIONS_DEFAULT( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ALLEGATIONS',
            P_NO_RESULT_MSG => '(No Allegations)' );

        -- Make Notes List (Updates / IDPs)
        APPEND_NOTES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'UPDATES',
            P_CODES_INCLUDE => T_STR_ARRAY('IDP'), P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Updates)' );

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Case_Status_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END CASE_STATUS_REPORT;

/** Summary
*<table>
*<tr><td>04/01/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION INITIAL_ASSESSMENT_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        --Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'INITIAL_ASSESSMENT');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'CASE_INIT_RPT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classifications block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initial tag replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        PREDICATION,
                        PLAN_RECOMMENDATION
                    FROM
                        T_F_INQUIRY
                    WHERE
                        OBJ = P_OBJECT
                    UNION ALL
                    SELECT
                        ID,
                        TITLE,
                        SUMMARY_ALLEGATION AS PREDICATION,
                        RECOMMENDATION AS PLAN_RECOMMENDATION
                    FROM
                        T_F_INV_ZERO
                    WHERE
                        OBJ = P_OBJECT
                    UNION ALL
                    SELECT
                        ID,
                        TITLE,
                        PREDICATION,
                        RECOMMENDATION AS PLAN_RECOMMENDATION
                    FROM
                        T_F_INVESTIGATION
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            -- Replace tags
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_NUM', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'PREDICATION', NVL(X.PREDICATION, '(No Predication)'));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RECOMMENDATION', NVL(X.PLAN_RECOMMENDATION, '(No Recommendation)'));
        END LOOP;

        -- Make Allegations List
        APPEND_ALLEGATIONS_DEFAULT( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ALLEGATIONS',
            P_NO_RESULT_MSG => '(No Allegations)' );

        -- Make Subject List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'SUBJECT_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUBJECT'), P_NO_RESULT_MSG => '(No Subjects)' );

        -- Make Bureau List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'BUREAU_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('VIC_AGENCY'), P_NO_RESULT_MSG => '(No Bureaus Involved)' );

        -- Make Associated Files List
        APPEND_ASSOCIATED_FILES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'FILES_LIST',
            P_NO_RESULT_MSG => '(No Associated Files)' );

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Initial_Assessment_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END INITIAL_ASSESSMENT_REPORT;

/** Summary
*<table>
*<tr><td>04/01/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION LETTER_OF_NOTIFICATION_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
        L_UNIT_FAX          T_PARTIC_CONTACT.VALUE%TYPE;
        L_PERSONNEL_PHONE   T_PARTIC_CONTACT.VALUE%TYPE;
        L_LEAD_PHONE        T_PARTIC_CONTACT.VALUE%TYPE;
    BEGIN
        --Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'LETTER_OF_NOTIFICATION');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'LT_O_NOTIF', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classifications block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);


        FOR X IN (  SELECT
                        ID
                    FROM
                        T_ACM_OBJ 
                    WHERE
                        SID = P_OBJECT
        ) LOOP
            -- Replace tags
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_NUM', NVL(X.ID, ''));
        END LOOP;

        FOR X IN (  SELECT
                        NAME.LAST_NAME,
                        NAME.FIRST_NAME,
                        NUM.VALUE AS SSN,
                        O.TAGLINE AS SUBJECT_AGENCY
                    FROM
                        T_PARTIC_INVOLVEMENT P
                        INNER JOIN T_PARTIC_NAME NAME ON NAME.OBJ = P.PARTICIPANT AND NAME.IS_CURRENT = 'Y'
                        LEFT JOIN T_PARTIC_NUMBER NUM ON NUM.OBJ = P.PARTICIPANT AND NUM.NUMBER_TYPE = 'PARTIC.INDIV.NUMBER_TYPE.SSN'
                        LEFT JOIN T_PARTIC_INDIV_DHS_COMP C ON C.PARTICIPANT = P.PARTICIPANT
                        LEFT JOIN T_ACM_OBJ O ON O.SID = C.DHS_COMPONENT
                    WHERE
                        P.OBJ = P_OBJECT
                        AND P.INVOLVEMENT_ROLE IN (SELECT SID FROM T_PARTIC_ROLE_TYPE WHERE CODE = 'SUBJECT')
        ) LOOP
            -- Replace tags
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUBJECT_NAME', INITCAP(X.FIRST_NAME) || ' ' || INITCAP(X.LAST_NAME));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUBJECT_LASTNAME', INITCAP(NVL(X.LAST_NAME, X.FIRST_NAME)));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUBJECT_AGENCY', NVL(X.SUBJECT_AGENCY, '(Unkown Agency)'));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUBJECT_SSN', NVL(X.SSN, '(Unknown SSN)'));
        END LOOP;

        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'OWNING_UNIT', GET_OWNING_UNIT(P_OBJECT));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'LEAD_AGENT', INITCAP(GET_LEAD(P_OBJECT)));

        -- Get lead agent office phone
        FOR X IN (  SELECT
                        PHONE_NUMBER
                    FROM (  SELECT
                                R.SEQ,
                                C.VALUE AS PHONE_NUMBER
                            FROM
                                V_CURR_OBJ_LEAD_PERS L
                                INNER JOIN T_PARTIC_CONTACT C ON C.OBJ = L.SH_OBJ
                                INNER JOIN T_PARTIC_CONTACT_REF R ON R.REF_KEY = C.CONTACT_TYPE
                            WHERE
                                L.OBJ = P_OBJECT
                                AND R.CODE IN ('OFFP','OFFA')
                            ORDER
                                BY R.SEQ)
                    WHERE
                        ROWNUM = 1
        ) LOOP
            -- Replace tags
            L_LEAD_PHONE := X.PHONE_NUMBER;

        END LOOP;

        -- Get unit fax number
        FOR X IN (  SELECT
                        FAX
                    FROM (  SELECT
                                R.SEQ,
                                C.VALUE AS FAX
                            FROM
                                V_CURR_OBJ_OWNING_UNIT U
                                INNER JOIN T_PARTIC_CONTACT C ON C.OBJ = U.SH_OBJ
                                INNER JOIN T_PARTIC_CONTACT_REF R ON R.REF_KEY = C.CONTACT_TYPE
                            WHERE
                                U.OBJ = P_OBJECT
                                AND R.CODE IN ('HOMEF','OFFF','DSNF')
                            ORDER
                                BY R.SEQ)
                    WHERE
                        ROWNUM = 1
        ) LOOP
            -- Replace tags
            L_UNIT_FAX := X.FAX;
        END LOOP;

        -- Get personnel number
        FOR X IN (  SELECT
                        PHONE_NUMBER
                    FROM (  SELECT
                                R.SEQ,
                                C.VALUE AS PHONE_NUMBER
                            FROM
                                T_PARTIC_CONTACT C
                                INNER JOIN T_PARTIC_CONTACT_REF R ON R.REF_KEY = C.CONTACT_TYPE
                            WHERE
                                C.OBJ = ACM_CONTEXT.PERSONNEL_OBJ
                                AND R.CODE IN ('OFFP','OFFA')
                            ORDER
                                BY R.SEQ)
                    WHERE
                        ROWNUM = 1
        ) LOOP
            -- Replace tags
            L_PERSONNEL_PHONE := X.PHONE_NUMBER;    
        END LOOP;


        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'LEAD_AGENT_PHONE', NVL(L_LEAD_PHONE, '(Unknown Lead Agent Office Phone)') );
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'UNIT_FAX_NUMBER', NVL(L_UNIT_FAX, '(Unknown Unit Fax)') );
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'PERSONNEL_PHONE', NVL(L_PERSONNEL_PHONE, '(Unknown Personnel Office Phone)') );

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Letter_of_Notification_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END LETTER_OF_NOTIFICATION_REPORT;

/** Summary
*<table>
*<tr><td>04/01/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION ROI_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'ROI_REPORT');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'ROI', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));


        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION,
                        PREDICATION
                    FROM
                        T_F_INQUIRY
                    WHERE
                        OBJ = P_OBJECT
                    UNION ALL
                    SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION,
                        SUMMARY_ALLEGATION AS PREDICATION
                    FROM
                        T_F_INV_ZERO
                    WHERE
                        OBJ = P_OBJECT
                    UNION ALL
                    SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION,
                        PREDICATION
                    FROM
                        T_F_INVESTIGATION
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'PREDICATION', NVL(X.PREDICATION, '(No Predication)'));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUMMARY', NVL(X.SUMMARY_INVESTIGATION, '(No Summary)'));
        END LOOP;

        -- Make Subject List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'SUBJECT_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUBJECT'), P_NO_RESULT_MSG => '(No Subjects)' );

        -- Make Allegations List
        APPEND_ALLEGATIONS_DEFAULT( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ALLEGATIONS',
            P_NO_RESULT_MSG => '(No Allegations)' );

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Activities)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'ROI_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END ROI_REPORT;

/** Summary
*<table>
*<tr><td>04/01/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION CIRA_REPORT (
        P_OBJECT                IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT                BLOB;
        L_SPEC_ROW              T_DOCX_REPORT_SPEC%ROWTYPE;
        L_SUMMARIES_TEMPLATE    CLOB;
        L_RISKS_TEMPLATE        CLOB;
        L_MITIGATION_TEMPLATE   CLOB;
        L_MIT_HEADING_TEMPLATE  CLOB;
        L_ITEM_LIST             CLOB;
        L_ITEM_ENTRY            CLOB;
        L_ITEM_CNT              PLS_INTEGER := 0;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'CIRA_REPORT');

        -- Get templates for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'SUMMARIES', P_FILE_BLOB => L_REPORT, P_CLOB => L_SUMMARIES_TEMPLATE);
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'CI_RISKS', P_FILE_BLOB => L_REPORT, P_CLOB => L_RISKS_TEMPLATE);
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'PARAGRAPH_4_MITIGATION_ITEM', P_FILE_BLOB => L_REPORT, P_CLOB => L_MITIGATION_TEMPLATE);

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'CIRA_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'LEAD_AGENT', GET_LEAD(P_OBJECT));

        -- Make Subject List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'SUBJECT_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUBJECT'), P_NO_RESULT_MSG => '(No Subjects)' );

        -- Query for risk level and risk level description
        FOR X IN (  SELECT
                        L.DISPLAY AS RISK_LEVEL,
                        L.LONG_DISPLAY AS RISK_LEVEL_DESCRIPTION
                    FROM
                        T_F_RISK_ASSESS R
                        INNER JOIN T_F_RISK_ASSESS_REF L ON L.SID = R.RISK_LEVEL
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RISK_LEVEL', NVL(X.RISK_LEVEL, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RISK_LEVEL_DESCRIPTION', NVL(X.RISK_LEVEL_DESCRIPTION, ''));

            -- Determine if mitigation is needed and get template for the appropriate heading
            IF X.RISK_LEVEL = 'Low' THEN
                GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'PARAGRAPH_4_NO_MITIGATION_HEADING', P_FILE_BLOB => L_REPORT, P_CLOB => L_MIT_HEADING_TEMPLATE);
            ELSE
                GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => 'PARAGRAPH_4_NEEDS_MITIGATION_HEADING', P_FILE_BLOB => L_REPORT, P_CLOB => L_MIT_HEADING_TEMPLATE);
            END IF;
        END LOOP;


        -- APPEND SUMMARIES ---------------------------------------------------
        -- Open list CLOB for all the summary entries
        DBMS_LOB.CREATETEMPORARY(L_ITEM_LIST, FALSE);
        DBMS_LOB.OPEN(L_ITEM_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ITEM_LIST, '<dummy>');

        -- Query for summaries
        FOR X IN (  SELECT
                        N.NARRATIVE,
                        N.CREATED_BY,
                        N.CREATED_ON
                    FROM
                        T_F_RISK_ASSESS_NARRATIVES N
                        INNER JOIN T_F_RISK_ASSESS_REF R ON R.SID = N.NARRATIVE_TYPE AND R.CODE = 'PRED'
                    WHERE
                        N.OBJ = P_OBJECT
                    ORDER BY
                        N.CREATED_ON
        )LOOP
            -- Iterate the counter
            L_ITEM_CNT := L_ITEM_CNT + 1;

            -- Open clob for entry
            DBMS_LOB.CREATETEMPORARY (L_ITEM_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ITEM_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ITEM_ENTRY, L_SUMMARIES_TEMPLATE, DBMS_LOB.GETLENGTH (L_SUMMARIES_TEMPLATE));

            -- Add paragraph number to the entry if it is the first entry
            IF L_ITEM_CNT = 1 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_2_NUMBER', '2. ');
            ELSE
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_2_NUMBER', '');
            END IF;

            -- Replace tokens for entry
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_2_SUMMARIES', X.NARRATIVE);
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_2_DATE', X.CREATED_ON);

            -- Append entry to the list
            DBMS_LOB.APPEND (L_ITEM_LIST, L_ITEM_ENTRY);
            -- Clean up the working copy
            DBMS_LOB.CLOSE (L_ITEM_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ITEM_ENTRY);

        END LOOP;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ITEM_LIST, '</dummy>');

        -- Replace token with list
        L_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => 'SUMMARIES', P_DATA_TO_INSERT => L_ITEM_LIST, P_FILE => L_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ITEM_LIST);
        DBMS_LOB.FREETEMPORARY (L_ITEM_LIST);



        -- APPEND RISKS -------------------------------------------------------
        -- Open list CLOB for all the risk entries
        DBMS_LOB.CREATETEMPORARY(L_ITEM_LIST, FALSE);
        DBMS_LOB.OPEN(L_ITEM_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ITEM_LIST, '<dummy>');

        -- Query for risks
        FOR X IN (  SELECT
                        N.NARRATIVE,
                        N.CREATED_BY,
                        N.CREATED_ON
                    FROM
                        T_F_RISK_ASSESS_NARRATIVES N
                        INNER JOIN T_F_RISK_ASSESS_REF R ON R.SID = N.NARRATIVE_TYPE AND R.CODE = 'CIRISK'
                    WHERE
                        N.OBJ = P_OBJECT
                    ORDER BY
                        N.CREATED_ON
        )LOOP
            -- Iterate the counter
            L_ITEM_CNT := L_ITEM_CNT + 1;

            -- Open clob for entry
            DBMS_LOB.CREATETEMPORARY (L_ITEM_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ITEM_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ITEM_ENTRY, L_RISKS_TEMPLATE, DBMS_LOB.GETLENGTH (L_RISKS_TEMPLATE));

            -- Add paragraph number to the entry if it is the first entry
            IF L_ITEM_CNT = 1 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_3_NUMBER', '3. ');
            ELSE
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_3_NUMBER', '');
            END IF;

            -- Replace tokens for entry
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'PARAGRAPH_3_CI_RISKS', X.NARRATIVE);

            -- Append entry to the list
            DBMS_LOB.APPEND (L_ITEM_LIST, L_ITEM_ENTRY);
            -- Clean up the working copy
            DBMS_LOB.CLOSE (L_ITEM_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ITEM_ENTRY);

        END LOOP;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ITEM_LIST, '</dummy>');

        -- Replace token with list
        L_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => 'CI_RISKS', P_DATA_TO_INSERT => L_ITEM_LIST, P_FILE => L_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ITEM_LIST);
        DBMS_LOB.FREETEMPORARY (L_ITEM_LIST);



        -- APPEND MITIGATIONS -------------------------------------------------
        -- Reset counter
        L_ITEM_CNT := 0;

        -- Open list CLOB for all the mitigation entries
        DBMS_LOB.CREATETEMPORARY(L_ITEM_LIST, FALSE);
        DBMS_LOB.OPEN(L_ITEM_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_ITEM_LIST, '<dummy>');

        -- Query for mitigations
        FOR X IN (  SELECT
                        R.SEQ,
                        R.DISPLAY AS RISK_MITGATION
                    FROM
                        T_F_RISK_ASSESS_MITIGATION M
                        INNER JOIN T_F_RISK_ASSESS_REF R ON R.SID = M.MITIGATION_TYPE
                    WHERE
                        M.OBJ = P_OBJECT
                    ORDER BY
                        R.SEQ
        )LOOP
            -- Iterate the counter
            L_ITEM_CNT := L_ITEM_CNT + 1;

            IF L_ITEM_CNT = 1 THEN
                -- Open list CLOB for all the entries
                DBMS_LOB.CREATETEMPORARY(L_ITEM_LIST, FALSE);
                DBMS_LOB.OPEN(L_ITEM_LIST, DBMS_LOB.LOB_READWRITE);
                DBMS_LOB.APPEND(L_ITEM_LIST, '<dummy>');

                -- Append appropriate heading to list
                DBMS_LOB.APPEND(L_ITEM_LIST, L_MIT_HEADING_TEMPLATE);
            END IF;

            -- Open clob for entry
            DBMS_LOB.CREATETEMPORARY (L_ITEM_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_ITEM_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_ITEM_ENTRY, L_MITIGATION_TEMPLATE, DBMS_LOB.GETLENGTH (L_MITIGATION_TEMPLATE));

            IF L_ITEM_CNT = 1 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MIT_LETTER', 'a.');
            ELSIF L_ITEM_CNT = 2 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MIT_LETTER', 'b.');
            ELSIF L_ITEM_CNT = 3 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MIT_LETTER', 'c.');
            ELSIF L_ITEM_CNT = 4 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MIT_LETTER', 'd.');
            ELSIF L_ITEM_CNT = 5 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MIT_LETTER', 'e.');
            ELSIF L_ITEM_CNT = 6 THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MIT_LETTER', 'f.');
            END IF;

            -- Replace entry tokens
            DOCX_TEMPLATE.REPLACE_TAG (L_ITEM_ENTRY, 'P4_RISK_MITIGATION', X.RISK_MITGATION);

            -- Append entry to the list
            DBMS_LOB.APPEND (L_ITEM_LIST, L_ITEM_ENTRY);
            -- Clean up the working copy
            DBMS_LOB.CLOSE (L_ITEM_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_ITEM_ENTRY);

        END LOOP;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_ITEM_LIST, '</dummy>');

        -- Replace token with list
        L_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => 'PARAGRAPH_4_MITIGATION', P_DATA_TO_INSERT => L_ITEM_LIST, P_FILE => L_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_ITEM_LIST);
        DBMS_LOB.FREETEMPORARY (L_ITEM_LIST);



        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'CIRA_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END CIRA_REPORT;

/** Summary
*<table>
*<tr><td>04/04/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION CYBER_FILE_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'COMMON_FILE');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'CYBER_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_TYPE', 'CYBER FILE');
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        DESCRIPTION
                    FROM
                        T_F_CYBER 
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'NARRATIVE', NVL(X.DESCRIPTION, ''));
        END LOOP;

        -- Make Participant List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PARTIC_LIST',
            P_NO_RESULT_MSG => '(No Participants)');

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Associated Activities)' );

        -- Make Associated Files List
        APPEND_ASSOCIATED_FILES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'FILES_LIST',
            P_NO_RESULT_MSG => '(No Associated Files)' );

        -- Make Notes List
        APPEND_NOTES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'NOTES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Notes)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Cyber_File_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END CYBER_FILE_REPORT;

/** Summary
*<table>
*<tr><td>04/01/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION SUB_VIC_WIT_LIST_REPORT (
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'SUBJECT_VICTIM_WITNESS_LIST');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'SVW_LIST', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));


        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TAGLINE AS TITLE
                    FROM
                        T_ACM_OBJ
                    WHERE
                        SID = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
        END LOOP;

        -- Make Subject List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'SUBJECT_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUBJECT'), P_NO_RESULT_MSG => '(No Subjects)' );

        -- Make Victim List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'VICTIM_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('VICTIM'), P_NO_RESULT_MSG => '(No Victims)' );

        -- Make Witness List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'WITNESS_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('WITNESS'), P_NO_RESULT_MSG => '(No Witnesses)' );

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'SUB_VIC_WIT_List_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END SUB_VIC_WIT_LIST_REPORT;

/** Summary
*<table>
*<tr><td>04/04/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION OP_FILE_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'COMMON_FILE');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'OP_F_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_TYPE', 'OPERATIONS');
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        SUMMARY
                    FROM
                        T_F_OPERATIONS
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'NARRATIVE', NVL(X.SUMMARY, ''));
        END LOOP;

        -- Make Participant List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PARTIC_LIST',
            P_NO_RESULT_MSG => '(No Participants)');

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Associated Activities)' );

        -- Make Associated Files List
        APPEND_ASSOCIATED_FILES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'FILES_LIST',
            P_NO_RESULT_MSG => '(No Associated Files)' );

        -- Make Notes List
        APPEND_NOTES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'NOTES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Notes)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Operations_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END OP_FILE_REPORT;

/** Summary
*<table>
*<tr><td>04/04/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION SUPPLY_CHAIN_REPORT (
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'SUPPLY_CHAIN_FILE');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'SPLYCHN_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_TYPE', 'OPERATIONS');
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE
                    FROM
                        T_F_SUPPLY_CHAIN
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
        END LOOP;

        -- Make Supplier List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PARTIC_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUPPLIER'), P_NO_RESULT_MSG => '(No Supplier)' );

        -- Make product list
        APPEND_PRODUCTS (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PRODUCT_LIST',
            P_NO_RESULT_MSG => '(No Products)' );

        -- Make narrative list
        APPEND_COMPANY_BG_DETAILS (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'DETAIL_LIST',
            P_NO_RESULT_MSG => '(No Company Background Details)' );

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Supply_Chain_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END SUPPLY_CHAIN_REPORT;

/** Summary
*<table>
*<tr><td>04/05/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION TRAVEL_FILE_REPORT (
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'COMMON_FILE');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'TRAVEL_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FILE_TYPE', 'TRAVEL');
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        SUMMARY
                    FROM
                        T_F_TRAVEL
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'NARRATIVE', NVL(X.SUMMARY, ''));
        END LOOP;

        -- Make Participant List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PARTIC_LIST',
            P_NO_RESULT_MSG => '(No Participants)');

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Associated Activities)' );

        -- Make Associated Files List
        APPEND_ASSOCIATED_FILES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'FILES_LIST',
            P_NO_RESULT_MSG => '(No Associated Files)' );

        -- Make Notes List
        APPEND_NOTES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'NOTES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Notes)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'Travel_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END TRAVEL_FILE_REPORT;

/** Summary
*<table>
*<tr><td>04/05/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION TSCM_FILE_REPORT (
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'TSCM_FILE');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'TSCM_REPORT', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));

        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE
                    FROM
                        T_F_TSCM
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
        END LOOP;

        -- Make Participant List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'PARTIC_LIST',
            P_NO_RESULT_MSG => '(No Participants)');

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Associated Activities)' );

        -- Make Associated Files List
        APPEND_ASSOCIATED_FILES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'FILES_LIST',
            P_NO_RESULT_MSG => '(No Associated Files)' );

        -- Make Notes List
        APPEND_NOTES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'NOTES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Notes)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'TSCM_File_Report_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END TSCM_FILE_REPORT;
    
/** Summary
*<table> 
*<tr><td>07/29/2020</td><td>LLSB</td><td>Task 12103 Updates</td></tr>
*</table>
*/ 
    PROCEDURE APPEND_IT_MATTERS_DEFAULT(
        P_OBJECT                IN VARCHAR2,
        P_DOCX_REPORT           IN OUT NOCOPY BLOB,
        P_TABLE_CAPTION         IN VARCHAR2,
        P_NO_RESULT_MSG         IN VARCHAR2 := ''
    ) IS
        L_IT_M_ENTRY_TEMPLATE  CLOB;
        L_IT_M_ENTRY           CLOB;
        L_IT_M_LIST            CLOB;
        L_IT_M_CNT             PLS_INTEGER := 0;
    BEGIN
        -- Get template for entry clobs
        GET_DOCX_BODY_TABLE_ROW_TEMPLATE(P_TABLE_CAPTION => P_TABLE_CAPTION, P_FILE_BLOB => P_DOCX_REPORT, P_CLOB => L_IT_M_ENTRY_TEMPLATE);

        -- Setup CLOB for attachment list
        DBMS_LOB.CREATETEMPORARY (L_IT_M_LIST, FALSE);
        DBMS_LOB.OPEN (L_IT_M_LIST, DBMS_LOB.LOB_READWRITE);
        DBMS_LOB.APPEND(L_IT_M_LIST, '<dummy>');

        -- Query Allegations
        FOR X IN (  SELECT
                        IR.DISPLAY,
                        IR.CODE,
                        IM.PRIORITY,
                        IM.INCIDENT_CITY,
                        IM.COUNTRY,
                        IM.INCIDENT_DATE
                    FROM
                        T_F_IT_MATTER IM
                        LEFT JOIN T_F_IRMO_REF IR ON IM.MATTER_SUBTYPE = IR.REF_KEY
                        LEFT JOIN T_F_IRMO_REF IR1 ON IM.MATTER = IR1.REF_KEY
                    WHERE
                        OBJ = P_OBJECT
                    ORDER BY
                        PRIORITY ASC
        )LOOP
            -- Iterate allegation count
            L_IT_M_CNT := L_IT_M_CNT + 1;

            -- Setup allegation entry CLOB
            DBMS_LOB.CREATETEMPORARY (L_IT_M_ENTRY, FALSE);
            DBMS_LOB.OPEN (L_IT_M_ENTRY, DBMS_LOB.LOB_READWRITE);
            DBMS_LOB.COPY (L_IT_M_ENTRY, L_IT_M_ENTRY_TEMPLATE, DBMS_LOB.GETLENGTH (L_IT_M_ENTRY_TEMPLATE));

            -- Replace tags in allegation entry CLOB
            DOCX_TEMPLATE.REPLACE_TAG (L_IT_M_ENTRY, 'ALLEGATION', NVL(X.DISPLAY,'No IT Matter on Record') ||
                ' (' || NVL(X.CODE,'Unknown') || ')' );
            DOCX_TEMPLATE.REPLACE_TAG (L_IT_M_ENTRY, 'ALLEGATION_DATE', NVL(TO_CHAR(X.INCIDENT_DATE, FMT_DATE),
                'No Date on Record'));
            DOCX_TEMPLATE.REPLACE_TAG (L_IT_M_ENTRY, 'ALLE_LOCATION', NVL(X.INCIDENT_CITY, 'No City on Record') || ',' ||
                NVL(X.COUNTRY, 'No Country on Record'));
            IF X.PRIORITY = 'ALLEGATION_PRIORITY.PRIMARY' THEN
                DOCX_TEMPLATE.REPLACE_TAG (L_IT_M_ENTRY, 'ALLEGATION_PRIORITY', 'PRIMARY IT MATTER');
            ELSE
                DOCX_TEMPLATE.REPLACE_TAG (L_IT_M_ENTRY, 'ALLEGATION_PRIORITY', 'SECONDARY IT MATTER');
            END IF;

            -- Append allegation entry to allegation list CLOB
            DBMS_LOB.APPEND (L_IT_M_LIST, L_IT_M_ENTRY);
            DBMS_LOB.CLOSE (L_IT_M_ENTRY);
            DBMS_LOB.FREETEMPORARY (L_IT_M_ENTRY);
        END LOOP;

        -- Check if there were any entries, otherwise append no results message
        IF L_IT_M_CNT = 0 THEN
            DBMS_LOB.APPEND(L_IT_M_LIST, P_NO_RESULT_MSG);
        END IF;

        -- Append dummy tag to the end of the list
        DBMS_LOB.APPEND(L_IT_M_LIST, '</dummy>');

        -- Replace token with list
        P_DOCX_REPORT := REPLACE_DOC_XML(P_TABLE_CAPTION => P_TABLE_CAPTION, P_DATA_TO_INSERT => L_IT_M_LIST, P_FILE => P_DOCX_REPORT);

        -- Clean up LOBs
        DBMS_LOB.CLOSE (L_IT_M_LIST);
        DBMS_LOB.FREETEMPORARY (L_IT_M_LIST);

    END APPEND_IT_MATTERS_DEFAULT;
    
/** Summary
*<table>
*<tr><td>04/01/2022</td><td>CPB</td><td>Initial addition to the baseline</td></tr>
*</table>
*/
    FUNCTION IT_ROI_REPORT(
        P_OBJECT            IN VARCHAR2
    ) RETURN NUMBER IS
        L_REPORT            BLOB;
        L_SPEC_ROW          T_DOCX_REPORT_SPEC%ROWTYPE;
    BEGIN
        -- Starts report
        START_REPORT (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TEMPLATE_NAME => 'IT_ROI');

        -- Get the spec row if it exists
        GET_SPEC (P_OBJECT => P_OBJECT, P_REPORT_CODE => 'ROI', P_SPEC_ROW => L_SPEC_ROW);

        -- Append the classifications marking
        APPEND_CLASSIFICATION_MARKING (P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT);

        -- Append the classification block
        REPLACE_CLASSIFICATION_BLOCK (P_DOCX_REPORT => L_REPORT, P_SPEC_SID => L_SPEC_ROW.SID);

        -- Initital Tag Replacements
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'REPORT_BY', GET_REPORT_PERSONNEL_NAME(P_OBJECT, P_SPEC_SID => L_SPEC_ROW.SID));
        DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'RPT_DATE', TO_CHAR(SYSDATE, FMT_DATE));


        -- Query for ID, title, and narrative
        FOR X IN (  SELECT
                        ID,
                        TITLE,
                        SUMMARY_INVESTIGATION
                    FROM
                        T_F_INV_1
                    WHERE
                        OBJ = P_OBJECT
        )LOOP
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'FULL_ID', NVL(X.ID, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'TITLE', NVL(X.TITLE, ''));
            DOCX_TEMPLATE.REPLACE_TAG (L_REPORT, 'SUMMARY', NVL(X.SUMMARY_INVESTIGATION, '(No Summary)'));
        END LOOP;

        -- Make Subject List
        APPEND_PARTICIPANTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'SUBJECT_LIST',
            P_CODES_INCLUDE => T_STR_ARRAY('SUBJECT'), P_NO_RESULT_MSG => '(No Subjects)' );

        -- Make IT Matters List
        APPEND_IT_MATTERS_DEFAULT( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'IT_MATTERS',
            P_NO_RESULT_MSG => '(No IT Matters)' );

        -- Make Associated Activities List
        APPEND_ACTIVITIES_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ACTIVITIES_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Activities)' );

        -- Make Attachments List
        APPEND_ATTACHMENTS_DEFAULT ( P_OBJECT => P_OBJECT, P_DOCX_REPORT => L_REPORT, P_TABLE_CAPTION => 'ATTACHMENTS_LIST',
            P_SPEC_SID => L_SPEC_ROW.SID, P_NO_RESULT_MSG => '(No Attachments)' );  

        -- Insert the new blob report into the table to download
        RETURN FINISH_REPORT ( P_FILE_NAME => 'IT_ROI_REPORT_' || TO_CHAR(SYSDATE, 'YYYYMMDD') || '.docx' , P_DOCX_REPORT => L_REPORT );
    END IT_ROI_REPORT;

END DOCX_REPORT;
/