


--T_F_IRMO_REF create

CREATE TABLE "ACM"."T_F_IRMO_REF" 
   (	"REF_KEY" VARCHAR2(108 BYTE) NOT NULL ENABLE, 
	"USAGE" VARCHAR2(50 BYTE), 
	"CODE" VARCHAR2(20 BYTE), 
	"DISPLAY" VARCHAR2(4000 BYTE), 
	"SEQ" NUMBER, 
	"ACTIVE" VARCHAR2(1 BYTE) DEFAULT 'Y', 
	"DEV_COMMENT" VARCHAR2(4000 BYTE), 
	"CREATED_BY" VARCHAR2(100 BYTE), 
	"CREATED_ON" DATE, 
	"LAST_MODIFIED_BY" VARCHAR2(100 BYTE), 
	"LAST_MODIFIED_ON" DATE, 
	"OBJ_TYPE" NUMBER, 
	"PARENT_KEY" VARCHAR2(108 BYTE), 
	 CONSTRAINT "NN_FIRMO_REF_ACTIVE" CHECK (ACTIVE IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_REF_REFKEY" CHECK (REF_KEY IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_REF_DISPLAY" CHECK (DISPLAY IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_REF_CODE" CHECK (CODE IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_REF_USAGE" CHECK (USAGE IS NOT NULL) ENABLE, 
	 CONSTRAINT "FK_FIRMO_REF_OBJ_TYPE" FOREIGN KEY ("OBJ_TYPE")
	  REFERENCES "ACM"."T_ACM_OBJ_TYPE" ("SID") ON DELETE CASCADE ENABLE
   )
  TABLESPACE "ACM_DATA" ;
  CREATE UNIQUE INDEX "ACM"."I_PK_FIRMO_REF" ON "ACM"."T_F_IRMO_REF" ("REF_KEY") 
  TABLESPACE "ACM_DATA" ;
  CREATE UNIQUE INDEX "ACM"."UK_FIRMO_REF_USAGE_CODE" ON "ACM"."T_F_IRMO_REF" ("USAGE", "CODE", "OBJ_TYPE") 
  TABLESPACE "ACM_DATA" ;
ALTER TABLE "ACM"."T_F_IRMO_REF" ADD CONSTRAINT "PK_FIRMO_REF" PRIMARY KEY ("REF_KEY")
  USING INDEX "ACM"."I_PK_FIRMO_REF"  ENABLE;
ALTER TABLE "ACM"."T_F_IRMO_REF" ADD CONSTRAINT "UK_FIRMO_REF_USAGE_CODE" UNIQUE ("USAGE", "CODE", "OBJ_TYPE")
  USING INDEX "ACM"."UK_FIRMO_REF_USAGE_CODE"  ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."F_IRMO_REF_B_IUD" BEFORE
    INSERT OR UPDATE ON T_F_IRMO_REF
    FOR EACH ROW
DECLARE
    V_REF_KEY_CHANGED BOOLEAN := FALSE;
BEGIN
    IF INSERTING THEN
        SET_TABLE_ROW_VALS_NO_SID(:NEW.CREATED_ON,:NEW.CREATED_BY,:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY);
    END IF;
    IF INSERTING OR UPDATING THEN
        :NEW.USAGE := REGEXP_REPLACE(
            :NEW.USAGE,
            '[^A-Z_0-9]',
            ''
        );
        :NEW.CODE := REGEXP_REPLACE(
            :NEW.CODE,
            '[^A-Z_0-9]',
            ''
        );
        IF :NEW.OBJ_TYPE IS NULL THEN
            :NEW.REF_KEY := :NEW.USAGE || '.' || :NEW.CODE;
        ELSE
            :NEW.REF_KEY := ACM_OBJ.GET_OBJTYPE_CODE(:NEW.OBJ_TYPE)|| '.' || :NEW.USAGE || '.' || :NEW.CODE;
        END IF;
        :NEW.REF_KEY := UPPER(:NEW.REF_KEY);
    END IF;
    IF UPDATING THEN
        SET_TABLE_ROW_MOD_VALS(:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY);
    END IF;
END;


/
ALTER TRIGGER "ACM"."F_IRMO_REF_B_IUD" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."BD_F_IRMO_REF_AUD" before delete on T_F_IRMO_REF for each row begin if sync_util.dg_from_instance=1 then insert into t_sync_del_rows(table_name,del_id)values('T_F_IRMO_REF',:OLD.REF_KEY); end if; end;

/
ALTER TRIGGER "ACM"."BD_F_IRMO_REF_AUD" ENABLE;


  GRANT SELECT ON "ACM"."T_F_IRMO_REF" TO "ADV_SEARCH";
  
  
--T_F_IRMO  create
  
  
  CREATE TABLE "ACM"."T_F_IRMO" 
   (	"SID" NUMBER NOT NULL ENABLE, 
	"OBJ" VARCHAR2(20 BYTE), 
	"ID" VARCHAR2(100 BYTE), 
	"TITLE" VARCHAR2(100 BYTE), 
	"IRMO_CONTROL" VARCHAR2(100 BYTE), 
	"RECEIPT_METHOD" VARCHAR2(4000 BYTE), 
	"CISO_COMMENTS" VARCHAR2(4000 BYTE), 
	"SUMMARY" CLOB, 
	"CREATED_BY" VARCHAR2(100 BYTE), 
	"CREATED_ON" DATE, 
	"LAST_MODIFIED_BY" VARCHAR2(100 BYTE), 
	"LAST_MODIFIED_ON" DATE, 
	"FIE_COUNTRY" VARCHAR2(108 BYTE), 
	"OBTAIN_LOCATION" VARCHAR2(1000 BYTE), 
	"IIR_NUMBER" NUMBER, 
	"IIR_REF_NUM" VARCHAR2(200 BYTE), 
	"FOLLOWUP_COMMENT" VARCHAR2(4000 BYTE), 
	"REFERRAL_TYPE" VARCHAR2(108 BYTE), 
	"REFERRAL_SUBTYPE" VARCHAR2(108 BYTE), 
	"ACQUISITION_DATE" DATE NOT NULL ENABLE, 
	"DATA_GROUP_ID" NUMBER DEFAULT NULL NOT NULL ENABLE, 
	 CONSTRAINT "NN_FIRMO_OBJ" CHECK ("OBJ" IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_SID" CHECK ("SID" IS NOT NULL) ENABLE, 
	 CONSTRAINT "FK_FIRMO_REF_SUBTYPE" FOREIGN KEY ("REFERRAL_SUBTYPE")
	  REFERENCES "ACM"."T_F_IRMO_REF" ("REF_KEY") ENABLE, 
	 CONSTRAINT "FK_FIRMO_OBJ" FOREIGN KEY ("OBJ")
	  REFERENCES "ACM"."T_ACM_OBJ" ("SID") ON DELETE CASCADE ENABLE, 
	 CONSTRAINT "FK_FIRMO_FIE_COUNTRY" FOREIGN KEY ("FIE_COUNTRY")
	  REFERENCES "ACM"."T_ACM_REF_CTRY" ("ISO_3166_1_ALPHA_2") ENABLE, 
	 CONSTRAINT "FK_FIRMO_RECEIPTMETHOD" FOREIGN KEY ("RECEIPT_METHOD")
	  REFERENCES "ACM"."T_F_IRMO_REF" ("REF_KEY") ENABLE
   )
  TABLESPACE "ACM_DATA" ;
  CREATE UNIQUE INDEX "ACM"."PK_FIRMO" ON "ACM"."T_F_IRMO" ("SID") 
  TABLESPACE "ACM_INDX" ;
ALTER TABLE "ACM"."T_F_IRMO" ADD CONSTRAINT "PK_FIRMO" PRIMARY KEY ("SID")
  USING INDEX "ACM"."PK_FIRMO"  ENABLE;

  CREATE INDEX "ACM"."I_FIRMO_FIE_COUNTRY" ON "ACM"."T_F_IRMO" ("FIE_COUNTRY") 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_FIRMO_OBJ" ON "ACM"."T_F_IRMO" ("OBJ") 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_F_IRMO_RECEIPTMETHOD" ON "ACM"."T_F_IRMO" ("RECEIPT_METHOD", 1) 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_F_IRMO_REFERRALSUBTYPE" ON "ACM"."T_F_IRMO" ("REFERRAL_SUBTYPE", 1) 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_F_IRMO_REFERRALTYPE" ON "ACM"."T_F_IRMO" ("REFERRAL_TYPE", 1) 
  TABLESPACE "ACM_INDX" ;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."BI_F_IRMO_DG" before insert on T_F_IRMO for each row declare  begin  if :NEW.data_group_id is null then :NEW.data_group_id := sync_util.dg_from_instance; end if;  end;

/
ALTER TRIGGER "ACM"."BI_F_IRMO_DG" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."F_IRMO_A_IUD" AFTER
    DELETE OR INSERT OR UPDATE ON T_F_IRMO
    FOR EACH ROW
BEGIN
    IF INSERTING THEN
      -- Getting Acquisition Date for the Due Dates
      ACM_BPE.SET_PARAM('ACQUISITION_DATE',:NEW.ACQUISITION_DATE);
    END IF;
END;


/
ALTER TRIGGER "ACM"."F_IRMO_A_IUD" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."BD_F_IRMO_AUD" before delete on T_F_IRMO for each row begin if sync_util.dg_from_instance=1 then insert into t_sync_del_rows(table_name,del_id)values('T_F_IRMO',:OLD.SID); end if; end;

/
ALTER TRIGGER "ACM"."BD_F_IRMO_AUD" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."F_IRMO_B_IUD" BEFORE
    INSERT OR UPDATE OR DELETE ON T_F_IRMO
    FOR EACH ROW
DECLARE
    V_OBJ T_ACM_OBJ.SID%TYPE;
BEGIN
    IF INSERTING AND :NEW.SID IS NULL THEN
        SET_NEW_FILE_ID(:NEW.ID);
        INSERT INTO T_ACM_OBJ(
            OBJ_TYPE,
            ID,
            TAGLINE
        )VALUES(
            COALESCE(
                ACM_OBJ.LOOKUP_OBJTYPE(
                    APEX_UTIL.GET_SESSION_STATE(
                        'P' || APEX_APPLICATION.G_FLOW_STEP_ID || '_OBJ_TYPE_CODE'
                    )
                ),
                TO_NUMBER(
                    ACM_BPE.GET_PARAM(
                        'OBJ_TYPE'
                    )
                )
            ),
            :NEW.ID,
            :NEW.TITLE
        )RETURNING SID INTO V_OBJ;
        :NEW.OBJ := V_OBJ;
        SET_TABLE_ROW_VALS(:NEW.CREATED_ON,:NEW.CREATED_BY,:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY,:NEW.SID);
    END IF;
    IF UPDATING THEN
        SET_TABLE_ROW_MOD_VALS(:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY);
    END IF;
END;


/
ALTER TRIGGER "ACM"."F_IRMO_B_IUD" ENABLE;


  GRANT SELECT ON "ACM"."T_F_IRMO" TO "ADV_SEARCH";





--T_F_IRMO_FINDING_REF create

CREATE TABLE "ACM"."T_F_IRMO_FINDING_REF" 
   (	"REF_KEY" VARCHAR2(108 BYTE) NOT NULL ENABLE, 
	"USAGE" VARCHAR2(40 BYTE), 
	"CODE" VARCHAR2(20 BYTE), 
	"DISPLAY" VARCHAR2(4000 BYTE), 
	"SEQ" NUMBER, 
	"ACTIVE" VARCHAR2(1 BYTE) DEFAULT 'Y', 
	"DEV_COMMENT" VARCHAR2(4000 BYTE), 
	"OBJ_TYPE" NUMBER, 
	"CREATED_BY" VARCHAR2(100 BYTE), 
	"CREATED_ON" DATE, 
	"LAST_MODIFIED_BY" VARCHAR2(100 BYTE), 
	"LAST_MODIFIED_ON" DATE, 
	 CONSTRAINT "NN_FIRMO_FINDINGREF_DISPLAY" CHECK (DISPLAY IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_FINDINGREF_ACTIVE" CHECK (ACTIVE IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_FINDINGREF_REFKEY" CHECK (REF_KEY IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_FINDINGREF_CODE" CHECK (CODE IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_FINDINGREF_USAGE" CHECK (USAGE IS NOT NULL) ENABLE
   )
  TABLESPACE "ACM_DATA" ;
  CREATE UNIQUE INDEX "ACM"."PK_FIRMO_FINDINGREF" ON "ACM"."T_F_IRMO_FINDING_REF" ("REF_KEY") 
  TABLESPACE "ACM_INDX" ;
ALTER TABLE "ACM"."T_F_IRMO_FINDING_REF" ADD CONSTRAINT "PK_FIRMO_FINDINGREF" PRIMARY KEY ("REF_KEY")
  USING INDEX "ACM"."PK_FIRMO_FINDINGREF"  ENABLE;

  CREATE INDEX "ACM"."I_FIRMO_FINDINGREF_CODE" ON "ACM"."T_F_IRMO_FINDING_REF" ("CODE") 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_FIRMO_FINDINGREF_USAGE" ON "ACM"."T_F_IRMO_FINDING_REF" ("USAGE") 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_F_IRMO_FINDING_REF_OBJTYPE" ON "ACM"."T_F_IRMO_FINDING_REF" ("OBJ_TYPE") 
  TABLESPACE "ACM_INDX" ;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."F_IRMO_FINDING_REF_B_IUD" BEFORE
    INSERT OR UPDATE OR DELETE ON T_F_IRMO_FINDING_REF
    FOR EACH ROW
BEGIN
    IF INSERTING THEN
        SET_TABLE_ROW_VALS_NO_SID(:NEW.CREATED_ON,:NEW.CREATED_BY,:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY);
    END IF;
    IF UPDATING THEN
        SET_TABLE_ROW_MOD_VALS(:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY);
    END IF;
END;


/
ALTER TRIGGER "ACM"."F_IRMO_FINDING_REF_B_IUD" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."BD_F_IRMO_FINDING_REF_AUD" before delete on T_F_IRMO_FINDING_REF for each row begin if sync_util.dg_from_instance=1 then insert into t_sync_del_rows(table_name,del_id)values('T_F_IRMO_FINDING_REF',:OLD.REF_KEY); end if; end;

/
ALTER TRIGGER "ACM"."BD_F_IRMO_FINDING_REF_AUD" ENABLE;


  GRANT SELECT ON "ACM"."T_F_IRMO_FINDING_REF" TO "ADV_SEARCH";


--T_F_IRMO_FINDING create



  CREATE TABLE "ACM"."T_F_IRMO_FINDING" 
   (	"SID" NUMBER NOT NULL ENABLE, 
	"OBJ" VARCHAR2(20 BYTE), 
	"RECOMMENDATION" VARCHAR2(108 BYTE), 
	"COMMENTS" VARCHAR2(4000 BYTE), 
	"CREATED_BY" VARCHAR2(100 BYTE), 
	"CREATED_ON" DATE, 
	"LAST_MODIFIED_BY" VARCHAR2(100 BYTE), 
	"LAST_MODIFIED_ON" DATE, 
	"FINDING_LEVEL" VARCHAR2(108 BYTE),
	"IRMO_INDICATOR" VARCHAR2(108 BYTE), 
	"DATA_GROUP_ID" NUMBER DEFAULT NULL NOT NULL ENABLE, 
	 CONSTRAINT "NN_FIRMO_FINDING_OBJ" CHECK (OBJ IS NOT NULL) ENABLE, 
	 CONSTRAINT "NN_FIRMO_FINDING_SID" CHECK (SID IS NOT NULL) ENABLE,  
	 CONSTRAINT "FK_IRMO_FINDING_IINTOR" FOREIGN KEY ("IRMO_INDICATOR")
	  REFERENCES "ACM"."T_F_ALLEG_TYPE" ("REF_KEY") ENABLE, 
	 CONSTRAINT "FK_FIRMO_FINDING_OBJ" FOREIGN KEY ("OBJ")
	  REFERENCES "ACM"."T_ACM_OBJ" ("SID") ON DELETE CASCADE ENABLE, 
	 CONSTRAINT "FK_FIRMO_FINDING_RECOMMEND" FOREIGN KEY ("RECOMMENDATION")
	  REFERENCES "ACM"."T_F_IRMO_FINDING_REF" ("REF_KEY") ENABLE
   )
  TABLESPACE "ACM_DATA" ;
  CREATE UNIQUE INDEX "ACM"."T_F_IRMO_FINDING_PK" ON "ACM"."T_F_IRMO_FINDING" ("SID") 
  TABLESPACE "ACM_DATA" ;
ALTER TABLE "ACM"."T_F_IRMO_FINDING" ADD CONSTRAINT "PK_FIRMO_FINDING" PRIMARY KEY ("SID")
  USING INDEX "ACM"."T_F_IRMO_FINDING_PK"  ENABLE;

  CREATE INDEX "ACM"."I_F_IRMO_FINDING_IINDICATOR" ON "ACM"."T_F_IRMO_FINDING" ("IRMO_INDICATOR", 1) 
  TABLESPACE "ACM_INDX" ;

  CREATE INDEX "ACM"."I_F_IRMO_FINDING_RECOMMENDATION" ON "ACM"."T_F_IRMO_FINDING" ("RECOMMENDATION", 1) 
  TABLESPACE "ACM_INDX" ;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."F_IRMO_FINDING_B_IUD" BEFORE
    INSERT OR UPDATE OR DELETE ON T_F_IRMO_FINDING
    FOR EACH ROW
BEGIN
    IF INSERTING THEN
        SET_TABLE_ROW_VALS(:NEW.CREATED_ON,:NEW.CREATED_BY,:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY,:NEW.SID);
    END IF;
    IF UPDATING THEN
        SET_TABLE_ROW_MOD_VALS(:NEW.LAST_MODIFIED_ON,:NEW.LAST_MODIFIED_BY);
    END IF;
END;


/
ALTER TRIGGER "ACM"."F_IRMO_FINDING_B_IUD" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."BD_F_IRMO_FINDING_AUD" before delete on T_F_IRMO_FINDING for each row begin if sync_util.dg_from_instance=1 then insert into t_sync_del_rows(table_name,del_id)values('T_F_IRMO_FINDING',:OLD.SID); end if; end;

/
ALTER TRIGGER "ACM"."BD_F_IRMO_FINDING_AUD" ENABLE;

  CREATE OR REPLACE EDITIONABLE TRIGGER "ACM"."BI_F_IRMO_FINDING_DG" before insert on T_F_IRMO_FINDING for each row declare  begin  if :NEW.data_group_id is null then :NEW.data_group_id := sync_util.dg_from_instance; end if;  end;

/
ALTER TRIGGER "ACM"."BI_F_IRMO_FINDING_DG" ENABLE;


  GRANT SELECT ON "ACM"."T_F_IRMO_FINDING" TO "ADV_SEARCH";
  
  
  -- Finding plug-in copy and rework


create or replace PACKAGE ACM_P_IRMO_FINDINGS as
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
FUNCTION I2MS_FINDINGS_REGION_RENDER (
        P_REGION IN APEX_PLUGIN.T_REGION,
        P_PLUGIN IN APEX_PLUGIN.T_PLUGIN,
        P_IS_PRINTER_FRIENDLY IN BOOLEAN
    ) RETURN APEX_PLUGIN.T_REGION_RENDER_RESULT;
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
    FUNCTION I2MS_FINDINGS_REGION_AJAX (
        P_REGION IN APEX_PLUGIN.T_REGION,
        P_PLUGIN IN APEX_PLUGIN.T_PLUGIN
    ) RETURN APEX_PLUGIN.T_REGION_AJAX_RESULT;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_INITIAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_FINAL_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_HQ_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_FIELD_REC RETURN VARCHAR2;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_REGIONAL_REC RETURN VARCHAR2;
end ACM_P_IRMO_FINDINGS;
/

---------------BODY

create or replace PACKAGE BODY     ACM_P_IRMO_FINDINGS AS
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
    PROCEDURE PRINT_REPORT (
        P_OBJ IN VARCHAR2, P_REGION IN APEX_PLUGIN.T_REGION, P_PLUGIN IN APEX_PLUGIN.T_PLUGIN
    ) IS
        L_RSLT_INDX NUMBER := 0;
        L_EDIT_AUTH VARCHAR2(10) := 'N';
    BEGIN
        L_EDIT_AUTH := ACM_BPE.CHECK_ACTION('EDIT_FINDING', P_OBJ);
        HTP.DIV(CATTRIBUTES => 'id="report_' || P_REGION.ID || '_catch"');
        FOR FINDING IN (
            SELECT TFF.SID, REC.DISPLAY RECOMMENDATION_DISPLAY, RT.DISPLAY REC_TYPE, CI.DISPLAY IRMO_INDICATOR,
            TFF.COMMENTS, TFF.CREATED_BY, TFF.CREATED_ON, TFF.LAST_MODIFIED_BY, TFF.LAST_MODIFIED_ON
            FROM T_F_IRMO_FINDING TFF
            LEFT OUTER JOIN T_F_IRMO_FINDING_REF REC ON TFF.RECOMMENDATION = REC.REF_KEY
            LEFT OUTER JOIN T_F_IRMO_FINDING_REF RT ON TFF.TYPE = RT.REF_KEY
            LEFT OUTER JOIN T_F_IRMO_REF CI ON TFF.IRMO_INDICATOR = CI.REF_KEY
            WHERE TFF.OBJ = P_OBJ
            ORDER BY TFF.CREATED_ON DESC
        ) LOOP
            L_RSLT_INDX := L_RSLT_INDX + 1;
            IF L_RSLT_INDX = 1 THEN
                ACM_REGION_UTIL.START_TABLE(P_REGION.ID, 'Recommendations"');
                IF L_EDIT_AUTH = 'Y' THEN
                    ACM_REGION_UTIL.THC('Edit', 'SID');
                ELSE
                    ACM_REGION_UTIL.THC('View', 'SID');
                END IF;
                ACM_REGION_UTIL.THC('Recommendation', 'RECOMMENDATION_DISPLAY');
                ACM_REGION_UTIL.THC('Type', 'REC_TYPE');
                ACM_REGION_UTIL.THC('Comments', 'COMMENTS');
                ACM_REGION_UTIL.THC('Created On', 'CREATED_ON');
                HTP.TABLEROWCLOSE;
                HTP.P('</thead>');
                HTP.P('<tbody>');
            END IF;
            HTP.TABLEROWOPEN();
            ACM_REGION_UTIL.TD('<a href="' || PREPARE_URL('5016', '5016', 'P5016_SID,P5016_OBJ', FINDING.SID || ',' || P_OBJ,P_REGION.STATIC_ID) || '">' ||
                CASE
                    WHEN L_EDIT_AUTH = 'Y' THEN
                        ACM_REGION_UTIL.EDIT_IMAGE('Edit Indicator')
                    ELSE ACM_REGION_UTIL.VIEW_IMAGE('View Indicator')
                END
                               || '</a>', 'SID');
            ACM_REGION_UTIL.TD(FINDING.RECOMMENDATION_DISPLAY, 'RECOMMENDATION_DISPLAY');
            ACM_REGION_UTIL.TD(FINDING.REC_TYPE, 'REC_TYPE'); 
            ACM_REGION_UTIL.TD(FINDING.COMMENTS, 'COMMENTS');
            ACM_REGION_UTIL.TD(TO_CHAR(FINDING.CREATED_ON, FMT_DATE), 'CREATED_ON');
            HTP.TABLEROWCLOSE;
        END LOOP;
        IF L_RSLT_INDX > 0 THEN
            HTP.P('</tbody>');
            HTP.TABLECLOSE;
            HTP.P('</div>');
            HTP.P('</div>');
            HTP.P('</div>');
        ELSE
            ACM_REGION_UTIL.NDF('No Recommendations');
        END IF;
        HTP.P('</div>');
    END;
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
    FUNCTION I2MS_FINDINGS_REGION_RENDER (
        P_REGION IN APEX_PLUGIN.T_REGION, P_PLUGIN IN APEX_PLUGIN.T_PLUGIN, P_IS_PRINTER_FRIENDLY IN BOOLEAN
    ) RETURN APEX_PLUGIN.T_REGION_RENDER_RESULT IS
        L_RTN APEX_PLUGIN.T_REGION_RENDER_RESULT;
        L_OBJ T_ACM_OBJ.SID%TYPE;
   --P_REGION.ATTRIBUTE_01 PAGE ITEM OBJ, NECESSARY FOR GETTING VALUES FOR THE REPORT
    BEGIN
        L_OBJ := APEX_UTIL.GET_SESSION_STATE(P_REGION.ATTRIBUTE_01);
        IF ACM_BPE.CHECK_ACTION('ADD_FINDING', L_OBJ) = 'Y' THEN
            ACM_REGION_UTIL.PRINT_BUTTON(PREPARE_URL('5016', '5016', 'P5016_OBJ', L_OBJ), 'B' || P_REGION.ID || '-add-finding', 'add-finding',
            'Add Recommendation');
        END IF;
        PRINT_REPORT(L_OBJ, P_REGION, P_PLUGIN);
        ACM_REGION_UTIL.ADD_ONLOAD_JS(P_REGION, P_REGION.ATTRIBUTE_01);
        RETURN L_RTN;
    END I2MS_FINDINGS_REGION_RENDER;
/** Summary
<table>
<tr><td>12/16/2019</td><td>LLSB</td><td>Created</td></tr>
</table>
*/
    FUNCTION I2MS_FINDINGS_REGION_AJAX (
        P_REGION IN APEX_PLUGIN.T_REGION, P_PLUGIN IN APEX_PLUGIN.T_PLUGIN
    ) RETURN APEX_PLUGIN.T_REGION_AJAX_RESULT IS
        L_RTN APEX_PLUGIN.T_REGION_AJAX_RESULT;
        L_OBJ T_ACM_OBJ.SID%TYPE;
    BEGIN
        L_OBJ := APEX_UTIL.GET_SESSION_STATE(P_REGION.ATTRIBUTE_01);
        PRINT_REPORT(L_OBJ, P_REGION, P_PLUGIN);
        RETURN L_RTN;
    END I2MS_FINDINGS_REGION_AJAX;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_INITIAL_REC RETURN VARCHAR2 IS
        L_RTN VARCHAR2(400) := 'File must have an Initial Recommendation';
    BEGIN
        FOR RSLT IN (
            SELECT TFF.SID
            FROM T_F_IRMO_FINDING TFF
            INNER JOIN T_F_IRMO_FINDING_REF TFFT ON TFFT.REF_KEY = TFF.TYPE
            WHERE TFFT.REF_KEY IN (
                'FILE.IRMO.RECOMMEND_TYPE.INITIAL'
            ) AND TFF.OBJ = ACM_BPE.GET_PARAM('OBJ')
        ) LOOP
            L_RTN := 'Y';
        END LOOP;
        RETURN L_RTN;
    END COND_HAS_INITIAL_REC;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_FINAL_REC RETURN VARCHAR2 IS
        L_RTN VARCHAR2(400) := 'File must have an Final Recommendation';
    BEGIN
        FOR RSLT IN (
            SELECT TFF.SID
            FROM T_F_IRMO_FINDING TFF
            INNER JOIN T_F_IRMO_FINDING_REF TFFT ON TFFT.REF_KEY = TFF.TYPE
            WHERE TFFT.REF_KEY IN (
                'FILE.IRMO.RECOMMEND_TYPE.FINAL'
            ) AND TFF.OBJ = ACM_BPE.GET_PARAM('OBJ')
              AND TFF.RECOMMENDATION IS NOT NULL
        ) LOOP
            L_RTN := 'Y';
        END LOOP;
        RETURN L_RTN;
    END COND_HAS_FINAL_REC;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_HQ_REC RETURN VARCHAR2 IS
        L_RTN VARCHAR2(400) := 'File must have a HQ Recommendation';
    BEGIN
        FOR RSLT IN (
            SELECT TFF.SID
            FROM T_F_IRMO_FINDING TFF
            INNER JOIN T_F_IRMO_FINDING_REF TFFT ON TFFT.REF_KEY = TFF.TYPE
            WHERE TFFT.REF_KEY = 'FILE.IRMO.RECOMMEND_TYPE.HQ' AND TFF.OBJ = ACM_BPE.GET_PARAM('OBJ')
        ) LOOP
            L_RTN := 'Y';
        END LOOP;
        RETURN L_RTN;
    END COND_HAS_HQ_REC;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_FIELD_REC RETURN VARCHAR2 IS
        L_RTN VARCHAR2(400) := 'File must have a field Recommendation';
    BEGIN
        FOR RSLT IN (
            SELECT TFF.SID
            FROM T_F_IRMO_FINDING TFF
            INNER JOIN T_F_IRMO_FINDING_REF TFFT ON TFFT.REF_KEY = TFF.TYPE
            WHERE TFFT.REF_KEY = 'FILE.IRMO.RECOMMEND_TYPE.FIELD' AND TFF.OBJ = ACM_BPE.GET_PARAM('OBJ')
        ) LOOP
            L_RTN := 'Y';
        END LOOP;
        RETURN L_RTN;
    END COND_HAS_FIELD_REC;
/** Summary
<table>
<tr><td>1/28/2020</td><td>DL</td><td>Created</td></tr>
</table>
*/
    FUNCTION COND_HAS_REGIONAL_REC RETURN VARCHAR2 IS
        L_RTN VARCHAR2(400) := 'File must have a Regional Recommendation';
    BEGIN
        FOR RSLT IN (
            SELECT TFF.SID
            FROM T_F_IRMO_FINDING TFF
            INNER JOIN T_F_IRMO_FINDING_REF TFFT ON TFFT.REF_KEY = TFF.TYPE
            WHERE TFFT.REF_KEY = 'FILE.IRMO.RECOMMEND_TYPE.REGNL' AND TFF.OBJ = ACM_BPE.GET_PARAM('OBJ')
        ) LOOP
            L_RTN := 'Y';
        END LOOP;
        RETURN L_RTN;
    END COND_HAS_REGIONAL_REC;
END ACM_P_IRMO_FINDINGS;
/


